-- *** Demo missions ***
-- Since: v0.4.2
-- Description: Demo missions of MISI Library

local STAGEC = require("modules.STAGEC")
local Game = require("modules.game")
local Ped = require("modules.ped")
local Car = require("modules.car")
local Obj = require("modules.obj")
local System = require("modules.system")

os.setlocale("en", "numeric") -- Set float style 1,0 to 1.0

local gamestate = 0
local missionstate = 0
local current_missionid = 1
local total_mission_stat = {}
local validstage = 0
local player = 0
local ped_pool = {}
local mission_ped_pool = {}
local mission_car_pool = {}
local mission_obj_pool = {}
local mission_arrow_pool = {}
local mission_timer_pool = {}
local car = 0
local parkedcar_pool = {} --{{struct = 0, time_tmp = 0, time_respawn = 0}}
local phone_pool = {} --{{struct = 0, arrow = 0, state = 0, missionid = 0}}
local door_pool = {}
local pickup_pool = {} --{{struct = 0, time_tmp = 0, time_respawn = 0}}
local x, y, z, a = 0
local walk_speed = 0.8
local sprint_speed = 6
local make_ped_lock = 0
local make_car_lock = 0
local make_obj_lock = 0
local make_arrow_lock = 0
local make_block_lock = 0
local total_remove_buildings = 0
local total_buildings = 0
local gametime_tmp = 0
-- 30 cycles = 30 frames = 1 sec
local game_minute_ms = 1000 -- 24 min = 24 ghours
local game_h = 15
local game_m = 0

local stagelight = {
	[0] = 0.2,
	[1] = 0.1,
	[2] = 0.1,
	[3] = 0.2,
	[4] = 0.2,
	[5] = 0.3,
	[6] = 0.4,
	[7] = 0.5,
	[8] = 0.6,
	[9] = 0.7,
	[10] = 0.8,
	[11] = 0.9,
	[12] = 1.0,
	[13] = 1.0,
	[14] = 0.9,
	[15] = 0.9,
	[16] = 0.8,
	[17] = 0.8,
	[18] = 0.7,
	[19] = 0.6,
	[20] = 0.5,
	[21] = 0.4,
	[22] = 0.4,
	[23] = 0.3
}

local ped_density = {
	[0] = 4,
	[1] = 3,
	[2] = 1,
	[3] = 1,
	[4] = 2,
	[5] = 3,
	[6] = 6,
	[7] = 8,
	[8] = 10,
	[9] = 11,
	[10] = 12,
	[11] = 18,
	[12] = 25,
	[13] = 22,
	[14] = 21,
	[15] = 19,
	[16] = 20,
	[17] = 21,
	[18] = 23,
	[19] = 20,
	[20] = 15,
	[21] = 12,
	[22] = 10,
	[23] = 8
}

local STATIC_PLAYER_SPAWNS = {
	{x = 93.0, y = 90.0, z = 2.0, a = 0, model = 1, skin = 25},
	{x = 93.0, y = 91.0, z = 2.0, a = 0, model = 1, skin = 26},
	{x = 93.0, y = 92.0, z = 2.0, a = 0, model = 1, skin = 27},
	{x = 93.0, y = 93.0, z = 2.0, a = 0, model = 1, skin = 28},
	{x = 93.0, y = 94.0, z = 2.0, a = 0, model = 1, skin = 29},
	{x = 93.0, y = 95.0, z = 2.0, a = 0, model = 1, skin = 30}
}

local STATIC_QUTFITS = {
	[0] = {model = 1, skin = 25, occupation = 0}, -- Player
	[1] = {model = 2, skin = 0, occupation = 24}, -- Cop
	[2] = {model = 0, skin = 16, occupation = 23}, -- Medic
	[3] = {model = 0, skin = 19, occupation = 10} -- Taxi
}

local STATIC_MISSION_PEDS = {
	{x = 225.5, y = 5.3, z = 2.0, a = 180, model = 2, skin = 8, weapon = 1, reaction = 0, occupation = 32},
	{x = 223.5, y = 3.6, z = 2.0, a = 240, model = 2, skin = 9, weapon = 1, reaction = 0, occupation = 32},
	{x = 221.5, y = 6.2, z = 2.0, a = 320, model = 2, skin = 10, weapon = 0, reaction = 3, occupation = 34},
	{x = 223.5, y = 8.5, z = 2.0, a = 60, model = 2, skin = 8, weapon = 1, reaction = 0, occupation = 32},
	{x = 222.8, y = 7.3, z = 2.0, a = 128, model = 2, skin = 8, weapon = 1, reaction = 0, occupation = 32},
	{x = 225.8, y = 3.3, z = 2.0, a = 310, model = 2, skin = 9, weapon = 1, reaction = 0, occupation = 32},
	{x = 222.2, y = 6.5, z = 2.0, a = 140, model = 2, skin = 9, weapon = 1, reaction = 0, occupation = 32},
	{x = 221.4, y = 4.8, z = 2.0, a = 79, model = 2, skin = 7, weapon = 1, reaction = 3, occupation = 32},
	{x = 223.8, y = 5.0, z = 2.0, a = 289, model = 2, skin = 2, weapon = 0, reaction = 3, occupation = 34}
}

local STATIC_MISSION_CARS = {
	{x = 225.2, y = 3.8, z = 2.0, a = 228, model = 32, color = 1, doorlock = 1, alarm = 128, armor = 1},
	{x = 222.1, y = 4.2, z = 2.0, a = 324, model = 32, color = 2, doorlock = 0, alarm = 128, armor = 0},
	{x = 226.0, y = 6.1, z = 2.0, a = 138, model = 2, color = 1, doorlock = 1, alarm = 128, armor = 1}
}

local STATIC_DOORS = {
	{x = 92.0, y = 83.0, z = 2.0, face_type = 4, wall_type = 1, bullet_type = 1, flat_type = 0, flip_type = 0, rotation = 0, tileid = 813}, -- Medic doors
	{x = 88.0, y = 86.0, z = 2.0, face_type = 4, wall_type = 1, bullet_type = 0, flat_type = 1, flip_type = 1, rotation = 0, tileid = 197},
	{x = 89.0, y = 86.0, z = 2.0, face_type = 4, wall_type = 1, bullet_type = 0, flat_type = 1, flip_type = 1, rotation = 0, tileid = 197}
}

local STATIC_PHONES = {
	{x = 95.7, y = 92.4, z = 2.0, a = 0.0, model = 163}, -- TNT job
	{x = 91.4, y = 84.3, z = 2.0, a = 90.0, model = 163}, -- Medic job
	{x = 75.8, y = 84.3, z = 2.0, a = 90.0, model = 163} -- Cop job
}

local STATIC_PARKED_CARS = {
	{x = 73.5, y = 83.4, z = 2.0, a = 180.0, model = 12, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000}, -- Cop cars
	{x = 74.4, y = 83.5, z = 2.0, a = 176.0, model = 12, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000},
	{x = 71.5, y = 84.7, z = 2.0, a = 1.0, model = 58, trailer = 65535, color = 65535, doorlock = 1, alarm = 0, time_respawn = 10000},
	{x = 88.6, y = 83.0, z = 2.0, a = 359.0, model = 34, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000}, -- Medic cars
	{x = 87.7, y = 83.0, z = 2.0, a = 361.0, model = 34, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000},
	{x = 90.4, y = 93.4, z = 2.0, a = 180.0, model = 56, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000}, -- Taxi cars
	{x = 90.4, y = 91.5, z = 2.0, a = 181.0, model = 51, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 10000},
	{x = 171.0, y = 206.4, z = 3.0, a = 180.0, model = 54, trailer = 65535, color = 65535, doorlock = 0, alarm = 0, time_respawn = 120000}
}

local STATIC_PICKUPS = {
	{x = 78.5, y = 84.3, z = 2.0, a = 0.0, model = 201, value = 30, time_respawn = 10000}, -- Pistol
	--{x = 77.7, y = 84.3, z = 2.0, a = 0.0, model = 231, value = 10, time_respawn = 10000}, -- Armor
	{x = 93.5, y = 85.5, z = 2.0, a = 0.0, model = 230, value = 100, time_respawn = 10000}, -- Health
	{x = 76.5, y = 82.5, z = 5.0, a = 0.0, model = 233, value = 1, time_respawn = 10000} -- Cop Bribe
}

local STATIC_REMOVE_BUILDINGS = {
	{67.5, 84.5, 4.0},
	{77.5, 84.5, 5.0},
	{75.5, 76.5, 4.0},
	{74.5, 76.5, 4.0},
	{65.5, 81.5, 4.0}
}

local STATIC_NEW_GROUND = {
	-- id, flat_type, flip_type, light, rotation, tileid, id2, field_type, slope_type
	{178,7,1, 5, 0, 1, 0, 0, 64, 6, 3, 0},
	{176,8,1, 5, 0, 1, 0, 0, 64, 6, 3, 0},
	{177,8,1, 5, 0, 1, 0, 0, 64, 6, 3, 0},
	{176,9,1, 5, 0, 1, 0, 0, 64, 6, 3, 0},
	{177,9,1, 5, 0, 1, 0, 0, 64, 6, 3, 0}
}

local STATIC_NEW_GROUND2 = {
	-- id, flat_type, flip_type, light, rotation, tileid, id2, field_type, slope_type
	{178,7,1, 5, 0, 1, 0, 0, 64, 6, 3, 0},
	{176,8,1, 5, 0, 0, 0, 0, 64, 6, 3, 0},
	{177,8,1, 5, 0, 0, 0, 0, 64, 6, 3, 0},
	{170,9,1, 5, 0, 1, 0, 270, 262, 6, 3, 0},
	{171,9,1, 5, 0, 1, 0, 180, 262, 6, 3, 0},	
	{176,9,1, 5, 0, 0, 0, 0, 64, 6, 3, 0},
	{177,9,1, 5, 0, 0, 0, 0, 64, 6, 3, 0},		
	{170,10,1, 5, 0, 1, 0, 90, 279, 6, 3, 0},
	{171,10,1, 5, 0, 0, 0, 90, 268, 6, 3, 0},
	{172,10,1, 5, 0, 0, 0, 0, 271, 6, 3, 0},
	{177,10,1, 5, 0, 0, 0, 270, 263, 6, 3, 0},
	{170,11,1, 5, 0, 1, 0, 90, 275, 6, 3, 0},
	{171,11,1, 5, 0, 0, 0, 90, 279, 6, 3, 0},	
	{177,11,1, 5, 0, 0, 0, 90, 271, 6, 3, 0},	
	{170,12,1, 5, 0, 0, 0, 270, 273, 6, 3, 0},
	{171,12,1, 5, 0, 0, 0, 90, 275, 6, 3, 0},
	{177,12,1, 5, 0, 0, 0, 90, 271, 6, 3, 0},	
	{176,13,1, 5, 0, 1, 0, 0, 279, 6, 3, 0},
	{177,13,1, 5, 0, 0, 0, 0, 268, 6, 3, 0},
	{178,13,1, 5, 0, 0, 0, 270, 262, 6, 3, 0}
}

local STATIC_BUILDINGS = {
	{77.5, 84.5, 4.0, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, 3, 0, 0, 1, 0, 0, 774, 4, 0, 0, 0, 0, 0, 774, 0, 0, 0, 0, 0, 0, 0, 0, 0} -- PD logo
}

local STATIC_NEW_PAINTSHOP = {
	{2, 4, 2, 1, 0, 0, 1, 0, 0, 757, 2, 0, 0, 0, 0, 0, 757, -3, 0, 0, 1, 0, 0, 757, -4, 0, 0, 0, 0, 0, 757, -5, 1, 1, 0, 270, 757, 6, 0, 63}, -- Lamp
	{2, 4, 2, 1, 0, 0, 1, 0, 0, 757, 2, 0, 0, 0, 0, 0, 757, -3, 0, 0, 1, 0, 0, 757, -4, 0, 0, 0, 0, 0, 757, 5, 1, 1, 0, 270, 750, 6, 0, 8}, -- lamp
	{3, 3, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 270, 482, 6, 3, 0}, -- 1
	{4, 3, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 487, 6, 3, 0}, -- 2
	{5, 3, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 487, 6, 3, 0}, -- 3
	{6, 3, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 90, 482, 6, 3, 0}, -- 4
	{4, 4, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 152, 6, 0, 0}, -- 5
	{5, 4, 1, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 152, 6, 0, 0}, -- 6
	{4, 1, 2, 1, 1, 1, 0, 0, 0, 404, -2, 0, 0, 0, 0, 0, 404, 3, 1, 1, 0, 0, 0, 404, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 398, 6, 0, 0}, -- 1
	{5, 1, 2, -1, 0, 0, 1, 0, 0, 774, 2, 1, 1, 0, 0, 0, 410, 3, 1, 1, 0, 0, 0, 404, -4, 0, 0, 0, 0, 0, 774, 5, 0, 0, 0, 0, 398, 6, 0, 0}, -- 2
	{6, 1, 2, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 1, 0, 0, 340, 3, 0, 0, 1, 0, 0, 340, 4, 0, 0, 0, 0, 0, 340, 5, 1, 0, 0, 0, 340, 6, 0, 0}, -- 3
	{7, 1, 2, 1, 0, 0, 0, 0, 0, 340, 2, 0, 0, 1, 0, 0, 340, 3, 0, 0, 1, 0, 0, 340, 4, 0, 0, 0, 0, 0, 340, 5, 1, 0, 0, 0, 340, 6, 0, 0}, -- 4
	{4, 2, 2, 1, 1, 1, 0, 0, 0, 410, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, 4, 1, 1, 0, 0, 0, 410, 5, 0, 1, 0, 180, 398, 6, 0, 0}, -- 5
	{5, 2, 2, -1, 0, 0, 0, 0, 0, 774, 2, 1, 1, 1, 0, 0, 410, -3, 0, 0, 1, 0, 0, 774, 4, 1, 1, 0, 0, 0, 407, 5, 0, 1, 0, 180, 398, 6, 0, 0}, -- 6
	{6, 2, 2, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 1, 0, 0, 340, 3, 0, 0, 0, 0, 0, 340, 4, 0, 0, 1, 0, 0, 340, 5, 1, 1, 0, 180, 340, 6, 0, 0}, -- 7
	{7, 2, 2, 1, 0, 0, 0, 0, 0, 340, 2, 0, 0, 1, 0, 0, 340, 3, 0, 0, 0, 0, 0, 340, 4, 0, 0, 1, 0, 0, 377, 5, 1, 1, 0, 180, 340, 6, 0, 0}, -- 8
	{8, 2, 2, 1, 0, 0, 1, 0, 270, 340, 2, 0, 0, 1, 0, 90, 340, 3, 0, 0, 1, 0, 90, 340, 4, 0, 0, 1, 0, 270, 340, 5, 1, 0, 0, 270, 340, 6, 0, 228}, -- 9
	{4, 3, 2, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 1, 0, 0, 0, 94, 6, 0, 0}, -- 10
	{5, 3, 2, -1, 0, 0, 1, 0, 0, 774, -2, 0, 0, 0, 0, 0, 774, -3, 0, 0, 1, 0, 0, 774, -4, 0, 0, 0, 0, 0, 774, 5, 1, 1, 0, 0, 94, 6, 0, 0}, -- 11
	{4, 4, 2, 1, 1, 1, 0, 0, 0, 404, -2, 0, 0, 0, 0, 0, 774, 3, 1, 1, 0, 0, 0, 409, 4, 1, 1, 0, 0, 0, 404, 5, 1, 0, 0, 0, 87, 6, 0, 0}, -- 12
	{5, 4, 2, -1, 0, 0, 1, 0, 0, 774, 2, 1, 1, 0, 0, 0, 404, 3, 1, 1, 0, 0, 0, 400, 4, 1, 1, 0, 0, 0, 404, 5, 1, 0, 0, 0, 87, 6, 0, 0}, -- 13	
	{5, 2, 3, 1, 1, 1, 0, 0, 0, 632, 2, 1, 1, 0, 0, 0, 632, 3, 1, 1, 0, 0, 0, 632, 4, 1, 1, 0, 0, 0, 632, 5, 0, 0, 0, 0, 632, 6, 0, 80} -- Baner	
}

local STATIC_PAINTSHOP_TNT = {
	{x = 24.5, y = 60.5, z = 3},
	{x = 219.5, y = 34.5, z = 2},
	{x = 209.5, y = 121.5, z = 2},
	{x = 46.5, y = 136.5, z = 2},
	{x = 204.5, y = 221.5, z = 2},
	{x = 86.5, y = 160.5, z = 2}
}

local STATIC_PAINTSHOP_MEDIC = {
	{x = 23.5, y = 57.5, z = 3},  
	{x = 215.5, y = 33.5, z = 2},
	{x = 208.5, y = 116.5, z = 2},
	{x = 46.5, y = 136.5, z = 2},
	{x = 203.5, y = 216.5, z = 2},
	{x = 85.5, y = 155.5, z = 2}
}

local STATIC_MISSION_PED_MEDIC = {
	{x = 72.6, y = 135.9, z = 2.0, a = 176, model = 0, skin = 16, weapon = 0, reaction = 0, occupation = 0},
	{x = 72.8, y = 136.0, z = 2.0, a = 175, model = 0, skin = 16, weapon = 0, reaction = 0, occupation = 0},
	{x = 73.0, y = 135.8, z = 2.0, a = 177, model = 0, skin = 16, weapon = 0, reaction = 0, occupation = 0}
}

local mission_pool = {
	{state = 0, time_tmp = {}, target = {},
		mission = function(ped, target, phoneid)
			if (target.name == "INIT") then
				if (phone_pool[phoneid].state == 1) then
					return false, target
				end			
				phone_pool[phoneid].icon = 3
				phone_pool[phoneid].color = 4
				phone_pool[phoneid].state = 1
				phone_pool[phoneid].missionid = 1
				total_mission_stat[1] = 1
				return false, target
			end
			if (target.name == "START") then -- 2
				STAGEC.RemoveAllDialogs()
				STAGEC.ShowDialog(1091, 1)
				STAGEC.ShowDialog(1092, 2)
				STAGEC.ShowDialog(1093, 2)				
				STAGEC.DeleteCar(parkedcar_pool[3].struct, 1)	
				parkedcar_pool[3].struct = 0
				parkedcar_pool[3].struct = STAGEC.CreateCar(STATIC_PARKED_CARS[3].x, STATIC_PARKED_CARS[3].y, STATIC_PARKED_CARS[3].z, STATIC_PARKED_CARS[3].a, STATIC_PARKED_CARS[3].model, STATIC_PARKED_CARS[3].trailer, STATIC_PARKED_CARS[3].color, 1)				
				mission_arrow_pool[1] = STAGEC.CreateArrow(71.5, 84.7, 2.0, 0, 4)
				--SetCarDoorLock(parkedcar_pool[3].struct, 0)
				target.name = "GET_TNT_CAR"
				return false, target
			end
			if (target.name == "GET_TNT_CAR") then
				if (GetCarDamage(parkedcar_pool[3].struct) ~= 125 and GetPedHealth(ped) > 0 and GetPedTask(ped) ~= 9) then
					if (IsPedInCar(ped, parkedcar_pool[3].struct)) then
						STAGEC.RemoveAllDialogs()
						STAGEC.ShowDialog(1096, 3)
						STAGEC.ShowDialog(1094, 2)
						mission_car_pool[1] = parkedcar_pool[3].struct
						STAGEC.DeleteArrow(mission_arrow_pool[1])
						mission_arrow_pool[1] = STAGEC.CreateArrow(STATIC_PAINTSHOP_TNT[target.stepid].x, STATIC_PAINTSHOP_TNT[target.stepid].y, STATIC_PAINTSHOP_TNT[target.stepid].z, 0, 4)
						mission_timer_pool[1] = STAGEC.CreateTimer(420)
						target.name = "GO_NEXT_PLANT"
						return false, target
					else
						target.name = "GET_TNT_CAR"
						return false, target
					end
				end
			end
			if (target.name == "GO_NEXT_PLANT") then
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and (not STAGEC.IsTimerOut(mission_timer_pool[1]))) then 
					if (not IsPedInCar(ped, mission_car_pool[1])) then
						if (IsPedAnyRangeOfPoint(ped, parkedcar_pool[3].x, parkedcar_pool[3].y, parkedcar_pool[3].z, 4)) then				
							if (IsPedRangeOfPoint(ped, STATIC_PAINTSHOP_TNT[target.stepid].x, STATIC_PAINTSHOP_TNT[target.stepid].y, STATIC_PAINTSHOP_TNT[target.stepid].z, 0.5) and GetPedSpeed(ped) == 0) then
								if (IsGameKeyPress(GetPedPlayer(ped), 0x8D)) then
									x, y, z = GetPedPos(ped)
									mission_arrow_pool[#mission_arrow_pool + 1] = STAGEC.CreateArrow(x, y, z, 108, 4) -- 100, 108
									mission_obj_pool[#mission_obj_pool + 1] = STAGEC.CreateObj(x, y, z, GetPedAngle(ped), 247, 0, 1)
									if (target.stepid >= #STATIC_PAINTSHOP_TNT) then
										--STAGEC.ShowDialog(1103, 3)
										STAGEC.DeleteArrow(mission_arrow_pool[1])
										mission_arrow_pool[1] = STAGEC.CreateArrow(95.7, 92.4, 2, 0, 4)
										STAGEC.SetPedPhoneState(ped, phone_pool[1].struct, 40, 1)
										target.name = "GO_PHONE_TNT"
										return false, target
									else
										STAGEC.ShowDialog(1094, 3)
										target.stepid = target.stepid + 1								
										STAGEC.DeleteArrow(mission_arrow_pool[1])
										mission_arrow_pool[1] = STAGEC.CreateArrow(STATIC_PAINTSHOP_TNT[target.stepid].x, STATIC_PAINTSHOP_TNT[target.stepid].y, STATIC_PAINTSHOP_TNT[target.stepid].z, 0, 4)								
										target.name = "GO_NEXT_PLANT"
										return false, target
									end
								else
									target.name = "GO_NEXT_PLANT"
									return false, target	
								end
							else
								target.name = "GO_NEXT_PLANT"
								return false, target
							end
						end			
					else
						parkedcar_pool[3].x, parkedcar_pool[3].y, parkedcar_pool[3].z = GetCarPos(mission_car_pool[1])				
						target.name = "GO_NEXT_PLANT"
						return false, target
					end
				end
			end
			if (target.name == "GO_PHONE_TNT") then
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and (not STAGEC.IsTimerOut(mission_timer_pool[1]))) then
					if (not IsPedInCar(ped, mission_car_pool[1])) then
						if (IsPedAnyRangeOfPoint(ped, parkedcar_pool[3].x, parkedcar_pool[3].y, parkedcar_pool[3].z, 4)) then										
							if (IsPedRangeOfPoint(ped, STATIC_PHONES[1].x, STATIC_PHONES[1].y, STATIC_PHONES[1].z, 0.5)) then								
								STAGEC.DeleteTimer(mission_timer_pool[1])
								STAGEC.DeleteArrow(mission_arrow_pool[1])
								STAGEC.SetPedPhoneState(ped, phone_pool[1].struct, 40, 0)
								mission_car_pool[1] = nil
								player = GetPedPlayer(ped)
								SetPlayerMoney(player, GetPlayerMoney(player) + 500000)
								STAGEC.RemoveAllDialogs()
								STAGEC.ShowDialog(1097, 3)
								OnPlayerMissionComplited(ped, 1)
								target.name = "MISSION_DONE"						
								return false, target
							else
								target.name = "GO_PHONE_TNT"						
								return false, target
							end
						end
					else
						parkedcar_pool[3].x, parkedcar_pool[3].y, parkedcar_pool[3].z = GetCarPos(mission_car_pool[1])				
						target.name = "GO_PHONE_TNT"
						return false, target
					end
				end
			end
			if (target.name == "MISSION_DONE") then
				OpenDoor(2)
				OpenDoor(3)	
				parkedcar_pool[3].doorlock = 0			
				for i = 1, 2 do
					STAGEC.SetLowerArea(174, 11, 174, 11)
					--coroutine.yield()
				end
				for i = 1, 2 do
					STAGEC.SetLowerArea(178, 7, 178, 7)
					--coroutine.yield()
				end
				for i = 1, 1 do
					STAGEC.SetLowerArea(175, 8, 177, 10)
					--coroutine.yield()
				end				
				SetGround(STATIC_NEW_GROUND, 1)
				SetGround(STATIC_NEW_GROUND2, 0)
				SetBuildings(STATIC_NEW_PAINTSHOP, 1, 170.5, 7.5, 0.0)
				STAGEC.CreateObj(175.0, 10.5, 2.0, 0, 130, 0, 2)
				-- x, y, z, range, intensity, red, green, blue, time_on, time_off, time_random
				STAGEC.CreateLight(172.5, 11.5, 3.0, 3.0, 205, 215, 215, 255, 220, 1, 40)
				STAGEC.CreateLight(177.5, 10.2, 2.0, 2.0, 105, 198, 198, 255, 0, 0, 10)
				STAGEC.CreateSound(175.0, 11.0, 2.0, 30, 0)								
				MissionInit(ped, 2, 2)
				target.name = "MISSION_COMPLITE"
				return true, target
			end
			STAGEC.RemoveAllDialogs()
			STAGEC.ShowDialog(1098, 3)
			if (mission_car_pool[1] ~= 0) then			
				CreateCustomExplosive(parkedcar_pool[3].x, parkedcar_pool[3].y, parkedcar_pool[3].z, 4, 12, 2)					
			end
			mission_car_pool[1] = nil
			target.name = "MISSION_FAILED"
			return true, OnPlayerMissionFailed(ped, 1)
		end
	},
	{state = 0, time_tmp = {}, target = {},
		mission = function(ped, target, phoneid)
			if (target.name == "INIT") then
				if (phone_pool[phoneid].state == 1) then
					return false, target
				end			
				phone_pool[phoneid].icon = 7
				phone_pool[phoneid].color = 4
				phone_pool[phoneid].state = 1
				phone_pool[phoneid].missionid = 2
				total_mission_stat[2] = 1
				return false, target
			end
			if (target.name == "START") then -- 2
				STAGEC.RemoveAllDialogs()
				STAGEC.ShowDialog(1099, 1)
				STAGEC.ShowDialog(1100, 2)
				STAGEC.ShowDialog(1101, 2)
				SetPedModel(ped, 0)
				SetPedSkin(ped, 16)
				target.name = "GO_TO_CORPSES"
				SetPoliceSkip(1)
				mission_timer_pool[1] = STAGEC.CreateTimer(120)
				return false, target
			end
			if (target.name == "GO_TO_CORPSES") then
				if (GetPedHealth(ped) > 0 and GetPedTask(ped) ~= 9 and (not STAGEC.IsTimerOut(mission_timer_pool[1]))) then
					mission_car_pool[1] = GetPedCar(ped)
					if (GetCarModel(mission_car_pool[1]) == 34) then
						STAGEC.ShowDialog(1102, 3)
						mission_arrow_pool[1] = STAGEC.CreateArrow(STATIC_PAINTSHOP_MEDIC[target.stepid].x, STATIC_PAINTSHOP_MEDIC[target.stepid].y, STATIC_PAINTSHOP_MEDIC[target.stepid].z, 0, 4)
						target.name = "GO_NEXT_CORPSES"
						return false, target
					end
					target.name = "GO_TO_CORPSES"
					return false, target
				end
			end
			if (target.name == "GO_NEXT_CORPSES") then		
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and (not STAGEC.IsTimerOut(mission_timer_pool[1]))) then
					if (IsCarRangeOfPoint(mission_car_pool[1], STATIC_PAINTSHOP_MEDIC[target.stepid].x, STATIC_PAINTSHOP_MEDIC[target.stepid].y, STATIC_PAINTSHOP_MEDIC[target.stepid].z, 0.5) and GetCarSpeed(mission_car_pool[1]) == 0) then
						if (target.stepid >= #STATIC_PAINTSHOP_MEDIC) then
							STAGEC.ShowDialog(1103, 3)
							STAGEC.DeleteTimer(mission_timer_pool[1])
							STAGEC.DeleteArrow(mission_arrow_pool[1])
							mission_arrow_pool[1] = STAGEC.CreateArrow(71.5, 135.5, 2, 0, 4)
							mission_ped_pool[1] = STAGEC.CreatePed(STATIC_MISSION_PED_MEDIC[1].x, STATIC_MISSION_PED_MEDIC[1].y, STATIC_MISSION_PED_MEDIC[1].z, STATIC_MISSION_PED_MEDIC[1].a, STATIC_MISSION_PED_MEDIC[1].skin, STATIC_MISSION_PED_MEDIC[1].occupation)
							SetPedModel(mission_ped_pool[1], STATIC_MISSION_PED_MEDIC[1].model)
							SetPedThreatReaction(mission_ped_pool[1], STATIC_MISSION_PED_MEDIC[1].reaction)	
							--STAGEC.CreatePedGroup(mission_ped_pool[1], 3)
							--STAGEC.SetPedGroupType(mission_ped_pool[1], 2)
							target.name = "GO_TO_CREMOTORY"
							return false, target
						else
							STAGEC.AddTimerTime(mission_timer_pool[1], 30)
							target.stepid = target.stepid + 1
							STAGEC.DeleteArrow(mission_arrow_pool[1])
							target.name = "GO_TO_CORPSES"
							return false, target
						end
					else					
						target.name = "GO_NEXT_CORPSES"
						return false, target
					end
				end
			end
			if (target.name == "GO_TO_CREMOTORY") then	
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and GetPedHealth(mission_ped_pool[1]) > 0) then
					x, y, z = GetArrowPos(mission_arrow_pool[1])
					if (IsCarRangeOfPoint(mission_car_pool[1], x, y, z, 0.5) and GetCarSpeed(mission_car_pool[1]) == 0) then
						STAGEC.ShowDialog(1106, 3)
						STAGEC.DeleteArrow(mission_arrow_pool[1])
						target.name = "CREMOTORIES"
						return false, target
					else
						local car_pool = {}
						local trytv = 0
						x, y, z = GetPedAnyPos(ped)
						car_pool = GetCarFromPoint(x, y, z, 6.0)
						for i,car in ipairs(car_pool) do
							if (GetCarModel(car) == 67) then
								STAGEC.ShowDialog(1104, 3)
								mission_ped_pool[2] = STAGEC.CreatePedInCar(car, math.random(4,36), 1)		
								STAGEC.DeleteArrow(mission_arrow_pool[1])
								STAGEC.SetPedObjective(mission_ped_pool[2], mission_car_pool[1], 0, 55, 2)
								STAGEC.SetPedMaxDriveSpeed(mission_ped_pool[2], 0.9)
								mission_arrow_pool[1] = STAGEC.CreateArrow(x, y, z, 0, 4)
								target.name = "ESCAPE_FROM_TV"
								return false, target
							end
						end
						target.name = "GO_TO_CREMOTORY"
						return false, target
					end
				end
			end
			if (target.name == "ESCAPE_FROM_TV") then	
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and IsPedInCar(ped, mission_car_pool[1])) then					
					if (#mission_ped_pool > 1) then
						x, y, z = GetPedAnyPos(mission_ped_pool[2])
						if (IsPedAnyRangeOfPoint(ped, x, y, z, 12.0)) then
							SetArrowPos(mission_arrow_pool[1], x, y, z)
							target.name = "ESCAPE_FROM_TV"
							return false, target
						else
							STAGEC.DeletePed(mission_ped_pool[2])
							STAGEC.DeleteArrow(mission_arrow_pool[1])
						end						
					end
					STAGEC.ShowDialog(1105, 3)
					mission_arrow_pool[1] = STAGEC.CreateArrow(71.5, 135.5, 2, 0, 4)
					target.name = "GO_TO_CREMOTORY"
					return false, target
				end
			end			
			if (target.name == "CREMOTORIES") then	
				if (GetCarDamage(mission_car_pool[1] and GetPedHealth(mission_ped_pool[1]) > 0) ~= 125) then
					if (IsPedInCar(ped, mission_car_pool[1])) then
						target.name = "CREMOTORIES"
						return false, target
					else
						STAGEC.SetPedObjective(mission_ped_pool[1], mission_car_pool[1], 0, 35, 2)
						STAGEC.SetPedDriveAggression(mission_ped_pool[1], 1)
						STAGEC.SetPedStayInCar(mission_ped_pool[1], 1)
						--STAGEC.SetPedNoting(mission_ped_pool[1])
						target.name = "TAKE_CAR"
						return false, target
					end
				end
			end
			if (target.name == "TAKE_CAR") then	
				if (GetCarDamage(mission_car_pool[1]) ~= 125 and GetPedHealth(mission_ped_pool[1]) > 0) then
					x, y, z = GetPedAnyPos(ped)
					if (IsPedAnyRangeOfPoint(mission_ped_pool[1], x, y, z, 8.0)) then
						target.name = "TAKE_CAR"
						return false, target						
					else
						STAGEC.ShowDialog(1107, 3)	
						SetPoliceSkip(0)
						SetCarDamage(mission_car_pool[1], 125)
						mission_car_pool[1] = nil		
						player = GetPedPlayer(ped)
						SetPlayerMoney(player, GetPlayerMoney(player) + 1000000)						
						OnPlayerMissionComplited(ped, 2)
						target.name = "MISSION_DONE"						
						return false, target
					end
				end
			end
			if (target.name == "MISSION_DONE") then
				AddStaticPickup(93.5, 84.5, 2.0, 0.0, 234, 1, 25000)
				MissionInit(ped, 3, 3)
				target.name = "MISSION_COMPLITE"
				return true, target
			end
			STAGEC.RemoveAllDialogs()
			STAGEC.ShowDialog(1108, 3)
			SetPoliceSkip(0)
			SetPedWanted(ped, 5)
			target.name = "MISSION_FAILED"
			return true, OnPlayerMissionFailed(ped, 2)
		end
	},
	{state = 0, time_tmp = {}, target = {},
		mission = function(ped, target, phoneid)
			if (target.name == "INIT") then -- 1
				if (phone_pool[phoneid].state == 1) then
					return false, target
				end
				phone_pool[phoneid].icon = 2
				phone_pool[phoneid].color = 4
				phone_pool[phoneid].state = 1
				phone_pool[phoneid].missionid = 3
				total_mission_stat[3] = 1
				return false, target
			end
			if (target.name == "START") then -- 2
				STAGEC.RemoveAllDialogs()
				STAGEC.ShowDialog(1008, 1)
				STAGEC.ShowDialog(1009, 2)
				STAGEC.ShowDialog(1010, 2)
				STAGEC.ShowDialog(1011, 2)
				STAGEC.ShowDialog(1012, 2)
				for i,ped in ipairs(STATIC_MISSION_PEDS) do
					make_ped_lock = 1
					mission_ped_pool[i] = STAGEC.CreatePed(ped.x, ped.y, ped.z, ped.a, ped.skin, ped.occupation)
					SetPedThreatReaction(mission_ped_pool[i], ped.reaction)
					STAGEC.GivePedWeapon(mission_ped_pool[i], ped.weapon, 255)
					make_ped_lock = 0
				end		
				for i,car in ipairs(STATIC_MISSION_CARS) do
					make_car_lock = 1
					mission_car_pool[i] = STAGEC.CreateCar(car.x, car.y, car.z, car.a, car.model, 65535, car.color, 1)
					Car.SetCarAlarmInit(mission_car_pool[i], car.alarm)
					SetCarDoorLock(mission_car_pool[i], car.doorlock)
					SetCarArmour(mission_car_pool[i], car.armor)
					make_car_lock = 0
				end	
				mission_arrow_pool[1] = STAGEC.CreateArrow(STATIC_MISSION_PEDS[3].x, STATIC_MISSION_PEDS[3].y, STATIC_MISSION_PEDS[3].z, 0, 4) -- red
				SetPedModel(ped, 2)
				SetPedSkin(ped, 0)
				target.name = "ATTAK_TO_PLAYER"
				return false, target
			end
			if (target.name == "ATTAK_TO_PLAYER") then -- 3
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					if (GetPedWanted(ped) <= 0) then
						if (IsPedAnyRangeOfPoint(ped, STATIC_MISSION_PEDS[3].x, STATIC_MISSION_PEDS[3].y, STATIC_MISSION_PEDS[3].z, 12)) then
							STAGEC.ShowDialog(1013, 3)
							for i,mission_ped in ipairs(mission_ped_pool) do
								SetPedThreatSearch(mission_ped_pool[i], 6)
								wait(50)
								STAGEC.SetPedObjective(mission_ped_pool[i], ped, 0, 20, 2)
							end	
							STAGEC.DeleteArrow(mission_arrow_pool[1])
							mission_arrow_pool[1] = STAGEC.CreateArrow(STATIC_MISSION_PEDS[3].x, STATIC_MISSION_PEDS[3].y, STATIC_MISSION_PEDS[3].z, 7, 4) -- red
							mission_arrow_pool[2] = STAGEC.CreateArrow(STATIC_MISSION_PEDS[9].x, STATIC_MISSION_PEDS[9].y, STATIC_MISSION_PEDS[9].z, 5, 4) -- yellow
							target.name = "KILL_TARGET_RED"
							return false, target
						else
							target.name = "ATTAK_TO_PLAYER"
							return false, target
						end
					end
				end
			end
			if (target.name == "KILL_TARGET_RED") then
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					x, y, z = GetPedAnyPos(mission_ped_pool[9])
					SetArrowPos(mission_arrow_pool[2], x, y, z)
					if (GetPedHealth(mission_ped_pool[3]) > 0) then
						x, y, z = GetPedAnyPos(mission_ped_pool[3])
						SetArrowPos(mission_arrow_pool[1], x, y, z)
						return false, target
					else
						STAGEC.DeleteArrow(mission_arrow_pool[1])
						target.name = "TOUCH_TARGET_YELLOW"
						return false, target
					end				
				end
			end
			if (target.name == "TOUCH_TARGET_YELLOW") then
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					x, y, z = GetPedAnyPos(mission_ped_pool[9])
					SetArrowPos(mission_arrow_pool[2], x, y, z)
					if (GetPedTouch(ped) == mission_ped_pool[9]) then
						STAGEC.DeleteArrow(mission_arrow_pool[2])	
						STAGEC.CreatePedGroup(mission_ped_pool[9], 0)
						STAGEC.SedPedGroupLeader(ped, mission_ped_pool[9])
						SetCarDoorLock(mission_car_pool[3], 0)
						mission_arrow_pool[1] = STAGEC.CreateArrow(77.0, 84.2, 2.0, 0, 4)
						target.name = "GO_TO_HOME"
						return false, target
					else
						target.name = "TOUCH_TARGET_YELLOW"
						return false, target
					end
				end
			end
			if (target.name == "GO_TO_HOME") then
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					if (GetPedWanted(ped) > 0) then
						STAGEC.RemoveAllDialogs()
						STAGEC.ShowDialog(1014, 3)
						STAGEC.DeleteArrow(mission_arrow_pool[1])
						mission_arrow_pool[1] = STAGEC.CreateArrow(219.3, 34.4, 2.0, 0, 4)
						target.name = "GO_TO_PAINTSHOP"
						return false, target	
					else
						if (IsPedAnyRangeOfPoint(mission_ped_pool[9], 77.0, 84.2, 2.0, 0.5) and GetPedSpeed(mission_ped_pool[9]) == 0) then
							STAGEC.RemoveAllDialogs()
							STAGEC.ShowDialog(1017, 3)	
							player = GetPedPlayer(ped)
							SetPlayerMoney(player, GetPlayerMoney(player) + 150000)							
							OnPlayerMissionComplited(ped, 3)
							target.name = "MISSION_DONE"
							return false, target 	
						else
							target.name = "GO_TO_HOME"
							return false, target	
						end
					end
				end
			end
			if (target.name == "GO_TO_PAINTSHOP") then
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					if (GetPedWanted(ped) > 0) then
						if (IsCarRangeOfPoint(GetPedCar(ped), 219.3, 34.4, 2.0, 0.5)) then
							STAGEC.RemoveAllDialogs()
							STAGEC.ShowDialog(1015, 3)
							STAGEC.DeleteArrow(mission_arrow_pool[1])
							mission_arrow_pool[1] = STAGEC.CreateArrow(175.0, 10.5, 2.0, 0, 4)
							target.name = "GO_NEW_PAINTSHOP"
							return false, target
						else
							target.name = "GO_TO_PAINTSHOP"
							return false, target
						end
					else
						STAGEC.RemoveAllDialogs()
						STAGEC.ShowDialog(1016, 3)
						STAGEC.DeleteArrow(mission_arrow_pool[1])
						mission_arrow_pool[1] = STAGEC.CreateArrow(77.0, 84.2, 2.0, 0, 4)
						target.name = "GO_TO_HOME"
						return false, target
					end	
				end
			end
			if (target.name == "GO_NEW_PAINTSHOP") then
				if (GetPedHealth(mission_ped_pool[9]) > 0 and GetPedTask(mission_ped_pool[9]) ~= 9 and GetPedTask(ped) ~= 9) then
					if (GetPedWanted(ped) > 0) then
						if (IsCarRangeOfPoint(GetPedCar(ped), 175.0, 10.5, 2.0, 0.5)) then
							STAGEC.DeleteArrow(mission_arrow_pool[1])
							target.name = "GO_NEW_PAINTSHOP"
							return false, target
						else
							target.name = "GO_NEW_PAINTSHOP"
							return false, target
						end
					else
						target.name = "GO_TO_PAINTSHOP"
						return false, target
					end
				end
			end
			if (target.name == "MISSION_DONE") then
				AddStaticPickup(77.7, 84.3, 2.0, 0.0, 231, 10, 10000)
				STAGEC.CreateObj(219.3, 34.4, 2.0, 0, 130, math.random(36), 2)
				target.name = "MISSION_COMPLITE"
				return true, target
			end
			STAGEC.RemoveAllDialogs()
			STAGEC.ShowDialog(1018, 3)			
			target.name = "MISSION_FAILED"
			return true, OnPlayerMissionFailed(ped, 3)
		end
	}
}

-- CreateDoor(x, y, z)

function CloseDoor(doorid)
	local new_block = {}
	if (doorid >= 1 and doorid <= #STATIC_DOORS) then
		if (door_pool[doorid] ~= nil and door_pool[doorid].state == 1) then
			STAGEC.DeleteBlock(door_pool[doorid].block, 0)
			door_pool[doorid] = nil
		end
		new_block = STAGEC.CreateBlock(STATIC_DOORS[doorid].x, STATIC_DOORS[doorid].y, STATIC_DOORS[doorid].z)
		STAGEC.SetBlockSide(new_block, STATIC_DOORS[doorid].face_type, STATIC_DOORS[doorid].wall_type, STATIC_DOORS[doorid].bullet_type, STATIC_DOORS[doorid].flat_type, STATIC_DOORS[doorid].flip_type, STATIC_DOORS[doorid].rotation, STATIC_DOORS[doorid].tileid) -- Set wall tile
		door_pool[doorid] = {block = new_block, state = 1}			
		total_buildings = total_buildings + 1
		--WriteInLog("CloseDoor() total_buildings "..total_buildings.." OK\n")
		return true
	end
	return false
end

function OpenDoor(doorid)
	if (doorid >= 1 and doorid <= #STATIC_DOORS) then
		if (door_pool[doorid] ~= nil) then
			STAGEC.DeleteBlock(door_pool[doorid].block, 0)
			door_pool[doorid] = nil
			total_remove_buildings = total_remove_buildings - 1
			--WriteInLog("OpenDoor() total_buildings "..total_buildings.." OK\n")
			return true
		end
	end
	return false
end

function RemoveStaticBuildings(buildings, mode)
	local new_block = {}
	for i,block in ipairs(buildings) do
		WriteInLog("RemoveStaticBuildings() index "..i.." pos x "..block[1].." y "..block[2].." z "..block[3].."\n")
		new_block = {x = block[1], y = block[2], z = block[3]}
		STAGEC.DeleteBlock(new_block, mode)
		total_remove_buildings = total_remove_buildings + 1
		--WriteInLog("RemoveStaticBuildings() index "..i.." total_remove_buildings "..total_remove_buildings.." OK\n")

	end	
	--WriteInLog("RemoveStaticBuildings() total_remove_buildings "..total_remove_buildings.." OK\n")
	return true
end

function SetBuildings(buildings, mode, offset_x, offset_y, offset_z)
	local new_block = {}
	if (offset_x == nil) then
		offset_x = 0
	end
	if (offset_y == nil) then
		offset_y = 0
	end
	if (offset_z == nil) then
		offset_z = 0
	end
	for i,block in ipairs(buildings) do
		--WriteInLog("CreateBuildings() index "..i.." pos x "..block[1].." y "..block[2].." z "..block[3].."\n")	
		if (block[1] > 0 and block[1] < 256 and block[2] > 0 and  block[2] < 256 and block[3] > 0 and block[3] < 8) then
			if (mode == 1) then
				new_block = STAGEC.CreateBlock(block[1] + offset_x,  block[2] + offset_y, block[3] + offset_z)
			else
				new_block = {x = block[1] + offset_x, y = block[2] + offset_y , z = block[3] + offset_z}
			end
		end
		if (block[4] == 1) then
			STAGEC.SetBlockSide(new_block, block[4], block[5], block[6], block[7], block[8], block[9], block[10])
		end
		if (block[11] == 2) then
			STAGEC.SetBlockSide(new_block, block[11], block[12], block[13], block[14], block[15], block[16], block[17])
		end
		if (block[18] == 3) then
			STAGEC.SetBlockSide(new_block, block[18], block[19], block[20], block[21], block[22], block[23], block[24])
		end
		if (block[25] == 4) then
			STAGEC.SetBlockSide(new_block, block[25], block[26], block[27], block[28], block[29], block[30], block[31])
		end
		if (block[32] == 5) then
			STAGEC.SetBlockLID(new_block, block[33], block[34], block[35], block[36], block[37])
		end
		if (block[38] == 6) then
			STAGEC.SetBlockType(new_block, block[39], block[40])
		end	
		--coroutine.yield()
		total_buildings = total_buildings + 1
		--WriteInLog("CreateBuildings() index "..i.." total_buildings "..total_buildings.." OK\n")
		--coroutine.yield()
	end	
	--WriteInLog("CreateBuildings() total_buildings "..total_buildings.." OK\n")
	return true
end

function SetGround(grounds, mode, offset_x, offset_y, offset_z)
	local new_block = {}
	if (offset_x == nil) then
		offset_x = 0
	end
	if (offset_y == nil) then
		offset_y = 0
	end
	if (offset_z == nil) then
		offset_z = 0
	end
	for i,block in ipairs(grounds) do	
		--WriteInLog("CreateGround() index "..i.." pos x "..block[1].." y "..block[2].." z "..block[3].." type "..tostring(block[11]).."\n")		
		if (block[1] > 0 and block[1] < 256 and block[2] > 0 and  block[2] < 256 and block[3] > 0 and block[3] < 8) then
			--STAGEC.DeleteBlock({x = block[1], y = block[2], z = block[3]}, 0)
			if (mode == 1) then
			new_block = STAGEC.CreateBlock(block[1] + offset_x, block[2] + offset_y, block[3] + offset_z)
			else 
			new_block = {x = block[1] + offset_x, y = block[2] + offset_y , z = block[3] + offset_z}
			end
		end
		if (block[4] == 5) then
			STAGEC.SetBlockLID(new_block, block[5], block[6], block[7], block[8], block[9])
		end
		if (block[10] == 6) then
			STAGEC.SetBlockType(new_block, block[11], block[12])
		end	
		--coroutine.yield()
		total_buildings = total_buildings + 1
		--WriteInLog("CreateGround() index "..i.." total_buildings "..total_buildings.." OK\n")
		--coroutine.yield()
	end	
	--WriteInLog("CreateGround() total_buildings "..total_buildings.." OK\n")
	return true
end

function CloseDoors()
	WriteInLog(string.format("#door_pool %d\n", #door_pool))
	for i,door in ipairs(STATIC_DOORS) do
		CloseDoor(i)
		--coroutine.yield()
	end	
	WriteInLog(string.format("#door_pool %d\n", #door_pool))
	return true
end

function CreatePhones()
	WriteInLog(string.format("#phone_pool %d\n", #phone_pool))
	for i,phone in ipairs(STATIC_PHONES) do
		make_obj_lock = 1
		--WriteInLog(string.format("model %d pos %f %f %f rot %f\n", phone.model, phone.x, phone.y, phone.z, phone.a))
		phone_pool[i] = {x = phone.x, y = phone.y, z = phone.z, struct = 0, arrow = 0, state = 0, missionid = 0, icon = 0, color = 4}
		phone_pool[i].struct = STAGEC.CreateObj(phone.x, phone.y, phone.z, phone.a, phone.model, 0, 1)
		make_obj_lock = 0
		--coroutine.yield()
	end	
	WriteInLog(string.format("#phone_pool %d\n", #phone_pool))
	return true
end

function EnableMissionPhones(ped)
	for i,phone in ipairs(STATIC_PHONES) do
		if (phone_pool[i].missionid > 0) then
			if (total_mission_stat[phone_pool[i].missionid] == 1 or total_mission_stat[phone_pool[i].missionid] == 2) then
				phone_pool[i].arrow = STAGEC.CreateArrow(phone_pool[i].x, phone_pool[i].y, phone_pool[i].z, phone_pool[i].icon, phone_pool[i].color)
				STAGEC.SetPedPhoneState(ped, phone_pool[i].struct, 40, 1)
				phone_pool[i].state = 1
			end		
		end		
	end
	return true
end

function DisableMissionPhones(ped)
	for i,phone in ipairs(phone_pool) do
		if (phone_pool[i].state > 0) then
			STAGEC.DeleteArrow(phone_pool[i].arrow)	
			STAGEC.SetPedPhoneState(ped, phone_pool[i].struct, 40, 0)
			phone_pool[i].state = 0
		end
	end
	return true
end

function CreatePickups()
	WriteInLog(string.format("#pickup_pool %d\n", #pickup_pool))
	for i,pickup in ipairs(STATIC_PICKUPS) do
		make_obj_lock = 1
		--WriteInLog(string.format("model %d pos %f %f %f rot %f\n", pickup.model, pickup.x, pickup.y, pickup.z, pickup.a))
		pickup_pool[i] = {struct = 0, time_tmp = 0, time_respawn = 0}
		pickup_pool[i].struct = STAGEC.CreateObj(pickup.x, pickup.y, pickup.z, pickup.a, pickup.model, pickup.value, 2)
		pickup_pool[i].time_tmp = os.clock() * 1000 + pickup.time_respawn
		pickup_pool[i].time_respawn = pickup.time_respawn
		make_obj_lock = 0
		--coroutine.yield()
	end	
	WriteInLog(string.format("#pickup_pool %d\n", #pickup_pool))
	return true
end

function AddStaticPickup(x, y, z, a, model, value, time_respawn)
	if (make_obj_lock == 0) then
		make_obj_lock = 1
		STATIC_PICKUPS[#STATIC_PICKUPS + 1] = {x = x, y = y, z = z, a = a, model = model, value = value, time_respawn = time_respawn}
		pickup_pool[#pickup_pool + 1] = {struct = 0, time_tmp = 0, time_respawn = 0}							
		pickup_pool[#pickup_pool].struct = STAGEC.CreateObj(STATIC_PICKUPS[#STATIC_PICKUPS].x, STATIC_PICKUPS[#STATIC_PICKUPS].y, STATIC_PICKUPS[#STATIC_PICKUPS].z, STATIC_PICKUPS[#STATIC_PICKUPS].a, STATIC_PICKUPS[#STATIC_PICKUPS].model, STATIC_PICKUPS[#STATIC_PICKUPS].value, 2)
		pickup_pool[#pickup_pool].time_tmp = os.clock() * 1000 + STATIC_PICKUPS[#STATIC_PICKUPS].time_respawn
		pickup_pool[#pickup_pool].time_respawn = STATIC_PICKUPS[#STATIC_PICKUPS].time_respawn
		make_obj_lock = 0
		return pickup_pool[#pickup_pool].struct
	end
	return 0
end

function CreateCustomExplosive(x, y, z, range, hits, mode)
	co_create_explosive = coroutine.create(function(x, y, z, range, hits, mode)
		if (mode == 1) then
			for i = 1, hits do		
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 2, 0)
				coroutine.yield()
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 2, 0)
				coroutine.yield()
			end
		elseif (mode == 2) then
			STAGEC.CreateExplode(x, y, z, 3, 0)
			coroutine.yield()
			for i = 1, hits-1 do		
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 3, 0)
				coroutine.yield()
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 3, 0)
				coroutine.yield()
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 1, 0)
				coroutine.yield()
				STAGEC.CreateExplode(x + math.random(-range, range), y + math.random(-range, range), z, 3, 0)
				coroutine.yield()
			end
		end
		return true
	end)
	coroutine.resume(co_create_explosive, x, y, z, range, hits, mode)
	return true
end
							
function CreateParkedCars()
	WriteInLog(string.format("#parkedcar_pool %d\n", #parkedcar_pool))
	for i,car in ipairs(STATIC_PARKED_CARS) do
		--WriteInLog(string.format("model %d pos %f %f %f rot %f\n", car.model, car.x, car.y, car.z, car.a))
		make_car_lock = 1
		parkedcar_pool[i] = {x = 0, y = 0, z = 0, struct = STAGEC.CreateCar(car.x, car.y, car.z, car.a, car.model, car.trailer, car.color, 1), time_tmp = os.clock() * 1000 + car.time_respawn, time_respawn = car.time_respawn}
		SetCarDoorLock(parkedcar_pool[i].struct, car.doorlock)
		Car.SetCarAlarmInit(parkedcar_pool[i].struct, car.alarm)	
		make_car_lock = 0
		--coroutine.yield()
	end	
	WriteInLog(string.format("#parkedcar_pool %d\n", #parkedcar_pool))
	return true
end

function MissionInit(ped, missionid, phoneid)
	if (total_mission_stat[missionid] ~= 3) then
		mission_pool[missionid].target = {name = "INIT", stepid = 1}
		mission_pool[missionid].state = 1
		mission_pool[missionid].mission(ped, mission_pool[missionid].target, phoneid)
		total_mission_stat[missionid] = 1	
		return true
	end
	return false
end

function MissionDone(ped)
	for i = 1, #total_mission_stat do
		if (total_mission_stat[i] == 3) then
			mission_pool[i].target = {name = "MISSION_DONE", stepid = 1}
			mission_pool[i].mission(ped, mission_pool[i].target)
		end
	end
	return true
end

function DeleteMission(ped, missionid)
	for i,arrow in ipairs(mission_arrow_pool) do
		STAGEC.DeleteArrow(arrow)
		mission_arrow_pool[i] = nil
	end
	for i,timer in ipairs(mission_timer_pool) do
		STAGEC.DeleteTimer(timer)
		mission_timer_pool[i] = nil
	end
	for i,ped in ipairs(mission_ped_pool) do
		STAGEC.DeletePed(ped)
		mission_ped_pool[i] = nil
	end
	for i,car in ipairs(mission_car_pool) do
		STAGEC.RemoveCar(car)
		mission_car_pool[i] = nil
	end
	for i,obj in ipairs(mission_obj_pool) do
		STAGEC.DeleteObj(obj)
		mission_obj_pool[i] = nil
	end
	for i,mis in ipairs(mission_pool) do
		mission_pool[i].target = nil
	end
	current_missionid = 1
	missionstate = 0	
	return true
end

function ClearStage()
	WriteInLog("ClearStage()...\n")
	make_ped_lock = 0
	make_car_lock = 0
	make_obj_lock = 0
	make_arrow_lock = 0
	make_block_lock = 0	
	STAGEC.MAX_ARROWS = 0
	current_missionid = 1
	missionstate = 0
	gamestate = 0
	for i,door in ipairs(door_pool) do
		door_pool[i] = nil
	end
	for i,phone in ipairs(phone_pool) do
		phone_pool[i] = nil
	end
	for i,parkedcar in ipairs(parkedcar_pool) do
		parkedcar_pool[i] = nil
	end
	for i,pickup in ipairs(pickup_pool) do
		pickup_pool[i] = nil
	end	
	for i,mis in ipairs(mission_pool) do
		DeleteMission(0, i)
	end	
	WriteInLog("ClearStage() OK\n")
	return true
end

function CheakPhones(ped)
	for i,phone in ipairs(STATIC_PHONES) do
		if (phone_pool[i].state == 1) then
			if (IsPedRangeOfPoint(ped, STATIC_PHONES[i].x, STATIC_PHONES[i].y, STATIC_PHONES[i].z, 0.5)) then
				--WriteInLog("CheakPhones() SetPedPhoneState() struct "..phone_pool[i].struct.."...\n")
				STAGEC.SetPedPhoneState(ped, phone_pool[i].struct, 40, 0)
				--WriteInLog("CheakPhones() SetPedPhoneState() OK\n")
				--WriteInLog("CheakPhones() DeleteArrow() struct "..phone_pool[i].arrow.."...\n")
				STAGEC.DeleteArrow(phone_pool[i].arrow)	
				--WriteInLog("CheakPhones() DeleteArrow() OK\n")
				phone_pool[i].state = 0	
				for j,mission in ipairs(mission_pool) do
					--WriteInLog("CheakPhones() Mission id "..j.."...\n")
					if (phone_pool[i].missionid == j) then
						--WriteInLog("CheakPhones() MissionStart...\n")
						DisableMissionPhones(ped)
						--WriteInLog("CheakPhones() MissionStart set...\n")
						mission_pool[j].target = {name = "START", stepid = 1}						
						current_missionid = j						
						missionstate = 1
						--WriteInLog("CheakPhones() MissionStart OK\n")						
						return true
					end
					coroutine.yield()
				end
				return false
			end
		end
		coroutine.yield()
	end
	return false
end

function CheakParkedCars(ped)
	for i,car in ipairs(STATIC_PARKED_CARS) do
		if (parkedcar_pool[i].struct ~= 0) then
			x, y, z = GetPedAnyPos(ped)
			if (make_car_lock == 0 and (GetCarDamage(parkedcar_pool[i].struct) >= 125 or parkedcar_pool[i].struct == 0) and Game.GetDistanceOfPoint(x, y, z, car.x, car.y, car.z) >= 8.0) then	
				if (os.clock() * 1000 - parkedcar_pool[i].time_tmp >= car.time_respawn) then
					make_car_lock = 1	
					--STAGEC.DeleteForceObj(parkedcar_pool[i].struct)
					STAGEC.DeleteCar(parkedcar_pool[i].struct)	
					parkedcar_pool[i].struct = STAGEC.CreateCar(car.x, car.y, car.z, car.a, car.model, car.trailer, car.color, 1)
					parkedcar_pool[i].time_tmp = os.clock() * 1000 + car.time_respawn
					parkedcar_pool[i].time_respawn = car.time_respawn
					SetCarDoorLock(parkedcar_pool[i].struct, car.doorlock)
					Car.SetCarAlarmInit(parkedcar_pool[i].struct, car.alarm)
					make_car_lock = 0
					WriteInLog(string.format("Generated parked car index %d id %d name %s struct %X model %d color %d\n",i,GetCarID(parkedcar_pool[i].struct),Car.GetCarName(parkedcar_pool[i].struct), parkedcar_pool[i].struct, GetCarModel(parkedcar_pool[i].struct), GetCarColor(parkedcar_pool[i].struct) ))	
				end
			else
				parkedcar_pool[i].time_tmp = os.clock() * 1000 + car.time_respawn
			end
		end
		coroutine.yield()
	end
	return false
end

function CheakPickups(ped)
	for i,pickup in ipairs(STATIC_PICKUPS) do
		if (pickup_pool[i].struct ~= 0) then
			x, y, z = GetPedAnyPos(ped)
			if (make_obj_lock == 0 and GetObjModel(pickup_pool[i].struct) == 0 and Game.GetDistanceOfPoint(x, y, z, pickup.x, pickup.y, pickup.z) >= 8.0) then
				if (os.clock() * 1000 - pickup_pool[i].time_tmp >= pickup_pool[i].time_respawn) then	
					make_obj_lock = 1
					WriteInLog(string.format("Generation pickup index %d...\n",i ))
					STAGEC.DeleteForceObj(pickup_pool[i].struct)
					if (pickup_pool[i].time_respawn == 0) then
						pickup_pool[i] = nil
						return true
					end
					pickup_pool[i].struct = STAGEC.CreateObj(pickup.x, pickup.y, pickup.z, pickup.a, pickup.model, pickup.value, 2)
					pickup_pool[i].time_tmp = os.clock() * 1000 + pickup.time_respawn
					pickup_pool[i].time_respawn = pickup.time_respawn
					make_obj_lock = 0
					WriteInLog(string.format("Generated pickup index %d id %d struct %X model %d\n",i,GetObjID(pickup_pool[i].struct),pickup_pool[i].struct, GetObjModel(pickup_pool[i].struct) ))
				end
			else
				pickup_pool[i].time_tmp = os.clock() * 1000 + pickup.time_respawn
			end
		end
		coroutine.yield()
	end
	return false
end
	
function OnNewGame(ped)	
	STAGEC.RemoveAllDialogs()
	STAGEC.ShowDialog(1001, 1)
	STAGEC.SetAmbientLevel(stagelight[game_h], 0)
	SetMaxPedDensity(ped_density[game_h])
	local playerid = GetPedID(ped)
	if (playerid < 1 or playerid > 6) then
		gamestate = 0
		return false
	end	
	player = GetPedPlayer(ped)
	SetPlayerGameCam(player, STATIC_PLAYER_SPAWNS[playerid].x,STATIC_PLAYER_SPAWNS[playerid].y,STATIC_PLAYER_SPAWNS[playerid].z)
	SetPedPos(ped, STATIC_PLAYER_SPAWNS[playerid].x,STATIC_PLAYER_SPAWNS[playerid].y,STATIC_PLAYER_SPAWNS[playerid].z)
	SetPedAngle(ped, STATIC_PLAYER_SPAWNS[playerid].a)	
	SetPedHealth(ped, 100)	
	SetPedModel(ped, STATIC_PLAYER_SPAWNS[playerid].model)
	SetPedSkin(ped, STATIC_PLAYER_SPAWNS[playerid].skin)	
	if (string.upper(string.match(GetGMPName(), "[/|\\](.*)")) == "WIL.GMP" or string.upper(string.match(GetGMPName(), "[/|\\](.*)")) == "WIL-MULTI.GMP") then	
		RemoveStaticBuildings(STATIC_REMOVE_BUILDINGS)	
		SetBuildings(STATIC_BUILDINGS, 0)			
		CloseDoors()
		CreateParkedCars()	
		CreatePhones()	
		CreatePickups()	
		MissionDone(ped)
		MissionInit(ped, 1, 1) -- TNT Men
		--MissionInit(ped, 2, 2) -- Medic
		--MissionInit(ped, 3, 3) -- Cop	
		EnableMissionPhones(ped)	
		co_phones = coroutine.create(CheakPhones, ped)
		co_parked_cars = coroutine.create(CheakParkedCars, ped)
		co_pickups = coroutine.create(CheakPickups, ped)
		STAGEC.SetTVDir(171, 206)	
		validstage = 1
	else
		validstage = 0
	end
	missionstate = 0
	gamestate = 2	
	WriteInLog(string.format("Player id %d struct %X new game is started.\n", playerid, ped))
	return true
end

function OnPlayerInput(ped)
	if (IsKeyDown(0x43)) then -- Down key "C" for save current position in MISI.log
		car = GetPedCar(ped)
		if (car ~= 0) then
			x, y, z = GetCarPos(car)
			WriteInLog(string.format("Saved car id %d, struct %X, {x = %.01f, y = %.01f, z = %.01f, a = %.01f, model = %d, color = %d}, name %s\n", GetCarID(car), car, x, y, z, GetCarAngle(car), GetCarModel(car), GetCarColor(car), Car.GetCarName(car)))
		else
			x, y, z = GetPedPos(ped)
			WriteInLog(string.format("Saved ped id %d, struct %X, {x = %.01f, y = %.01f, z = %.01f, a = %.01f, model = %d, skin = %d, weapon = %d}, name %s\n", GetPedID(ped), ped, x, y, z, GetPedAngle(ped), GetPedModel(ped), GetPedSkin(ped), GetPedCurrentWeaponID(ped), Ped.GetWeaponName(GetPedCurrentWeaponID(ped))))
		end
		return true
	end
	if (IsKeyPress(0xA2)) then -- Press key CTRL for walking
		SetPedSpeed(ped, walk_speed)
		return true
	end
	if (IsKeyPress(0xA0)) then -- Press key LSHIFT for sprinting
		SetPedSpeed(ped, sprint_speed)
		return true
	end
	if (IsKeyDown(0x47)) then -- Down key G for enter as passenger
		x, y, z = GetPedPos(ped)
		--CreateCustomExplosive(x, y, z, 12, 200, 1) -- Air strike	
		car_pool = GetCarFromPoint(x, y, z, 2)
		for i,car in ipairs(car_pool) do
			if (car ~= 0) then
				--WriteInLog(string.format("car index %d id %d struct %X\n",i,GetCarID(car),car))
				SetPedCurrentCarDoorID(ped, 1)
				STAGEC.SetPedObjective(ped, car, 1, 35, 2)
				break
			end
		end
		return true
	end
	return false
end

function OnPlayerSpawn(ped)
	--STAGEC.RemoveAllDialogs()
	STAGEC.ShowDialog(1003, 1)
	local playerid = GetPedID(ped)
	if (playerid < 1 or playerid > 6) then
		gamestate = 0
		return false
	end
	player = GetPedPlayer(ped)
	SetPlayerGameCam(player, STATIC_PLAYER_SPAWNS[playerid].x,STATIC_PLAYER_SPAWNS[playerid].y,STATIC_PLAYER_SPAWNS[playerid].z)
	SetPedPos(ped, STATIC_PLAYER_SPAWNS[playerid].x,STATIC_PLAYER_SPAWNS[playerid].y,STATIC_PLAYER_SPAWNS[playerid].z)
	SetPedAngle(ped, STATIC_PLAYER_SPAWNS[playerid].a)
	SetPedModel(ped, STATIC_PLAYER_SPAWNS[playerid].model)
	SetPedSkin(ped, STATIC_PLAYER_SPAWNS[playerid].skin)
	WriteInLog(string.format("Player id %d struct %X is spawned.\n", playerid, ped))
	return true
end

function OnPlayerDead(ped)
	--STAGEC.RemoveAllDialogs()
	STAGEC.ShowDialog(1005, 1)
	if (missionstate == 1) then
		OnPlayerMissionFailed(ped, current_missionid)
	end
	WriteInLog(string.format("Player id %d struct %X is died.\n", GetPedID(ped), ped))	
	return true
end

function OnPlayerMissionComplited(ped, missionid)
	--STAGEC.RemoveAllDialogs()
	STAGEC.ShowDialog(1007, 1)
	mission_pool[missionid].target.name = "MISSION_DONE"
	mission_pool[missionid].mission(ped, mission_pool[missionid].target)
	total_mission_stat[missionid] = 3
	DeleteMission(ped, missionid)
	EnableMissionPhones(ped)
	return true
end

function OnPlayerMissionFailed(ped, missionid)
	--STAGEC.RemoveAllDialogs()
	STAGEC.ShowDialog(1006, 1)
	total_mission_stat[missionid] = 2
	DeleteMission(ped, missionid)
	EnableMissionPhones(ped)
	return true
end

while(GetGameFrame() == 0) do end

--local sstart, send = find(tostring(GetSCRName()), '/', 1, true)
if (string.match(GetSCRName(), "[/|\\](.*)") ~= "STAGEC.SCR" ) then -- Load Stage Constructor
    SetSCRName("data/STAGEC.SCR")
end

while (true) do	
	wait(0)
	if (GetGameFrame() > 0) then
		if (gamestate == 0) then -- If start new game		
			for i = 1, 6, 1 do
				ped = GetPedStruct(i)
				--WriteInLog(string.format("ped index %d id %d struct %X\n",i,GetPedID(ped),ped))
				ped_pool[i] = ped
			end			
			--WriteInLog(string.format("MATH %s, %s, '%s' model %d\n", GetSCRName(), string.match(GetGMPName(), "[/|\\](.*)"), string.match(GetSCRName(), "/(.*)"), ReadProcessMemory(GetSCROffset() + 0x3594, 2)  ))
			if (#ped_pool > 0) then 
				gamestate = 1
			end
		else
			for i,ped in ipairs(ped_pool) do			  
				if (not IsPedDefined(ped_pool[1])) then 
					gamestate = 0
					break
				end
				if (ped ~= 0) then	
					-- Game state
					if (gamestate == 1 and GetPedHealth(ped) > 0) then
						OnNewGame(ped)
					end
					-- Game time 
					if (os.clock() * 1000 - gametime_tmp >= game_minute_ms) then						
						game_m = game_m + 1
						if (game_m >= 60) then
							game_m = 0
							game_h = game_h + 1
							if (game_h >= 24) then
								game_h = 0	
							end								
							STAGEC.SetAmbientLevel(stagelight[game_h], 30)
							SetMaxPedDensity(ped_density[game_h])
						end
						--WriteInLog(string.format("GetGameTime() %02d:%02d light %.01f ped_density %d\n", game_h, game_m, stagelight[game_h], GetMaxPedDensity()))												
						gametime_tmp = os.clock() * 1000 + game_minute_ms
					end				
					-- Cheak player health
					if (gamestate == 1 and GetPedHealth(ped) <= 0 or GetPedTask(ped) == 9) then
						--WriteInLog(string.format("OnPlayerDead() 1 index %d hp %s OK\n", i, GetPedHealth(ped)))
						SetPedTask(ped, 9)
						OnPlayerDead(ped)
						while (GetPedTask(ped) == 9) do 
							--WriteInLog(string.format("OnPlayerDead() 2 index %d task %d OK\n", i, GetPedTask(ped)))
						end
						OnPlayerSpawn(ped)
					end
					-- Cheak control
					if (gamestate == 2) then
						OnPlayerInput(ped)	
					end
					-- Stage state
					if (validstage == 1) then -- If stage is WIL.GMP or WIL-MULTI.GMP					
						-- Cheaks	
						if (co_phones ~= nil) then
							if (coroutine.status(co_phones) == "suspended") then
								coroutine.resume(co_phones, ped)						
							else
								co_phones = coroutine.create(CheakPhones, ped)
							end
						end						
						if (co_parked_cars ~= nil) then
							if (coroutine.status(co_parked_cars) == "suspended") then
								coroutine.resume(co_parked_cars, ped)						
							else
								co_parked_cars = coroutine.create(CheakParkedCars, ped)
							end
						end						
						if (co_pickups ~= nil) then
							if (coroutine.status(co_pickups) == "suspended") then
								coroutine.resume(co_pickups, ped)						
							else
								co_pickups = coroutine.create(CheakPickups, ped)
							end
						end						
						if (co_create_explosive ~= nil) then						
							if (coroutine.status(co_create_explosive) == "suspended") then
								coroutine.resume(co_create_explosive)
							end
						end
						-- Cheak Missions
						if (gamestate == 2 and missionstate == 1) then	
							state, mission_pool[current_missionid].target = mission_pool[current_missionid].mission(ped, mission_pool[current_missionid].target)
							--WriteInLog(string.format("Mission state %s target %s OK\n", tostring(state), tostring(mission_pool[current_missionid].target) ))
						end
					end -- Valid stage EOF
				end
			end
		end
	else
		if (gamestate == 2) then
			ClearStage()
		end
		gamestate = 0 -- If not frame to game restart or game not loaded
	end
end