--[[
-- STAGEC (Stage Constructor) methods for STAGEC.SCR
-- Since: v0.4.2
--
-- I have been thinking a lot about how to create new objects in GTA2 at the level without the need to compile the * .MIS script.
-- Comrade Sector (with http://gtamp.com) prompted a useful address in the game memory where all the declared objects from the current *.SCR are downloaded.
-- Through this address, you can access the level memory and dynamically change all the indicators of objects in real time.
-- "I decided to use it." Created a set of methods in the classical GBHScript, compiled them into STAGEC.SCR.
-- The result is a "level designer" that contains the compiled functions that we access with MISI, changing the value of counters.
-- In order to access the methods in STAGEC, you need to load this script together with the *.GMP level (basic missions of course are not included in it).
-- After you have a completely clean level to create your missions in the LUA.
-- This is an experimental way in connection with the lack of a drug, although if you think about the MISI project is an experiment.
-- Note: all MISI methods work the same with new objects and with the theme that are obtained through the structure
-- the player so you can apply them to the created objects.
]]--
local STAGEC ={MAX_ARROWS = 0}

--[[
-- Objects 16
0x2EF0 CHAR_DATA ped
0x2F14 CHAR_DATA targetped
0x2F38 CAR_DATA car
0x2F58 CAR_DATA targetcar
0x2F78 OBJ_DATA obj
0x2F94 OBJ_DATA targetobj
0x2FB0 LIGHT lt
0x2FBC LIGHT targetlt
0x2FC8 SOUND sd
0x2FE4 SOUND targetsd
0x3000 ARROW_DATA arrow
0x300C ARROW_DATA targetarrow
0x0000 MAP_ZONE zone
0x0000 MAP_ZONE targetzone
0x3028 TIMER_DATA timer
0x3034 TIMER_DATA targettimer
-- Counters 67
0x3108 true = 1
]]--

function STAGEC.CreatePed(x, y, z, rotation, skin, occupation)
	local ped_struct = 0
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 256) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x3450, math.floor(x * 16384), 4)
		WriteProcessMemory(scr_offset + 0x3454, math.floor(y * 16384), 4)
		WriteProcessMemory(scr_offset + 0x3458, math.floor(z * 16384), 4)
		WriteProcessMemory(scr_offset + 0x345C, rotation, 2)
		WriteProcessMemory(scr_offset + 0x345E, skin, 2)
		WriteProcessMemory(scr_offset + 0x3460, occupation, 2)
		WriteProcessMemory(scr_offset + 0x312C, 1, 4)
		wait(50)
		ped_struct = ReadProcessMemory(scr_offset + 0x2EF0, 4)
	end
	return ped_struct
end

function STAGEC.CreatePedInCar(car, skin, occupation)
	local ped_struct = 0
	if (car ~= 0) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x2F38, car, 4)
		WriteProcessMemory(scr_offset + 0x3488, skin, 2)	
		WriteProcessMemory(scr_offset + 0x348A, occupation, 2)
		WriteProcessMemory(scr_offset + 0x312C, 2, 4)
		wait(50)
		ped_struct = ReadProcessMemory(scr_offset + 0x2EF0, 4)
	end
	return ped_struct
end

function STAGEC.CreateCar(x, y, z, rotation, model, trailermodel, color, mode)
	local car_struct = 0
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 256) then
		local scr_offset = GetSCROffset()
		if (mode == 1) then
			WriteProcessMemory(scr_offset + 0x34CC, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x34D0, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x34D4, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x34D8, rotation, 2)
			WriteProcessMemory(scr_offset + 0x34DA, color, 2)
			WriteProcessMemory(scr_offset + 0x34DC, model, 2)
			WriteProcessMemory(scr_offset + 0x34DE, trailermodel, 2)
			WriteProcessMemory(scr_offset + 0x3168, 1, 4)
		elseif (mode == 2) then
			WriteProcessMemory(scr_offset + 0x3504, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3508, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x350C, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3510, rotation, 2)
			WriteProcessMemory(scr_offset + 0x3512, color, 2)
			WriteProcessMemory(scr_offset + 0x3514, model, 2)
			WriteProcessMemory(scr_offset + 0x3516, trailermodel, 2)
			WriteProcessMemory(scr_offset + 0x3168, 2, 4)
		end
		wait(50)
		car_struct = ReadProcessMemory(scr_offset + 0x2F38, 4)
	end
	return car_struct
end

function STAGEC.CreateObj(x, y, z, rotation, model, value, mode)
	local obj_struct = 0
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 256) then
		local scr_offset = GetSCROffset()
		if (mode == 1) then
			WriteProcessMemory(scr_offset + 0x3554, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3558, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x355C, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3560, model, 2)
			WriteProcessMemory(scr_offset + 0x3562, rotation, 2)
			WriteProcessMemory(scr_offset + 0x31A4, 1, 4)	
		elseif (mode == 2) then
			WriteProcessMemory(scr_offset + 0x3588, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x358C, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3590, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3594, model, 2)
			WriteProcessMemory(scr_offset + 0x3596, rotation, 2)
			WriteProcessMemory(scr_offset + 0x3598, value, 2)
			WriteProcessMemory(scr_offset + 0x31A4, 2, 4)
		end
		wait(100)
		obj_struct = ReadProcessMemory(scr_offset + 0x2F78, 4)			
	end
	return obj_struct
end

function STAGEC.CreateLight(x, y, z, range, intensity, red, green, blue, time_on, time_off, time_random)
	local light_struct = 0
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x35D8, math.floor(x * 16384), 4)
		WriteProcessMemory(scr_offset + 0x35DC, math.floor(y * 16384), 4)
		WriteProcessMemory(scr_offset + 0x35E0, math.floor(z * 16384), 4)
		WriteProcessMemory(scr_offset + 0x35E6, red, 1)
		WriteProcessMemory(scr_offset + 0x35E5, green, 1)
		WriteProcessMemory(scr_offset + 0x35E4, blue, 1)
		WriteProcessMemory(scr_offset + 0x35E8, math.floor(range * 16384), 4)
		WriteProcessMemory(scr_offset + 0x35EC, intensity, 1)	
		WriteProcessMemory(scr_offset + 0x35ED, time_on, 1)
		WriteProcessMemory(scr_offset + 0x35EE, time_off, 1)
		WriteProcessMemory(scr_offset + 0x35EF, time_random, 1)
		WriteProcessMemory(scr_offset + 0x31E0, 1, 4)
		wait(50)
		light_struct = ReadProcessMemory(scr_offset + 0x2FB0, 4)
	end
	return light_struct
end

function STAGEC.CreateSound(x, y, z, soundid, playmode)
	local sound_struct = 0
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x3620, math.floor(x * 16384), 4)
		WriteProcessMemory(scr_offset + 0x3624, math.floor(y * 16384), 4)
		WriteProcessMemory(scr_offset + 0x3628, math.floor(z * 16384), 4)
		WriteProcessMemory(scr_offset + 0x362C, soundid, 1)
		WriteProcessMemory(scr_offset + 0x362D, playmode, 1)
		WriteProcessMemory(scr_offset + 0x321C, 1, 4)
		wait(50)
		sound_struct = ReadProcessMemory(scr_offset + 0x2FC8, 4)			
	end
	return sound_struct
end

function STAGEC.CreateBlock(x, y, z)
	local block = {}
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
		local scr_offset = GetSCROffset()
		block.x = math.floor(x)
		block.y = math.floor(y)
		block.z = math.floor(z)
		WriteProcessMemory(scr_offset + 0x365C, block.x, 1)
		WriteProcessMemory(scr_offset + 0x365D, block.y, 1)
		WriteProcessMemory(scr_offset + 0x365E, block.z, 1)
		WriteProcessMemory(scr_offset + 0x327C, 1, 4)
		wait(50)
	end
	return block
end

function STAGEC.CreateArrow(x, y, z, model, color)
	local arrow_struct = 0
	if (STAGEC.MAX_ARROWS < 17) then
		if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
			local scr_offset = GetSCROffset()
			--WriteProcessMemory(scr_offset + 0x3000, 0, 4) -- new arrow struct
			--wait(50)
			WriteProcessMemory(scr_offset + 0x3690, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3694, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3698, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3258, 1, 4)
			wait(50)
			arrow_struct = ReadProcessMemory(scr_offset + 0x3000, 4)
			WriteProcessMemory(arrow_struct + 0x20, color, 2)
			WriteProcessMemory(arrow_struct + 0x24, model, 2)
			WriteProcessMemory(scr_offset + 0x3000, 0, 4) -- new arrow struct
			STAGEC.MAX_ARROWS = STAGEC.MAX_ARROWS + 1
			--wait(50)
		end
	end
	return arrow_struct
end

function STAGEC.SetBlockType(block, field_type, slope_type)
	if (block ~= nil) then
		if (block.x > 0 and block.x < 256 and block.y > 0 and block.y < 256 and block.z > 0 and block.z < 8) then
			local scr_offset = GetSCROffset()
			local flags = 0
			-- AIR
			--if (field_type == 0) then 
			-- ROAD
			if (field_type == 1) then
				flags = flags + 0x1
			-- PAVEMENT
			elseif (field_type == 2) then
				flags = flags + 0x2
			-- FIEALD
			elseif (field_type == 3) then
				flags = flags + 0x3
			end
			-- SLOPE
			if (slope_type > 0) then
				flags = flags + slope_type -- Need search all (0 to 63) bits for this command
			end	
			WriteProcessMemory(scr_offset + 0x39D8, math.floor(block.x), 1)
			WriteProcessMemory(scr_offset + 0x39D9, math.floor(block.y), 1)
			WriteProcessMemory(scr_offset + 0x39DA, math.floor(block.z), 1)
			-- GROUND 6 (not used in basic arguments)
			--WriteProcessMemory(scr_offset + 0x39DB, 6, 1) 
			WriteProcessMemory(scr_offset + 0x39DC, flags, 1)
			WriteProcessMemory(scr_offset + 0x3288, 1, 4)
			block.field_type = field_type
			block.slope_type = slope_type
			wait(50)
		end
	end
end

function STAGEC.SetBlockLID(block, flat_type, flip_type, light, rotation, tileid)
	if (block ~= nil) then
		if (block.x > 0 and block.x < 256 and block.y > 0 and block.y < 256 and block.z > 0 and block.z < 8) then
			local scr_offset = GetSCROffset()	
			local flags = 0
			if (flat_type == 1) then
				flags = flags + 0x10
			end
			if (flip_type == 1) then
				flags = flags + 0x20
			end
			if (light == 1) then
				flags = flags + 0x4
			elseif (light == 2) then
				flags = flags + 0x8
			elseif (light == 3) then
				flags = flags + 0xC
			end
			--if (rotation >= 0 and rotation < 90) then
			if (rotation >= 90 and rotation < 180) then	
				flags = flags + 0x40
			elseif (rotation >= 180 and rotation < 270) then	
				flags = flags + 0x80
			elseif (rotation >= 270 and rotation < 360) then
				flags = flags + 0xC0
			end
			WriteProcessMemory(scr_offset + 0x3A00, math.floor(block.x), 1)
			WriteProcessMemory(scr_offset + 0x3A01, math.floor(block.y), 1)
			WriteProcessMemory(scr_offset + 0x3A02, math.floor(block.z), 1)
			-- UNGROUND? 5 (not used in basic arguments)
			--WriteProcessMemory(scr_offset + 0x3A03, 5, 1)
			flags = flags + math.floor(tileid / 256)
			WriteProcessMemory(scr_offset + 0x3A04, tileid % 256, 1)
			WriteProcessMemory(scr_offset + 0x3A05, flags, 1)
			WriteProcessMemory(scr_offset + 0x3288, 2, 4)	
			block.flat_type = flat_type
			block.flip_type = flip_type		
			block.light = light
			block.rotation = rotation
			block.tileid = tileid
			wait(50)
		end
	end
end

function STAGEC.SetBlockSide(block, face_type, wall_type, bullet_type, flat_type, flip_type, rotation, tileid)
	if (block ~= nil) then
		if (block.x > 0 and block.x < 256 and block.y > 0 and block.y < 256 and block.z > 0 and block.z < 8) then
			local scr_offset = GetSCROffset()
			local flags = 0
			if (wall_type == 1) then
				flags = flags + 0x4
			end
			if (bullet_type == 1) then
				flags = flags + 0x8
			end
			if (flat_type == 1) then
				flags = flags + 0x10
			end
			if (flip_type == 1) then
				flags = flags + 0x20
			end
			--if (rotation >= 0 and rotation < 90) then
			if (rotation >= 90 and rotation < 180) then	
				flags = flags + 0x40
			elseif (rotation >= 180 and rotation < 270) then	
				flags = flags + 0x80
			elseif (rotation >= 270 and rotation < 360) then
				flags = flags + 0xC0
			end
			flags = flags + math.floor(tileid / 256)
			WriteProcessMemory(scr_offset + 0x3A28, math.floor(block.x), 1)
			WriteProcessMemory(scr_offset + 0x3A29, math.floor(block.y), 1)
			WriteProcessMemory(scr_offset + 0x3A2A, math.floor(block.z), 1)
			WriteProcessMemory(scr_offset + 0x3A2B, face_type, 1)
			WriteProcessMemory(scr_offset + 0x3A2C, tileid % 256, 1)
			WriteProcessMemory(scr_offset + 0x3A2D, flags, 1)
			WriteProcessMemory(scr_offset + 0x3288, 3, 4)
			block.face_type = face_type
			block.wall_type = wall_type
			block.bullet_type = bullet_type
			block.flat_type = flat_type
			block.flip_type = flip_type		
			block.light = light
			block.rotation = rotation
			block.tileid = tileid
			wait(50)
		end
	end
end

function STAGEC.SetLowerArea(minx, miny, maxx, maxy)
	if (minx > 0 and maxx < 256 and miny > 0 and maxy < 256) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x3A50, math.floor(minx), 1)
		WriteProcessMemory(scr_offset + 0x3A51, math.floor(miny), 1)
		WriteProcessMemory(scr_offset + 0x3A52, math.floor(maxx), 1)
		WriteProcessMemory(scr_offset + 0x3A53, math.floor(maxy), 1)
		WriteProcessMemory(scr_offset + 0x3288, 4, 4)
		wait(50)
	end
end
		
function STAGEC.SetPedPhoneState(ped, phone, cyclesring, mode)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			if (mode == 0) then
				WriteProcessMemory(scr_offset + 0x2F94, phone, 4)
				WriteProcessMemory(scr_offset + 0x3318, 2, 4)
				wait(50)
			elseif (mode == 1) then
				WriteProcessMemory(scr_offset + 0x2F78, phone, 4)
				WriteProcessMemory(scr_offset + 0x3318, 2, 4) -- Clear ring
				wait(20)
				WriteProcessMemory(scr_offset + 0x43A8, cyclesring, 2) -- Default 40 cycles
				WriteProcessMemory(scr_offset + 0x3318, 1, 4)
				wait(50)
			end
		end
	end
end

function STAGEC.SetZoneDensity(zone, car_density, car_good, car_bad, car_police, ped_density, ped_murgger, ped_thefter, ped_elvis, ped_gang, ped_police)
	if (zone ~= 0) then
		local scr_offset = GetSCROffset()	
		WriteProcessMemory(scr_offset + 0x42E0, car_density, 2)
		WriteProcessMemory(scr_offset + 0x42F0, car_good, 2)
		WriteProcessMemory(scr_offset + 0x4300, car_bad, 2)
		WriteProcessMemory(scr_offset + 0x4310, car_police, 2)
		WriteProcessMemory(scr_offset + 0x4320, ped_density, 2)
		WriteProcessMemory(scr_offset + 0x4330, ped_murgger, 2)
		WriteProcessMemory(scr_offset + 0x4340, ped_thefter, 2)
		WriteProcessMemory(scr_offset + 0x4350, ped_elvis, 2)
		WriteProcessMemory(scr_offset + 0x4360, ped_gang, 2)
		WriteProcessMemory(scr_offset + 0x4370, ped_police, 2)
		WriteProcessMemory(scr_offset + 0x4204, zone, 4)
		WriteProcessMemory(scr_offset + 0x3330, 1, 4)
		wait(50)
	end
end

function STAGEC.ShowDialog(dialogid, mode)
	if (dialogid > 0 and dialogid <= 65535) then
		local scr_offset = GetSCROffset()
		if (mode == 1) then
			WriteProcessMemory(scr_offset + 0x3818, dialogid, 2)
			WriteProcessMemory(scr_offset + 0x32DC, mode, 4)
		elseif (mode == 2) then
			WriteProcessMemory(scr_offset + 0x383C, dialogid, 2)
			WriteProcessMemory(scr_offset + 0x32DC, mode, 4)
		elseif (mode == 3) then
			WriteProcessMemory(scr_offset + 0x3860, dialogid, 2)
			WriteProcessMemory(scr_offset + 0x32DC, mode, 4)
		elseif (mode == 4) then
			WriteProcessMemory(scr_offset + 0x3884, dialogid, 2)
			WriteProcessMemory(scr_offset + 0x32DC, mode, 4)
		elseif (mode == 5) then
			WriteProcessMemory(scr_offset + 0x38AA, dialogid, 2)
			WriteProcessMemory(scr_offset + 0x32DC, mode, 4)
		end
		wait(50)
	end
end

function STAGEC.RemoveAllDialogs()
	local scr_offset = GetSCROffset()
	WriteProcessMemory(scr_offset + 0x32F4, 1, 4)
	wait(50)
end

function STAGEC.SetShadingLevel(value)
	if (value >= 0 and value <= 31) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x3972, value, 2)
		WriteProcessMemory(scr_offset + 0x3354, 1, 4)
		wait(50)
	end
end

function STAGEC.SetAmbientLevel(value, timeupdate)
	if (value >= 0 and value <= 1) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x393C, math.floor(value * 16384), 4)
		WriteProcessMemory(scr_offset + 0x3940, timeupdate, 2)
		WriteProcessMemory(scr_offset + 0x3348, 1, 4)
		wait(50)
	end
end

function STAGEC.SetTVDir(x, y)
	if (x > 0 and x < 256 and y > 0 and y < 256) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x39A0, math.floor(x * 16384), 4)
		WriteProcessMemory(scr_offset + 0x39A4, math.floor(y * 16384), 4)
		WriteProcessMemory(scr_offset + 0x333C, 1, 4)
		wait(50)
	end
end

function STAGEC.CreateExplode(x, y, z, mode, face_type)
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
		local scr_offset = GetSCROffset()
		if (mode == 1) then
			WriteProcessMemory(scr_offset + 0x36CC, math.floor(x * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x36D0, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x36D4, math.floor(z * 16384), 4)
		elseif (mode == 2) then
			WriteProcessMemory(scr_offset + 0x36F8, math.floor(x * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x36FC, math.floor(y * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x3700, math.floor(z * 16384), 4) 	
		elseif (mode == 3) then
			WriteProcessMemory(scr_offset + 0x3724, math.floor(x * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x3728, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x372C, math.floor(z * 16384), 4) 		
		elseif (mode == 4) then
			WriteProcessMemory(scr_offset + 0x3750, math.floor(x * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x3754, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3758, math.floor(z * 16384), 4) 	
		elseif (mode == 5) then
			WriteProcessMemory(scr_offset + 0x377C, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3780, math.floor(y * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x3784, math.floor(z * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x3788, face_type, 1) 		
		end
		WriteProcessMemory(scr_offset + 0x3300, mode, 2) 
		wait(50)
	end
end

function STAGEC.CreatePedGroup(ped_leader, maxmembers)
	if (ped_leader ~= nil) then
		if (ped_leader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped_leader, 4)
			WriteProcessMemory(scr_offset + 0x3D6E, maxmembers, 2)
			WriteProcessMemory(scr_offset + 0x3138, 1, 4)
			wait(50)
			return ped_leader
		end
	end
end

--[[
function STAGEC.SetPedThreatSearch(ped, threatsearch_type)
	if (ped ~= 0) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
		WriteProcessMemory(scr_offset + 0x3D9E, threatsearch_type, 2)
		WriteProcessMemory(scr_offset + 0x3138, 2, 4)	
		wait(50)
	end
end

function STAGEC.SetPedThreatReaction(ped, threatreaction_type)
	if (ped ~= 0) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
		WriteProcessMemory(scr_offset + 0x3DCE, threatreaction_type, 2)
		WriteProcessMemory(scr_offset + 0x3138, 3, 4)
		wait(50)
	end
end
]]--

function STAGEC.SetPedObjective(ped, obj, target, objective_type, mode)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			if (mode == 1) then
				WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
				WriteProcessMemory(scr_offset + 0x3DFE, objective_type, 2)
				WriteProcessMemory(scr_offset + 0x3138, 4, 4)
				wait(50)
			elseif (mode == 2 and obj ~= 0 and target ~= nil) then
				local flags = 0
				if (objective_type == 35) then
					flags = target -- seat id: 0 - driver, 1 - passenger, 2,3 - back passengers
					--objective_type = 35
					WriteProcessMemory(scr_offset + 0x3E32, flags, 2)
				end
				WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
				WriteProcessMemory(scr_offset + 0x3E2E, objective_type, 2)
				WriteProcessMemory(scr_offset + 0x2F78, obj, 4)
				WriteProcessMemory(scr_offset + 0x3138, 5, 4)
				wait(50)
			elseif (mode == 3 and target ~= nil) then
				if (target.x >= 0 and target.x <= 255 and target.y >= 0 and target.y <= 255 and target.z >= 0 and target.z <= 255) then
					WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
					WriteProcessMemory(scr_offset + 0x3E62, objective_type, 2)
					WriteProcessMemory(scr_offset + 0x3E64, math.floor(target.x * 16384), 4) 
					WriteProcessMemory(scr_offset + 0x3E68, math.floor(target.y * 16384), 4)
					WriteProcessMemory(scr_offset + 0x3E6C, math.floor(target.z * 16384), 4)
					WriteProcessMemory(scr_offset + 0x3138, 6, 4)
					wait(50)
				end
			end
		end 
	end
end

function STAGEC.SedPedGroupLeader(ped, ped_oldleader)
	if (ped ~= nil and ped_oldleader ~= nil) then
		if (ped ~= 0 and ped_oldleader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4) -- ped
			WriteProcessMemory(scr_offset + 0x2F14, ped_oldleader, 4) -- targetped
			WriteProcessMemory(scr_offset + 0x3138, 7, 4)
			wait(50)
			return ped
		end
	end	
end

function STAGEC.SetPedNoting(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3138, 8, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedStayInCar(ped, state)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3EFE, state, 1)
			WriteProcessMemory(scr_offset + 0x3138, 9, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedUseCarWeapon(ped, state)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3F2E, state, 1)
			WriteProcessMemory(scr_offset + 0x3138, 10, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedStopDriving(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3138, 11, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedExitBus(ped, state)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3F8E, state, 1)
			WriteProcessMemory(scr_offset + 0x3138, 12, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedDriveCar(ped, car)
	if (ped ~= nil and car ~= nil) then
		if (ped ~= 0 and car ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x2F38, car, 4)
			WriteProcessMemory(scr_offset + 0x3138, 13, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedExitCar(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3138, 14, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedRemoteControl(ped, car)
	if (ped ~= nil and car ~= nil) then
		if (ped ~= 0 and car ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x2F38, car, 4)
			WriteProcessMemory(scr_offset + 0x3138, 15, 4)
			wait(50)
		end 
	end
end

function STAGEC.RemovePedFromCar(ped, x, y, z, rotation)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x4050, math.floor(x * 16384), 4) 
			WriteProcessMemory(scr_offset + 0x4054, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x4058, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x404E, rotation, 2)
			WriteProcessMemory(scr_offset + 0x3138, 16, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedPatrolPoint(ped, x, y, z)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x408C, math.floor(x * 16384), 4)
			WriteProcessMemory(scr_offset + 0x4090, math.floor(y * 16384), 4)
			WriteProcessMemory(scr_offset + 0x4094, math.floor(z * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3138, 17, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedSkill(ped, skill_type, value)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			if (skill_type == 1) then
				WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
				WriteProcessMemory(scr_offset + 0x40C6, value, 2)
				WriteProcessMemory(scr_offset + 0x3138, 18, 4)
				WaitFrame(1)
			elseif (skill_type == 2) then
				WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
				WriteProcessMemory(scr_offset + 0x40F6, value, 2)
				WriteProcessMemory(scr_offset + 0x3138, 19, 4)
				wait(50)
			end
		end 
	end
end

function STAGEC.AddPedInGroup(ped, ped_leader)
	if (ped ~= nil and ped_leader ~= nil) then
		if (ped ~= 0 and ped_leader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped_leader, 4) -- ped
			WriteProcessMemory(scr_offset + 0x2F14, ped, 4) -- targetped
			WriteProcessMemory(scr_offset + 0x3138, 20, 4)
			wait(50)
			return ped
		end
	end	
end

function STAGEC.SetPedGroupMinMembers(ped_leader, minmembers)
	if (ped_leader ~= nil) then
		if (ped_leader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped_leader, 4)
			WriteProcessMemory(scr_offset + 0x39F8, minmembers, 2) 
			WriteProcessMemory(scr_offset + 0x3138, 21, 4)
			wait(50)
			return ped_leader
		end
	end
end

function STAGEC.SetPedGroupType(ped_leader, group_type)
	if (ped_leader ~= nil and group_type ~= nil) then
		if (ped_leader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped_leader, 4)
			WriteProcessMemory(scr_offset + 0x4188, group_type, 2) 
			WriteProcessMemory(scr_offset + 0x3138, 22, 4)
			wait(50)
			return ped_leader
		end
	end
end

function STAGEC.RemovePedFromGroup(ped, ped_leader)
	if (ped ~= nil and ped_leader ~= nil) then
		if (ped ~= 0 and ped_leader ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped_leader, 4) -- ped
			WriteProcessMemory(scr_offset + 0x2F14, ped, 4) -- targetped
			WriteProcessMemory(scr_offset + 0x3138, 23, 4)
			wait(50)
		end 
	end
end

function STAGEC.GivePedWeapon(ped, weaponid, ammo)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x41EA, weaponid, 2) 
			WriteProcessMemory(scr_offset + 0x41EC, ammo, 2)
			WriteProcessMemory(scr_offset + 0x3138, 24, 4)
			wait(50)
		end 
	end
end

function STAGEC.RemovePedAllWeapons(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3138, 25, 4)
			wait(50)
		end 
	end
end

function STAGEC.KillPed(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x3138, 26, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedDriveAggression(ped, state)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x43DA, state, 1)
			WriteProcessMemory(scr_offset + 0x3138, 27, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedMaxDriveSpeed(ped, speed)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x440C, math.floor(speed * 16384), 4)
			WriteProcessMemory(scr_offset + 0x3138, 28, 4)
			wait(50)
		end 
	end
end

function STAGEC.SetPedFavoriteCarModel(ped, model)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2EF0, ped, 4)
			WriteProcessMemory(scr_offset + 0x4440, model, 2)
			WriteProcessMemory(scr_offset + 0x3138, 29, 4)
			wait(50)
		end 
	end
end

--[[
function STAGEC.SetGangInfo(x, y, z, gangid, skin, arrow_icon, weapons, kills, car_model, car_remap)
	if (x > 0 and x < 256 and y > 0 and y < 256 and z > 0 and z < 8) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x4435, gangid, 1)
		WriteProcessMemory(scr_offset + 0x4436, skin, 1)
		WriteProcessMemory(scr_offset + 0x4437, weapons[1], 1)
		WriteProcessMemory(scr_offset + 0x4438, weapons[2], 1)
		WriteProcessMemory(scr_offset + 0x4439, weapons[3], 1)
		WriteProcessMemory(scr_offset + 0x443A, arrow_icon, 1)
		WriteProcessMemory(scr_offset + 0x443B, kills, 1)
		WriteProcessMemory(scr_offset + 0x443C, math.floor(x * 16384), 4)
		WriteProcessMemory(scr_offset + 0x4440, math.floor(y * 16384), 4)
		WriteProcessMemory(scr_offset + 0x4444, math.floor(z * 16384), 4)
		WriteProcessMemory(scr_offset + 0x4448, car_model, 2)
		WriteProcessMemory(scr_offset + 0x444A, car_remap, 2)
		WriteProcessMemory(scr_offset + 0x3138, 27, 4)
		wait(50)
	end 
end
]]--

function STAGEC.SetCarPutTrailer(car, trailer)
	if (car ~= nil and trailer ~= nil) then
		if (car ~= 0 and trailer ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F38, car, 4)
			WriteProcessMemory(scr_offset + 0x2F58, trailer, 4)
			WriteProcessMemory(scr_offset + 0x3174, 1, 4)
			wait(50)
		end 
	end
end

function STAGEC.KillCarPassengers(car)
	if (car ~= nil) then
		if (car ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F38, car, 4)
			WriteProcessMemory(scr_offset + 0x3174, 2, 4)
			wait(50)
		end 
	end
end
		
function STAGEC.CreateTimer(timesec)
	local scr_offset = GetSCROffset()
	local timer = {}
	timer.time = timesec
	timer.time_tmp = os.clock() + timesec
	WriteProcessMemory(scr_offset + 0x37EA, timesec, 2)
	WriteProcessMemory(scr_offset + 0x32AC, 1, 4)
	wait(50)
	timer.id = ReadProcessMemory(scr_offset + 0x3028, 2)
	return timer
end

function STAGEC.AddTimerTime(timer, timesec)
	if (timer ~= nil) then
		if (timer.id >= 0 and timer.id <= 3) then
			local scr_offset = GetSCROffset()
			timer.time = timer.time + timesec
			timer.time_tmp = timer.time_tmp + timesec
			WriteProcessMemory(scr_offset + 0x390C, timesec, 2)
			WriteProcessMemory(scr_offset + 0x32B8, 1, 4)
			wait(50)
		end
	end
end

function STAGEC.IsTimerOut(timer)
	if (timer ~= nil and timer.time_tmp ~= nil) then
		if (os.clock() >= timer.time_tmp) then
			--WriteInLog("time lost "..os.clock()-timer.time_tmp.. " > "..timer.time)
			return true
		end
	end
	return false
end

function STAGEC.DeleteTimer(timer)
	if (timer ~= nil) then
		if (timer.id >= 0 and timer.id <= 3) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x3034, timer.id, 2)
			WriteProcessMemory(scr_offset + 0x32C4, 2, 4)
			timer = nil
			wait(50)
		end
	end
end
					
function STAGEC.DeletePed(ped)
	if (ped ~= nil) then
		if (ped ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F14, ped, 4)
			WriteProcessMemory(scr_offset + 0x3144, 1, 4)
			wait(50)
		end
	end
end

function STAGEC.RemoveCar(car)
	if (car ~= nil) then
		if (car ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F58, car, 4)
			WriteProcessMemory(scr_offset + 0x3180, 1, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteCar(car)
	if (car ~= nil) then
		if (car ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F58, car, 4)
			WriteProcessMemory(scr_offset + 0x3180, 2, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteObj(obj)
	if (obj ~= nil) then
		if (obj ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F94, obj, 4)
			WriteProcessMemory(scr_offset + 0x31BC, 1, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteForceObj(obj)
	if (obj ~= nil) then
		if (obj ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F94, obj, 4)
			WriteProcessMemory(scr_offset + 0x31BC, 2, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteLight(light)
	if (light ~= nil) then
		if (light ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2FBC, light, 4)
			--WriteProcessMemory(scr_offset + 0x31C8, light, 4)
			WriteProcessMemory(scr_offset + 0x31F8, 1, 4)
			--WriteProcessMemory(scr_offset + 0x31C8, 0, 4) -- Clear counter
			--wait(20)
			--WriteProcessMemory(scr_offset + 0x2FBC, 255, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteSound(sound)
	if (sound ~= nil) then
		if (sound ~= 0) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x2F94, sound, 4)
			WriteProcessMemory(scr_offset + 0x31BC, 1, 4)
			wait(50)
		end
	end
end

function STAGEC.DeleteBlock(block, mode)
	if (block ~= nil) then -- ?
		if (block.x ~= nil and block.y ~= nil and block.z ~= nil) then
			local scr_offset = GetSCROffset()
			WriteProcessMemory(scr_offset + 0x3C80, math.floor(block.x), 1)
			WriteProcessMemory(scr_offset + 0x3C81, math.floor(block.y), 1)
			WriteProcessMemory(scr_offset + 0x3C82, math.floor(block.z), 1)
			WriteProcessMemory(scr_offset + 0x3C83, mode, 1)
			WriteProcessMemory(scr_offset + 0x3294, 1, 4)
			block = nil
			wait(50)
		end
	end
end

function STAGEC.DeleteArrow(arrow)
	if (arrow ~= nil) then
		if (arrow ~= 0) then
			local scr_offset = GetSCROffset()
			--WriteProcessMemory(scr_offset + 0x3240, arrow, 4)
			--WriteProcessMemory(scr_offset + 0x3000, arrow, 4)
			WriteProcessMemory(scr_offset + 0x3000, arrow, 4)		
			WriteProcessMemory(scr_offset + 0x3270, 1, 4)
			if (STAGEC.MAX_ARROWS > 0) then
				STAGEC.MAX_ARROWS = STAGEC.MAX_ARROWS - 1
			end
			wait(50)
		end
	end
end

function STAGEC.SaveGame()
	if (GetGameFrame() > 0) then
		local scr_offset = GetSCROffset()
		WriteProcessMemory(scr_offset + 0x3360, 1, 4)
		wait(50)
	end
end

return STAGEC