/*
* Public Domain
*
* This MISI library (hereinafter "application") is distributed by
* license (Public Domain) free software "as is"
* without any guarantees.
*
* This application (hereinafter "project") is
* public domain of the GTAMP community.
* All source codes, logos, texts, project names are open,
* are the combined work of people united by a common idea and a dream.
*
* Everyone can use this project without restrictions
* at its discretion:
* Compile, modify, correct, supplement, update, sell,
* distribute under its own brand and mark.
*
* http://gtamp.com/forum/viewtopic.php?f=4&t=1136#p10506
*/
#include "stdafx.h"
#include "gta2_address.h"
#include <stdio.h>
#include <iostream>
#include <process.h>
#include "misimethods.h"
#include "appmethods.h"
#include <process.h>
#include <corecrt_wstdio.h>

extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}

#pragma warning(disable:4996)

using namespace std;
namespace fs = std::experimental::filesystem;

HMODULE head_module;
HANDLE head_process_handle;
HANDLE head_thread_handle;
DWORD head_threadid;
LPDWORD head_thread_exit_code = 0;
DWORD head_pricessid;
int misi_state;
int log_size = MIN_LOG_SIZE;
FILE *flog;
lua_State *head_lua_state;
int max_threads = 0;

typedef struct key_struct {
	lua_State *lua_state;
	int keys[2];
	int akeys[256];
} key_struct;

typedef struct thread_params_t {
	lua_State *luastate;
	char *lua_path;
	int index;
} thread_params_t;
HANDLE child_thread_handle[MAX_SCRIPTS];

struct lua_t {
	char *lua_path;
	lua_State *luastate;
	int keys[256];
	int upkeys[256];
};
lua_t lua[MAX_SCRIPTS];

void ChildThread(void *pParams) {
	thread_params_t *params = (thread_params_t*)pParams;
#ifdef LOG_LOWLVL_ENABLED
	writeLog(LOG_FILENAME, "a", true, "** Thread index (%d) started...\n", params->index);
	//cout << "[" << "_" << "] " << "** Thread index (" << params->index << ") started..." << endl;
	//fprintf(flog, "[%s] ** Thread index (%d) started...\n", GetCurrTime(), params->index);
#endif
	lua_State *lua_state = luaL_newstate();  //lua_newthread((lua_State *)params->luastate);
	if (lua_state == NULL) {
		writeLog(LOG_FILENAME, "a", true, "ERROR in LUA state: %s\n", lua_tostring(lua_state, 1));
		//DllExit(0, head_module);
		_endthread();
	}
	int index = params->index;
	lua[index].lua_path = (char *)malloc(MAX_PATH);
	strncpy(lua[index].lua_path, params->lua_path, MAX_PATH);
	lua[index].luastate = (lua_State *)malloc(sizeof(lua_State *));
	lua[index].luastate = lua_state;
	writeLog(LOG_FILENAME, "a", true, "* LUA Script (%d) %s loading...\n", params->index, lua[index].lua_path);
	static const luaL_Reg lualibs[] =
	{
		{ "base", luaopen_base },
		{ "coroutine", luaopen_coroutine },
		{ "debug", luaopen_debug },
		{ "io", luaopen_io },
		{ "math", luaopen_math },
		{ "os", luaopen_os },
		{ "package", luaopen_package },
		{ "string", luaopen_string },
		{ "table", luaopen_table },
		{ "utf8", luaopen_utf8 },
		//{ "bit32",luaopen_bit32 },
		{ NULL, NULL }
	};
	for (const luaL_Reg *lib = lualibs; lib->func != NULL; lib++)
	{
		luaL_requiref(lua_state, lib->name, lib->func, 1);
		lua_settop(lua_state, 0);
	}
	// System method
	lua_register(lua_state, "GetGameHandle", l_GetGameHandle);
	lua_register(lua_state, "wait", l_wait);
	lua_register(lua_state, "ReadProcessMemory", l_ReadProcessMemory);
	lua_register(lua_state, "WriteProcessMemory", l_WriteProcessMemory);
	lua_register(lua_state, "WriteInLog", l_WriteInLog);
	lua_register(lua_state, "GetMISIBuild", l_GetMISIBuild);
	lua_register(lua_state, "GetLogSize", l_GetLogSize);
	lua_register(lua_state, "SetLogSize", l_SetLogSize);
	lua_register(lua_state, "IsReadPointer", l_IsReadPointer);
	lua_register(lua_state, "IsWritePointer", l_IsWritePointer);

	// Input method
	lua_register(lua_state, "IsGameKeyPress", l_IsGameKeyPress);
	lua_register(lua_state, "GetKeyState", l_GetKeyState);
	lua_register(lua_state, "IsKeyPress", l_IsKeyPress);
	lua_register(lua_state, "IsKeyDown", l_IsKeyDown);
	lua_register(lua_state, "IsKeyUp", l_IsKeyUp);
	lua_register(lua_state, "GetXInputConnectedDevicesNumber", l_GetXInputConnectedDevicesNumber);
	lua_register(lua_state, "IsXInputKeyPress", l_IsXInputKeyPress);
	lua_register(lua_state, "GetXInputAxes", l_GetXInputAxes);
	lua_register(lua_state, "SendInput", l_SendInput);
	lua_register(lua_state, "SendMenuInput", l_SendMenuInput);

	// Math method
	//lua_register(lua_state, "GetDistanceOfPoints", l_GetDistanceOfPoints);
	//lua_register(lua_state, "IsArea", l_IsArea);

	// Game method
	lua_register(lua_state, "GetGameFrame", l_GetGameFrame);
	lua_register(lua_state, "GetNetwork", l_GetNetwork);
	lua_register(lua_state, "GetSCRName", l_GetSCRName);
	lua_register(lua_state, "GetMMPName", l_GetMMPName);
	lua_register(lua_state, "GetSTYName", l_GetSTYName);
	lua_register(lua_state, "GetGMPName", l_GetGMPName);
	lua_register(lua_state, "GetGXTName", l_GetGXTName);
	lua_register(lua_state, "GetSCROffset", l_GetSCROffset);
	lua_register(lua_state, "GetMaxPedDensity", l_GetMaxPedDensity);
	lua_register(lua_state, "GetShowPedAndCarID", l_GetShowPedAndCarID);
	lua_register(lua_state, "GetShowObjID", l_GetShowObjID);
	lua_register(lua_state, "GetShowBlood", l_GetShowBlood);
	lua_register(lua_state, "GetSoundMessage", l_GetSoundMessage);
	lua_register(lua_state, "GetPoliceSkip", l_GetPoliceSkip);
	lua_register(lua_state, "GetMedicSkip", l_GetMedicSkip);
	lua_register(lua_state, "GetFiremanSkip", l_GetFiremanSkip);
	lua_register(lua_state, "GetRiotMode", l_GetRiotMode);
	lua_register(lua_state, "GetMaxPlayers", l_GetMaxPlayers);
	lua_register(lua_state, "GetPlayerFrags", l_GetPlayerFrags);
	lua_register(lua_state, "GetPlayerKills", l_GetPlayerKills);
	lua_register(lua_state, "GetPlayerName", l_GetPlayerName);

	lua_register(lua_state, "SetSCRName", l_SetSCRName);
	lua_register(lua_state, "SetMMPName", l_SetMMPName);
	lua_register(lua_state, "SetSTYName", l_SetSTYName);
	lua_register(lua_state, "SetGMPName", l_SetGMPName);
	lua_register(lua_state, "SetMaxPedDensity", l_SetMaxPedDensity);
	lua_register(lua_state, "SetShowPedAndCarID", l_SetShowPedAndCarID);
	lua_register(lua_state, "SetShowObjID", l_SetShowObjID);
	lua_register(lua_state, "SetShowBlood", l_SetShowBlood);
	lua_register(lua_state, "SetSoundMessage", l_SetSoundMessage);
	lua_register(lua_state, "SetPoliceSkip", l_SetPoliceSkip);
	lua_register(lua_state, "SetMedicSkip", l_SetMedicSkip);
	lua_register(lua_state, "SetFiremanSkip", l_SetFiremanSkip);
	lua_register(lua_state, "SetRiotMode", l_SetRiotMode);
	lua_register(lua_state, "SetPlayerFrags", l_SetPlayerFrags);
	lua_register(lua_state, "SetPlayerKills", l_SetPlayerKills);
	lua_register(lua_state, "SetPlayerName", l_SetPlayerName);

	// Ped method
	lua_register(lua_state, "GetPedID", l_GetPedID);
	lua_register(lua_state, "GetPedStruct", l_GetPedStruct);
	lua_register(lua_state, "GetPedPlayer", l_GetPedPlayer);
	//lua_register(lua_state, "GetPedStructOfID", l_GetPedStructOfID);
	//lua_register(lua_state, "GetPedSprite", l_GetPedSprite);
	lua_register(lua_state, "GetPedCar", l_GetPedCar);
	lua_register(lua_state, "GetPedPos", l_GetPedPos);
	lua_register(lua_state, "GetPedAnyPos", l_GetPedAnyPos);
	//lua_register(lua_state, "GetPedRangeOfPoint", l_GetPedRangeOfPoint);
	lua_register(lua_state, "GetPedHealth", l_GetPedHealth);
	lua_register(lua_state, "GetPedAngle", l_GetPedAngle);
	lua_register(lua_state, "GetPedAnyAngle", l_GetPedAnyAngle);
	lua_register(lua_state, "GetPedMaxSpeed", l_GetPedMaxSpeed);
	lua_register(lua_state, "GetPedSpeed", l_GetPedSpeed);
	lua_register(lua_state, "GetPedSkin", l_GetPedSkin);
	lua_register(lua_state, "GetPedModel", l_GetPedModel);
	lua_register(lua_state, "GetPedWanted", l_GetPedWanted);
	lua_register(lua_state, "GetPedOccupation", l_GetPedOccupation);
	lua_register(lua_state, "GetPedThreatReaction", l_GetPedThreatReaction);
	lua_register(lua_state, "GetPedThreatSearch", l_GetPedThreatSearch);
	lua_register(lua_state, "GetPedInvisible", l_GetPedInvisible);
	//lua_register(lua_state, "GetPedSpecMode", l_GetPedSpecMode);
	lua_register(lua_state, "GetPedTask", l_GetPedTask);
	lua_register(lua_state, "GetPedCurrentWeaponID", l_GetPedCurrentWeaponID);
	lua_register(lua_state, "GetPedCurrentAmmo", l_GetPedCurrentAmmo);
	lua_register(lua_state, "GetPedTouch", l_GetPedTouch);
	lua_register(lua_state, "GetPedFromPoint", l_GetPedFromPoint);
	lua_register(lua_state, "GetPedCamPos", l_GetPedCamPos);
	//lua_register(lua_state, "GetPedPool", l_GetPedPool);
	lua_register(lua_state, "GetPedCurrentCarDoorID", l_GetPedCurrentCarDoorID);

	lua_register(lua_state, "SetPedID", l_SetPedID);
	lua_register(lua_state, "SetPedPos", l_SetPedPos);
	lua_register(lua_state, "SetPedAnyPos", l_SetPedAnyPos);
	lua_register(lua_state, "SetPedHealth", l_SetPedHealth);
	lua_register(lua_state, "SetPedAngle", l_SetPedAngle);
	lua_register(lua_state, "SetPedAnyAngle", l_SetPedAnyAngle);
	lua_register(lua_state, "SetPedMaxSpeed", l_SetPedMaxSpeed);
	lua_register(lua_state, "SetPedSpeed", l_SetPedSpeed);
	lua_register(lua_state, "SetPedSkin", l_SetPedSkin);
	lua_register(lua_state, "SetPedModel", l_SetPedModel);
	lua_register(lua_state, "SetPedWanted", l_SetPedWanted);
	lua_register(lua_state, "SetPedOccupation", l_SetPedOccupation);
	lua_register(lua_state, "SetPedThreatReaction", l_SetPedThreatReaction);
	lua_register(lua_state, "SetPedThreatSearch", l_SetPedThreatSearch);
	lua_register(lua_state, "SetPedInvisible", l_SetPedInvisible);
	lua_register(lua_state, "SetPedTask", l_SetPedTask);
	lua_register(lua_state, "SetPedCurrentWeaponID", l_SetPedCurrentWeaponID);
	lua_register(lua_state, "SetPedCurrentAmmo", l_SetPedCurrentAmmo);
	//lua_register(lua_state, "SetPedWeapon", l_SetPedWeapon);
	lua_register(lua_state, "SetPedCamPos", l_SetPedCamPos);
	lua_register(lua_state, "SetPedCurrentCarDoorID", l_SetPedCurrentCarDoorID);

	lua_register(lua_state, "IsPedDefined", l_IsPedDefined);
	lua_register(lua_state, "IsPedRangeOfPoint", l_IsPedRangeOfPoint);
	lua_register(lua_state, "IsPedAnyRangeOfPoint", l_IsPedAnyRangeOfPoint);
	lua_register(lua_state, "IsPedInAnyCar", l_IsPedInAnyCar);
	lua_register(lua_state, "IsPedInCar", l_IsPedInCar);
	lua_register(lua_state, "DeletePedSprite", l_DeletePedSprite);
	lua_register(lua_state, "EnablePedSecondaryAngle", l_EnablePedSecondaryAngle);
	lua_register(lua_state, "DisablePedSecondaryAngle", l_DisablePedSecondaryAngle);
	lua_register(lua_state, "SetPedSecondaryAngle", l_SetPedSecondaryAngle);
	lua_register(lua_state, "GetPedSecondaryAngle", l_GetPedSecondaryAngle);

	// Player method

	//lua_register(lua_state, "GetPlayerStruct", l_GetPlayerStruct);
	lua_register(lua_state, "GetPlayerArmour", l_GetPlayerArmour);
	lua_register(lua_state, "GetPlayerGameCam", l_GetPlayerGameCam);
	lua_register(lua_state, "GetPlayerViewCam", l_GetPlayerViewCam);
	lua_register(lua_state, "GetPlayerAuxCam", l_GetPlayerAuxCam);
	lua_register(lua_state, "GetPlayerWeaponSlot", l_GetPlayerWeaponSlot);

	lua_register(lua_state, "GetPlayerMoney", l_GetPlayerMoney);
	lua_register(lua_state, "SetPlayerArmour", l_SetPlayerArmour);
	lua_register(lua_state, "SetPlayerGameCam", l_SetPlayerGameCam);
	lua_register(lua_state, "SetPlayerViewCam", l_SetPlayerViewCam);
	lua_register(lua_state, "SetPlayerAuxCam", l_SetPlayerAuxCam);
	lua_register(lua_state, "SetPlayerWeaponSlot", l_SetPlayerWeaponSlot);
	lua_register(lua_state, "SetPlayerMoney", l_SetPlayerMoney);

	// Car method
	lua_register(lua_state, "GetCarID", l_GetCarID);
	lua_register(lua_state, "GetCarPos", l_GetCarPos);
	lua_register(lua_state, "GetCarAngle", l_GetCarAngle);
	lua_register(lua_state, "GetCarDamage", l_GetCarDamage);
	lua_register(lua_state, "GetCarModel", l_GetCarModel);
	lua_register(lua_state, "GetCarColor", l_GetCarColor);
	lua_register(lua_state, "GetCarEngine", l_GetCarEngine);
	lua_register(lua_state, "GetCarDoorLock", l_GetCarDoorLock);
	lua_register(lua_state, "GetCarEmergySiren", l_GetCarEmergySiren);
	lua_register(lua_state, "GetCarHorn", l_GetCarHorn);
	lua_register(lua_state, "GetCarDirection", l_GetCarDirection);
	lua_register(lua_state, "GetCarSpeed", l_GetCarSpeed);
	lua_register(lua_state, "GetCarArmour", l_GetCarArmour);
	lua_register(lua_state, "GetCarFromPoint", l_GetCarFromPoint);
	//lua_register(lua_state, "GetCarPool", l_GetCarPool);

	lua_register(lua_state, "SetCarPos", l_SetCarPos);
	lua_register(lua_state, "SetCarAngle", l_SetCarAngle);
	lua_register(lua_state, "SetCarDamage", l_SetCarDamage);
	lua_register(lua_state, "SetCarModel", l_SetCarModel);
	lua_register(lua_state, "SetCarColor", l_SetCarColor);
	lua_register(lua_state, "SetCarEngine", l_SetCarEngine);
	lua_register(lua_state, "SetCarDoorLock", l_SetCarDoorLock);
	lua_register(lua_state, "SetCarEmergySiren", l_SetCarEmergySiren);
	lua_register(lua_state, "SetCarHorn", l_SetCarHorn);
	lua_register(lua_state, "SetCarDirection", l_SetCarDirection);
	lua_register(lua_state, "SetCarArmour", l_SetCarArmour);

	lua_register(lua_state, "IsCarRangeOfPoint", l_IsCarRangeOfPoint);

	// Object methods
	lua_register(lua_state, "GetObjID", l_GetObjID);
	lua_register(lua_state, "GetObjPos", l_GetObjPos);
	lua_register(lua_state, "GetObjAngle", l_GetObjAngle);
	lua_register(lua_state, "GetObjModel", l_GetObjModel);

	lua_register(lua_state, "SetObjPos", l_SetObjPos);

	// Light methods
	lua_register(lua_state, "GetLightPos", l_GetLightPos);
	lua_register(lua_state, "GetLightIntensity", l_GetLightIntensity);
	lua_register(lua_state, "GetLightRange", l_GetLightRange);
	lua_register(lua_state, "GetLightColor", l_GetLightColor);
	lua_register(lua_state, "GetLightTime", l_GetLightTime);

	lua_register(lua_state, "SetLightPos", l_SetLightPos);
	lua_register(lua_state, "SetLightIntensity", l_SetLightIntensity);
	lua_register(lua_state, "SetLightRange", l_SetLightRange);
	lua_register(lua_state, "SetLightColor", l_SetLightColor);
	lua_register(lua_state, "SetLightTime", l_SetLightTime);

	// Arrow methods
	lua_register(lua_state, "GetArrowPos", l_GetArrowPos);
	lua_register(lua_state, "GetArrowColor", l_GetArrowColor);
	lua_register(lua_state, "GetArrowIcon", l_GetArrowIcon);

	lua_register(lua_state, "SetArrowPos", l_SetArrowPos);
	lua_register(lua_state, "SetArrowColor", l_SetArrowColor);
	lua_register(lua_state, "SetArrowIcon", l_SetArrowIcon);

	// Sound methods
	lua_register(lua_state, "GetSoundPos", l_GetSoundPos);
	lua_register(lua_state, "GetSoundID", l_GetSoundID);

	lua_register(lua_state, "SetSoundPos", l_SetSoundPos);
	lua_register(lua_state, "SetSoundID", l_SetSoundID);

	int stat = luaL_dofile(lua_state, lua[index].lua_path);
	//int stat = luaL_loadfile(lua_state, lua[index].lua_path);
	//int state_ref = luaL_ref(lua_state, LUA_REGISTRYINDEX);
	//lua_rawgeti(lua_state, LUA_REGISTRYINDEX, state_ref);
	//lua_call(lua_state, 0, 0);
	if (stat != LUA_OK) {
		// Выводим сообщение об ошибке в LUA
		//cerr << "[" << "_" << "] " << "ERROR in LUA script: " << (char *)lua_tostring(lua_state, 1) << endl;
		writeLog(LOG_FILENAME, "a", true, "ERROR in LUA script: %s\n", lua_tostring(lua_state, 1));
		//cout << "[" << "_" << "] " << "ERROR in LUA script: " << lua_tostring(lua_state, 1) << endl;
		//fprintf(flog, "[%s] ERROR in LUA script: %s\n", GetCurrTime(), lua_tostring(lua_state, 1));
		//lua_pop(lua_state, 1);
	}
	else {
		//cout << "[" << "_" << "] " << "* LUA Script index (" << params->index << ") " << lua[index].lua_path << " EOF." << endl;
		writeLog(LOG_FILENAME, "a", true, "* LUA Script (%d) %s eof.\n", params->index, lua[index].lua_path);
		//cout << "[" << "_" << "] " << "* LUA Script index (" << params->index << ") " << lua[index].lua_path << " end." << endl;
		//fprintf(flog, "[%s] * LUA Script index (%d) %s end.\n", GetCurrTime(), params->index, lua[index].lua_path);
	}
	lua_gc(lua_state, LUA_GCCOLLECT, 0);
	lua_close(lua_state);
	int state_ref = luaL_ref(lua_state, LUA_REGISTRYINDEX);
	luaL_unref(lua_state, LUA_REGISTRYINDEX, state_ref);
	//state_ref = LUA_REFNIL;
	writeLog(LOG_FILENAME, "a", true, "* LUA Script (%d) %s unloaded.\n", params->index, lua[index].lua_path);
#ifdef LOG_LOWLVL_ENABLED
	writeLog(LOG_FILENAME, "a", true, "** Thread index (%d) end.\n", params->index);
	//cout << "[" << "_" << "] " << "** Thread index (" << params->index << ") end." << endl;
#endif
	_endthread();
}

void MainThread(void* pParams) {
#ifdef LOG_LOWLVL_ENABLED
	writeLog(LOG_FILENAME, "a", true, "*** Main thread started...\n");
	//
	//cout << "[" << "_" << "] " << "*** Main thread started..." << endl;
	//fprintf(flog, "[%s] *** Main thread started...\n", GetCurrTime());
#endif
	/*
	head_lua_state = luaL_newstate(); 
	if (head_lua_state == NULL) {
		writeLog(LOG_FILENAME, "a", true, "ERROR in LUA state: %s\n", lua_tostring(head_lua_state, 1));
		DllExit(0, head_module);
	}	
	static const luaL_Reg lualibs[] =
	{
		{ "base", luaopen_base },
		{ "coroutine", luaopen_coroutine },
		{ "debug", luaopen_debug },
		{ "io", luaopen_io },
		{ "math", luaopen_math },
		{ "os", luaopen_os },
		{ "package", luaopen_package },
		{ "string", luaopen_string },
		{ "table", luaopen_table },
		{ "utf8", luaopen_utf8 },
		//{ "bit32",luaopen_bit32 },
		{ NULL, NULL }
	};
	for (const luaL_Reg *lib = lualibs; lib->func != NULL; lib++)
	{
		luaL_requiref(head_lua_state, lib->name, lib->func, 1);
		lua_settop(head_lua_state, 0);
	}	
	// System method
	lua_register(head_lua_state, "GetGameHandle", l_GetGameHandle);
	lua_register(head_lua_state, "wait", l_wait);
	lua_register(head_lua_state, "ReadProcessMemory", l_ReadProcessMemory);
	lua_register(head_lua_state, "WriteProcessMemory", l_WriteProcessMemory);
	lua_register(head_lua_state, "WriteInLog", l_WriteInLog);
	lua_register(head_lua_state, "GetMISIBuild", l_GetMISIBuild);
	lua_register(head_lua_state, "GetLogSize", l_GetLogSize);
	lua_register(head_lua_state, "SetLogSize", l_SetLogSize);
	lua_register(head_lua_state, "IsReadPointer", l_IsReadPointer);
	lua_register(head_lua_state, "IsWritePointer", l_IsWritePointer);

	// Input method
	lua_register(head_lua_state, "IsGameKeyPress", l_IsGameKeyPress);
	lua_register(head_lua_state, "GetKeyState", l_GetKeyState);
	lua_register(head_lua_state, "IsKeyPress", l_IsKeyPress);
	lua_register(head_lua_state, "IsKeyDown", l_IsKeyDown);
	lua_register(head_lua_state, "IsKeyUp", l_IsKeyUp);

	// Math method
	//lua_register(head_lua_state, "GetDistanceOfPoints", l_GetDistanceOfPoints);
	//lua_register(head_lua_state, "IsArea", l_IsArea);

	// Game method
	lua_register(head_lua_state, "GetGameFrame", l_GetGameFrame);
	lua_register(head_lua_state, "GetNetwork", l_GetNetwork);
	lua_register(head_lua_state, "GetSCRName", l_GetSCRName);
	lua_register(head_lua_state, "GetMMPName", l_GetMMPName);
	lua_register(head_lua_state, "GetSTYName", l_GetSTYName);
	lua_register(head_lua_state, "GetGMPName", l_GetGMPName);
	lua_register(head_lua_state, "GetGXTName", l_GetGXTName);
	lua_register(head_lua_state, "GetSCROffset", l_GetSCROffset);
	lua_register(head_lua_state, "GetMaxPedDensity", l_GetMaxPedDensity);
	lua_register(head_lua_state, "GetShowPedAndCarID", l_GetShowPedAndCarID);
	lua_register(head_lua_state, "GetShowObjID", l_GetShowObjID);
	lua_register(head_lua_state, "GetShowBlood", l_GetShowBlood);
	lua_register(head_lua_state, "GetSoundMessage", l_GetSoundMessage);
	lua_register(head_lua_state, "GetPoliceSkip", l_GetPoliceSkip);
	lua_register(head_lua_state, "GetMedicSkip", l_GetMedicSkip);
	lua_register(head_lua_state, "GetFiremanSkip", l_GetFiremanSkip);
	lua_register(head_lua_state, "GetRiotMode", l_GetRiotMode);
	lua_register(head_lua_state, "GetMaxPlayers", l_GetMaxPlayers);
	lua_register(head_lua_state, "GetPlayerFrags", l_GetPlayerFrags);
	lua_register(head_lua_state, "GetPlayerKills", l_GetPlayerKills);
	lua_register(head_lua_state, "GetPlayerName", l_GetPlayerName);

	lua_register(head_lua_state, "SetSCRName", l_SetSCRName);
	lua_register(head_lua_state, "SetMMPName", l_SetMMPName);
	lua_register(head_lua_state, "SetSTYName", l_SetSTYName);
	lua_register(head_lua_state, "SetGMPName", l_SetGMPName);
	lua_register(head_lua_state, "SetMaxPedDensity", l_SetMaxPedDensity);
	lua_register(head_lua_state, "SetShowPedAndCarID", l_SetShowPedAndCarID);
	lua_register(head_lua_state, "SetShowObjID", l_SetShowObjID);
	lua_register(head_lua_state, "SetShowBlood", l_SetShowBlood);
	lua_register(head_lua_state, "SetSoundMessage", l_SetSoundMessage);
	lua_register(head_lua_state, "SetPoliceSkip", l_SetPoliceSkip);
	lua_register(head_lua_state, "SetMedicSkip", l_SetMedicSkip);
	lua_register(head_lua_state, "SetFiremanSkip", l_SetFiremanSkip);
	lua_register(head_lua_state, "SetRiotMode", l_SetRiotMode);
	lua_register(head_lua_state, "SetPlayerFrags", l_SetPlayerFrags);
	lua_register(head_lua_state, "SetPlayerKills", l_SetPlayerKills);
	lua_register(head_lua_state, "SetPlayerName", l_SetPlayerName);

	// Ped method
	lua_register(head_lua_state, "GetPedID", l_GetPedID);
	lua_register(head_lua_state, "GetPedStruct", l_GetPedStruct);
	lua_register(head_lua_state, "GetPedPlayer", l_GetPedPlayer);
	//lua_register(head_lua_state, "GetPedStructOfID", l_GetPedStructOfID);
	//lua_register(head_lua_state, "GetPedSprite", l_GetPedSprite);
	lua_register(head_lua_state, "GetPedCar", l_GetPedCar);
	lua_register(head_lua_state, "GetPedPos", l_GetPedPos);
	lua_register(head_lua_state, "GetPedRangeOfPoint", l_GetPedRangeOfPoint);
	lua_register(head_lua_state, "GetPedHealth", l_GetPedHealth);
	lua_register(head_lua_state, "GetPedAngle", l_GetPedAngle);
	lua_register(head_lua_state, "GetPedMaxSpeed", l_GetPedMaxSpeed);
	lua_register(head_lua_state, "GetPedSpeed", l_GetPedSpeed);
	lua_register(head_lua_state, "GetPedSkin", l_GetPedSkin);
	lua_register(head_lua_state, "GetPedModel", l_GetPedModel);
	lua_register(head_lua_state, "GetPedWanted", l_GetPedWanted);
	lua_register(head_lua_state, "GetPedOccupation", l_GetPedOccupation);
	lua_register(head_lua_state, "GetPedThreatReaction", l_GetPedThreatReaction);
	lua_register(head_lua_state, "GetPedThreatSearch", l_GetPedThreatSearch);
	lua_register(head_lua_state, "GetPedInvisible", l_GetPedInvisible);
	//lua_register(head_lua_state, "GetPedSpecMode", l_GetPedSpecMode);
	lua_register(head_lua_state, "GetPedTask", l_GetPedTask);
	lua_register(head_lua_state, "GetPedCurrentWeaponID", l_GetPedCurrentWeaponID);
	lua_register(head_lua_state, "GetPedCurrentAmmo", l_GetPedCurrentAmmo);
	lua_register(head_lua_state, "GetPedTouch", l_GetPedTouch);
	lua_register(head_lua_state, "GetPedFromPoint", l_GetPedFromPoint);
	lua_register(head_lua_state, "GetPedCamPos", l_GetPedCamPos);
	//lua_register(head_lua_state, "GetPedPool", l_GetPedPool);
	lua_register(head_lua_state, "GetPedCurrentCarDoorID", l_GetPedCurrentCarDoorID);

	lua_register(head_lua_state, "SetPedID", l_SetPedID);
	lua_register(head_lua_state, "SetPedPos", l_SetPedPos);
	lua_register(head_lua_state, "SetPedHealth", l_SetPedHealth);
	lua_register(head_lua_state, "SetPedAngle", l_SetPedAngle);
	lua_register(head_lua_state, "SetPedMaxSpeed", l_SetPedMaxSpeed);
	lua_register(head_lua_state, "SetPedSpeed", l_SetPedSpeed);
	lua_register(head_lua_state, "SetPedSkin", l_SetPedSkin);
	lua_register(head_lua_state, "SetPedModel", l_SetPedModel);
	lua_register(head_lua_state, "SetPedWanted", l_SetPedWanted);
	lua_register(head_lua_state, "SetPedOccupation", l_SetPedOccupation);
	lua_register(head_lua_state, "SetPedThreatReaction", l_SetPedThreatReaction);
	lua_register(head_lua_state, "SetPedThreatSearch", l_SetPedThreatSearch);
	lua_register(head_lua_state, "SetPedInvisible", l_SetPedInvisible);
	lua_register(head_lua_state, "SetPedTask", l_SetPedTask);
	lua_register(head_lua_state, "SetPedCurrentWeaponID", l_SetPedCurrentWeaponID);
	lua_register(head_lua_state, "SetPedCurrentAmmo", l_SetPedCurrentAmmo);
	//lua_register(head_lua_state, "SetPedWeapon", l_SetPedWeapon);
	lua_register(head_lua_state, "SetPedCamPos", l_SetPedCamPos);
	lua_register(head_lua_state, "SetPedCurrentCarDoorID", l_SetPedCurrentCarDoorID);
	lua_register(head_lua_state, "IsPedDefined", l_IsPedDefined);
	lua_register(head_lua_state, "DeletePedSprite", l_DeletePedSprite);

	// Player method

	//lua_register(head_lua_state, "GetPlayerStruct", l_GetPlayerStruct);
	lua_register(head_lua_state, "GetPlayerArmour", l_GetPlayerArmour);
	lua_register(head_lua_state, "GetPlayerGameCam", l_GetPlayerGameCam);
	lua_register(head_lua_state, "GetPlayerViewCam", l_GetPlayerViewCam);
	lua_register(head_lua_state, "GetPlayerAuxCam", l_GetPlayerAuxCam);
	lua_register(head_lua_state, "GetPlayerWeaponSlot", l_GetPlayerWeaponSlot);

	lua_register(head_lua_state, "GetPlayerMoney", l_GetPlayerMoney);
	lua_register(head_lua_state, "SetPlayerArmour", l_SetPlayerArmour);
	lua_register(head_lua_state, "SetPlayerGameCam", l_SetPlayerGameCam);
	lua_register(head_lua_state, "SetPlayerViewCam", l_SetPlayerViewCam);
	lua_register(head_lua_state, "SetPlayerAuxCam", l_SetPlayerAuxCam);
	lua_register(head_lua_state, "SetPlayerWeaponSlot", l_SetPlayerWeaponSlot);
	lua_register(head_lua_state, "SetPlayerMoney", l_SetPlayerMoney);
	
	// Car method
	lua_register(head_lua_state, "GetCarID", l_GetCarID);
	lua_register(head_lua_state, "GetCarPos", l_GetCarPos);
	lua_register(head_lua_state, "GetCarAngle", l_GetCarAngle);
	lua_register(head_lua_state, "GetCarDamage", l_GetCarDamage);
	lua_register(head_lua_state, "GetCarModel", l_GetCarModel);
	lua_register(head_lua_state, "GetCarColor", l_GetCarColor);
	lua_register(head_lua_state, "GetCarEngine", l_GetCarEngine);
	lua_register(head_lua_state, "GetCarDoorLock", l_GetCarDoorLock);
	lua_register(head_lua_state, "GetCarEmergySiren", l_GetCarEmergySiren);
	lua_register(head_lua_state, "GetCarHorn", l_GetCarHorn);
	lua_register(head_lua_state, "GetCarDirection", l_GetCarDirection);
	lua_register(head_lua_state, "GetCarSpeed", l_GetCarSpeed);
	lua_register(head_lua_state, "GetCarArmour", l_GetCarArmour);
	lua_register(head_lua_state, "GetCarFromPoint", l_GetCarFromPoint);
	//lua_register(head_lua_state, "GetCarPool", l_GetCarPool);

	lua_register(head_lua_state, "SetCarPos", l_SetCarPos);
	lua_register(head_lua_state, "SetCarAngle", l_SetCarAngle);
	lua_register(head_lua_state, "SetCarDamage", l_SetCarDamage);
	lua_register(head_lua_state, "SetCarModel", l_SetCarModel);
	lua_register(head_lua_state, "SetCarColor", l_SetCarColor);
	lua_register(head_lua_state, "SetCarEngine", l_SetCarEngine);
	lua_register(head_lua_state, "SetCarDoorLock", l_SetCarDoorLock);
	lua_register(head_lua_state, "SetCarEmergySiren", l_SetCarEmergySiren);
	lua_register(head_lua_state, "SetCarHorn", l_SetCarHorn);
	lua_register(head_lua_state, "SetCarDirection", l_SetCarDirection);
	lua_register(head_lua_state, "SetCarArmour", l_SetCarArmour);

	// Object methods
	lua_register(head_lua_state, "GetObjPos", l_GetObjPos);
	lua_register(head_lua_state, "GetObjAngle", l_GetObjAngle);
	lua_register(head_lua_state, "GetObjModel", l_GetObjModel);
	//lua_register(head_lua_state, "GetObjID", l_GetObjID);

	lua_register(head_lua_state, "SetObjPos", l_SetObjPos);

	// Light methods
	lua_register(head_lua_state, "GetLightPos", l_GetLightPos);
	lua_register(head_lua_state, "GetLightIntensity", l_GetLightIntensity);
	lua_register(head_lua_state, "GetLightRange", l_GetLightRange);
	lua_register(head_lua_state, "GetLightColor", l_GetLightColor);
	lua_register(head_lua_state, "GetLightTime", l_GetLightTime);

	lua_register(head_lua_state, "SetLightPos", l_SetLightPos);
	lua_register(head_lua_state, "SetLightIntensity", l_SetLightIntensity);
	lua_register(head_lua_state, "SetLightRange", l_SetLightRange);
	lua_register(head_lua_state, "SetLightColor", l_SetLightColor);
	lua_register(head_lua_state, "SetLightTime", l_SetLightTime);

	// Arrow methods
	lua_register(head_lua_state, "GetArrowPos", l_GetArrowPos);
	lua_register(head_lua_state, "GetArrowColor", l_GetArrowColor);
	lua_register(head_lua_state, "GetArrowIcon", l_GetArrowIcon);

	lua_register(head_lua_state, "SetArrowPos", l_SetArrowPos);
	lua_register(head_lua_state, "SetArrowColor", l_SetArrowColor);
	lua_register(head_lua_state, "SetArrowIcon", l_SetArrowIcon);

	// Sound methods
	lua_register(head_lua_state, "GetSoundPos", l_GetSoundPos);
	lua_register(head_lua_state, "GetSoundID", l_GetSoundID);

	lua_register(head_lua_state, "SetSoundPos", l_SetSoundPos);
	lua_register(head_lua_state, "SetSoundID", l_SetSoundID);	
	*/
	//lua_pushcfunction(head_lua_state, (lua_CFunction)l_OnKeyState);
	//lua_setglobal(head_lua_state, "OnKeyState");
	fs::path p = GetProcessPath(0);
	p += "\\";
	p += "scripts";
	std::string path(p.generic_string());
	const char MAX_FORMATFILES = 2;
	const char aexp[MAX_FORMATFILES][14] = { ".lua", ".TXT" };
	int index = 0;
	for (auto& p : fs::recursive_directory_iterator(path)) {
		index = 0;
		while (index < MAX_FORMATFILES) {
			std::string str = p.path().extension().generic_string();
			if (!strcmp(strupr((char *)str.c_str()), strupr((char *)aexp[index]))) {
				thread_params_t *params = new thread_params_t;
				params->luastate = head_lua_state;
				params->lua_path = (char *)malloc(MAX_PATH);
				if (params->lua_path) {
					strncpy(params->lua_path, p.path().generic_string().c_str(), MAX_PATH);
					params->index = max_threads;
					child_thread_handle[max_threads] = (HANDLE)_beginthread(ChildThread, 0, (void *)params);
					SuspendThread(child_thread_handle[max_threads]);
					max_threads++;
				}
			}
			index++;
		}
	}
	writeLog(LOG_FILENAME, "a", true, "Total scripts loaded: %d of %d\n", max_threads, MAX_SCRIPTS);
	for (int i = 0; i < max_threads; i++) {		
		ResumeThread(child_thread_handle[i]);		
		Sleep(50);
	}
	LPDWORD thread_exit_code;
	do {
		GetExitCodeThread(head_thread_handle, (LPDWORD)&thread_exit_code);
		Sleep(0);
	} while (thread_exit_code == (LPDWORD)STILL_ACTIVE);
	//WaitForSingleObject(child_thread_handle[i], INFINITE);
	//WaitForMultipleObjects(max_threads, child_thread_handle, TRUE, INFINITE);
#ifdef LOG_LOWLVL_ENABLED
	writeLog(LOG_FILENAME, "a", true, "*** Main thread stoped.\n");
	//cout << "[" << "_" << "] " << "*** Main thread stoped." << endl;
	//fprintf(flog, "[%s] *** Main thread stoped.\n", GetCurrTime());
	//fclose(stdout);
#endif
	DllExit(0, head_module);
}

void DllExit(int param, HMODULE hModule)
{
	misi_state = 0;
	//lua_close(head_lua_state);
	//for (int i = 0; i < max_threads; i++) {
	//	TerminateThread(child_thread_handle[i], 0);
	//}
	char module_name[MAX_PATH];
	GetModuleFileNameA(hModule, module_name, sizeof(module_name));
	//CloseHandle(head_process_handle);
	if (param != 1) {
		writeLog(LOG_FILENAME, "a", true, "**** MISI library: %s unloaded.\n", module_name);
	}
	#ifdef DEBUG_LOG_ENABLED
	//writeLog(LOG_FILENAME, "a", true, "DLL_EXIT\n");
	cout << "[" << GetCurrentTime() << "]" << "DLL_EXIT" << endl;
	//fclose(flog);
	#endif
	//exit(0);
	//FreeModule(hModule);
	//FreeLibraryAndExitThread(GetModuleHandleA("MISI.ASI"), 1);
}

/*
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	writeLog(LOG_FILENAME, "a", true, "**** Window.\n");
	switch (message) {
		case WM_CREATE: {
			writeLog(LOG_FILENAME, "a", true, "**** MISI library: Create Window.\n");
			break;
		}
		case WM_DESTROY: {
			writeLog(LOG_FILENAME, "a", true, "**** MISI library: Destroy Window.\n");
			break;
		}
		default: {
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
	}
	return 0;
}
*/

BOOL APIENTRY DllMain(HMODULE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) {
	switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH: {
		misi_state = 1;
		//setlocale(0, "Russian");
		head_pricessid = GetCurrentProcessId();
		if (!head_pricessid) {
			cerr << "ERROR: Current process ID not found." << endl;
			DllExit(0, hModule);
			return FALSE;
		}
		int exe_size = GetProcessSize(head_pricessid);
		int exe_offset = (int)GetModuleHandle(0);
		flog = fopen(LOG_FILENAME, "r");
		if (!flog) {
			char module_name[MAX_PATH];
			GetModuleFileNameA(hModule, module_name, sizeof(module_name));	
			writeLog(LOG_FILENAME, "a", true, "------------------------------------------------------------------\n");
			writeLog(LOG_FILENAME, "a", true, "***** %s *****\n", APP_BUILD);
			writeLog(LOG_FILENAME, "a", true, "Main thread: %s\n", APP_URL);
			writeLog(LOG_FILENAME, "a", true, "\n");
			writeLog(LOG_FILENAME, "a", true, "Game exe: offset 0x%X size %d detected version %s\n", exe_offset, exe_size, GetGameVersion(exe_offset, exe_size));
			writeLog(LOG_FILENAME, "a", true, "LUA scripts directory: %s%s\n", ExtractPath(module_name), "\\scripts");
			writeLog(LOG_FILENAME, "a", true, "------------------------------------------------------------------\n");		
		}
		else {
			fclose(flog);		
		}	
		if (GAME_OFFSET_VER_11_44 != exe_offset || GAME_SIZE_VER_11_44 != exe_size) {
			writeLog(LOG_FILENAME, "a", true, "Not supported game exe %s. MISI only supported version: %s\n", GetGameVersion(exe_offset, exe_size), GetGameVersion(GAME_OFFSET_VER_11_44, GAME_SIZE_VER_11_44));
			//cout << "[" << "_" << "] " << "Not supported game exe " << GetGameVersion(exe_offset, exe_size) << ". MISI only supported version: " << GetGameVersion(GAME_OFFSET_VER_11_44, GAME_SIZE_VER_11_44) << endl;
			DllExit(1, hModule);
			return FALSE;
		}
		//hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, head_pricessid);
		head_module = hModule;
		head_process_handle = OpenProcess(PROCESS_ALL_ACCESS, 0, head_pricessid);
		if (head_process_handle == NULL) {
			writeLog(LOG_FILENAME, "a", true, "ERROR: Current process handle not access.\n");
			DllExit(0, hModule);
			return FALSE;
		}
		size_t error_process;
		error_process = GetLastError();
		if (error_process == ERROR_ACCESS_DENIED) {
			writeLog(LOG_FILENAME, "a", true, "ERROR: Code %d access is denied.\n", error_process);
			DllExit(0, hModule);
			return FALSE;
		}
		head_thread_handle = (HANDLE)_beginthread(MainThread, 0, NULL);
		head_threadid = GetThreadId(head_thread_handle);
		char module_name[MAX_PATH];
		GetModuleFileNameA(hModule, module_name, sizeof(module_name));
		writeLog(LOG_FILENAME, "a", true, "**** MISI library: %s loaded.\n", module_name);
		#ifdef DEBUG_LOG_ENABLED
		//writeLog(LOG_FILENAME, "a", true, "DLL_PROCESS_ATTACH\n");
		cout << "[" << GetCurrentTime() << "]" << "DLL_PROCESS_ATTACH" << endl;
		#endif
		break;
	}
	case DLL_THREAD_ATTACH: {
		break;
	}
	case DLL_THREAD_DETACH: {
		break;
	}
	case DLL_PROCESS_DETACH:
		GetExitCodeThread(head_thread_handle, (LPDWORD)&head_thread_exit_code);
		//GetExitCodeProcess(head_thread_handle, (LPDWORD)&head_thread_exit_code);
		if (misi_state == 1) {
			misi_state = 2;
			#ifdef DEBUG_LOG_ENABLED
			//writeLog(LOG_FILENAME, "a", true, "DLL_PROCESS_DETACH exit code %d\n", head_thread_exit_code);
			cout << "[" << GetCurrentTime() << "]" << "DLL_PROCESS_DETACH exit code " << head_thread_exit_code << endl;
			#endif
		}
		if (misi_state != 0 && head_thread_exit_code != (LPDWORD)STILL_ACTIVE) {
#ifdef DEBUG_LOG_ENABLED
			//writeLog(LOG_FILENAME, "a", true, "head_thread_exit_code %d\n", head_thread_exit_code);
			cout << "[" << GetCurrentTime() << "]" << "head_thread_exit_code " << head_thread_exit_code << endl;
#endif
			DllExit(0, hModule);
		}
		break;
	}
	return TRUE;
}

int l_SetSCRName(lua_State *l) {
	if ((char *)lua_tostring(l, 1) == "" || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	char name[32];
	strcpy(name, (char *)lua_tostring(l, 1));
	if (WriteProcessMemory(head_process_handle, (LPVOID)SCR_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetMMPName(lua_State *l) {
	if ((char *)lua_tostring(l, 1) == "" || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	char name[32];
	strcpy(name, (char *)lua_tostring(l, 1));
	if (WriteProcessMemory(head_process_handle, (LPVOID)MMP_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetSTYName(lua_State *l) {
	if ((char *)lua_tostring(l, 1) == "" || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	char name[32];
	strcpy(name, (char *)lua_tostring(l, 1));
	if (WriteProcessMemory(head_process_handle, (LPVOID)STY_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetGMPName(lua_State *l) {
	if ((char *)lua_tostring(l, 1) == "" || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	char name[32];
	strcpy(name, (char *)lua_tostring(l, 1));
	if (WriteProcessMemory(head_process_handle, (LPVOID)GMP_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedCurrentWeaponID(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	ped_struct += 0x170;
	int ped_current_weaponid_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_current_weaponid_ptr, 4, NULL)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	int ped_weaponid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(ped_current_weaponid_ptr + 0x1C), &ped_weaponid, 2, NULL)) {
		lua_pushinteger(l, ped_weaponid);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_GetPedCurrentAmmo(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x170;
	int ped_current_ammo_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_current_ammo_ptr, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int ped_ammo = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_current_ammo_ptr, &ped_ammo, 2, NULL)) {
		ped_ammo = ped_ammo / 10;
		lua_pushnumber(l, ceil(ped_ammo));
		return 1;
	}
	lua_pushnumber(l, 0);
	return 1;
}


int l_GetPedTouch(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x138; // ped_struct + ped_occupation_ptr
	int ped_touch_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_touch_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int ped_touch_struct = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(ped_touch_ptr + 0x7C), &ped_touch_struct, 4, NULL)) {
		lua_pushinteger(l, ped_touch_struct);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPlayerWeaponSlot(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	player_struct += 0x788;
	int player_weapon_slot = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &player_weapon_slot, 2, NULL)) {
		if (player_weapon_slot == 65535) {
			player_weapon_slot = 0;
		}
		else {
			player_weapon_slot += 1;
		}
		lua_pushinteger(l, player_weapon_slot);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPlayerMoney(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	player_struct += 0x2D4;
	int player_money = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &player_money, 4, NULL)) {
		lua_pushinteger(l, player_money);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetMaxPlayers(lua_State *l) {
	int max_players_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)PLAYER_OFFSET, &max_players_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int max_players = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(max_players_ptr + 0x23), &max_players, 1, NULL)) {
		lua_pushinteger(l, max_players);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedCurrentWeaponID(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x170; // ped_struct + ped_current_ammo_ptr
	int ped_current_weaponid_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_current_weaponid_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_weaponid = (int)lua_tointeger(l, 2);
	if (ped_weaponid < 0 || ped_weaponid > 15) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_current_weaponid_ptr += 0x1C;
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_current_weaponid_ptr, &ped_weaponid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPedCurrentAmmo(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x170; // ped_struct + ped_current_ammo_ptr
	int ped_current_ammo = (int)lua_tointeger(l, 2);
	if (ped_current_ammo >= 65535) {
		ped_current_ammo = 65535;
	}
	else {
		ped_current_ammo *= 10;
	}
	int ped_current_ammo_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_current_ammo_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_current_ammo_ptr, &ped_current_ammo, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

/*
int l_SetPedWeapon(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += (int)lua_tointeger(l, 2) * 0x30; // ped_struct + ped_ammo_ptr
	int ped_ammo = (int)lua_tointeger(l, 3);
	int network = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)0x673E2C, &network, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (network > 0) { // IsNetwork
		ped_struct += (0x203B8 - 0x30);
	}
	else {
		ped_struct += (0x262C8 - 0x30);
	}
	if (ped_ammo >= 65535) {
		ped_ammo = 65535;
	}
	else {
		ped_ammo *= 10;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_ammo, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}
*/

int l_SetPlayerWeaponSlot(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	player_struct += 0x788;
	int player_weapon_slot = (int)lua_tointeger(l, 2);
	if (player_weapon_slot <= 0) {
		player_weapon_slot = 65535;
	}
	else {
		player_weapon_slot -= 1;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &player_weapon_slot, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPlayerMoney(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	player_struct += 0x2D4;
	int player_money = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &player_money, 4, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_DeletePedSprite(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x234;
	int ped_specmode = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_specmode, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

// Since v0.4.2 changed return integer to boolean
int l_IsPedDefined(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	ped_struct += 0x234;
	int ped_sprite = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite, 1, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (ped_sprite == 99) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetPedTask(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x278;
	int ped_task = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_task, 4, NULL)) {
		lua_pushinteger(l, ped_task);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedTask(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x278; // ped_struct + task_ptr
	int ped_task = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_task, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

// Math method
/*
int l_GetDistanceOfPoints(lua_State *l) {
	float x1 = (float)lua_tonumber(l, 1);
	float y1 = (float)lua_tonumber(l, 2);
	float z1 = (float)lua_tonumber(l, 3);
	float x2 = (float)lua_tonumber(l, 4);
	float y2 = (float)lua_tonumber(l, 5);
	float z2 = (float)lua_tonumber(l, 6);
	lua_pushnumber(l, (float)sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)));
	return 1;
}

int l_IsArea(lua_State *l) {
	float x = (float)lua_tonumber(l, 1);
	float y = (float)lua_tonumber(l, 2);
	float z = (float)lua_tonumber(l, 3);
	float minx = (float)lua_tonumber(l, 4);
	float miny = (float)lua_tonumber(l, 5);
	float minz = (float)lua_tonumber(l, 6);
	float maxx = (float)lua_tonumber(l, 7);
	float maxy = (float)lua_tonumber(l, 8);
	float maxz = (float)lua_tonumber(l, 9);
	if (x >= minx && x <= maxx && y >= miny && y <= maxy && z >= minz && z <= maxz) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}
*/

int l_GetSCRName(lua_State *l) {
	char name[32];
	if (ReadProcessMemory(head_process_handle, (LPVOID)SCR_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushstring(l, name);
		return 1;
	}
	lua_pushstring(l, "");
	return 1;
}

int l_GetSCROffset(lua_State *l) {
	lua_Integer scr_offset = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)SCR_OFFSET, &scr_offset, 4, NULL)) {
		lua_pushinteger(l, (scr_offset + 0x46C)); // 0x35C
		return 1;
	}
	lua_pushinteger(l, 0); 
	return 1;
}

int l_GetGXTName(lua_State *l) {
	char name[32];
	if (ReadProcessMemory(head_process_handle, (LPVOID)GXT_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushstring(l, name);
		return 1;
	}
	lua_pushstring(l, "");
	return 1;
}

int l_GetMaxPedDensity(lua_State *l) {
	int ped_density_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)PED_DENSITY, &ped_density_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int ped_density = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_density_ptr, &ped_density, 2, NULL)) {
		lua_pushinteger(l, ped_density);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetMaxPedDensity(lua_State *l) {
	int ped_density = (int)lua_tointeger(l, 1);
	if (ped_density < 0 || ped_density > 65535 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_density_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)PED_DENSITY, &ped_density_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_density_ptr, &ped_density, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetShowPedAndCarID(lua_State *l) {
	int show_id = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)SHOW_PED_CAR_ID, &show_id, 1, NULL)) {
		lua_pushinteger(l, show_id);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetShowPedAndCarID(lua_State *l) {
	int show_id = (int)lua_tointeger(l, 1);
	if (show_id < 0 || show_id > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)SHOW_PED_CAR_ID, &show_id, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetShowObjID(lua_State *l) {
	int show_objid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)SHOW_OBJ_ID, &show_objid, 1, NULL)) {
		lua_pushinteger(l, show_objid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetShowObjID(lua_State *l) {
	int show_objid = (int)lua_tointeger(l, 1);
	if (show_objid < 0 || show_objid > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)SHOW_OBJ_ID, &show_objid, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetShowBlood(lua_State *l) {
	int show_blood = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)SHOW_BLOOD, &show_blood, 1, NULL)) {
		lua_pushinteger(l, show_blood);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetShowBlood(lua_State *l) {
	int show_blood = (int)lua_tointeger(l, 1);
	if (show_blood < 0 || show_blood > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)SHOW_BLOOD, &show_blood, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPoliceSkip(lua_State *l) {
	int police_skip = (int)lua_tointeger(l, 1);
	if (police_skip < 0 || police_skip > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)POLICE_SKIP, &police_skip, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPoliceSkip(lua_State *l) {
	int police_skip = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)POLICE_SKIP, &police_skip, 1, NULL)) {
		lua_pushinteger(l, police_skip);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetMedicSkip(lua_State *l) {
	int medic_skip = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)MEDIC_SKIP, &medic_skip, 1, NULL)) {
		lua_pushinteger(l, medic_skip);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetMedicSkip(lua_State *l) {
	int medic_skip = (int)lua_tointeger(l, 1);
	if (medic_skip < 0 || medic_skip > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)MEDIC_SKIP, &medic_skip, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetFiremanSkip(lua_State *l) {
	int fireman_skip = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)FIREMAN_SKIP, &fireman_skip, 1, NULL)) {
		lua_pushinteger(l, fireman_skip);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetFiremanSkip(lua_State *l) {
	int fireman_skip = (int)lua_tointeger(l, 1);
	if (fireman_skip < 0 || fireman_skip > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)FIREMAN_SKIP, &fireman_skip, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetRiotMode(lua_State *l) {
	int riot_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)RIOT_MODE, &riot_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int riot_mode = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(riot_mode + 0x7), &riot_mode, 1, NULL)) {
		lua_pushinteger(l, riot_mode);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetRiotMode(lua_State *l) {
	int riot_mode = (int)lua_tointeger(l, 1);
	if (riot_mode < 0 || riot_mode > 1 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int riot_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)RIOT_MODE, &riot_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)(riot_mode + 0x7), &riot_mode, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetSoundMessage(lua_State *l) {
	int sound_msgid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)MESSAGE_SOUND_ADDRESS, &sound_msgid, 2, NULL)) {
		lua_pushinteger(l, sound_msgid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetSoundMessage(lua_State *l) {
	int sound_msgid = (int)lua_tointeger(l, 1);
	if (sound_msgid < 0 || sound_msgid > 62) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)MESSAGE_SOUND_ADDRESS, &sound_msgid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetMMPName(lua_State *l) {
	char name[32];
	if (ReadProcessMemory(head_process_handle, (LPVOID)MMP_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushstring(l, name);
		return 1;
	}
	lua_pushstring(l, "");
	return 1;
}

int l_GetSTYName(lua_State *l) {
	char name[32];
	if (ReadProcessMemory(head_process_handle, (LPVOID)STY_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushstring(l, name);
		return 1;
	}
	lua_pushstring(l, "");
	return 1;
}

int l_GetGMPName(lua_State *l) {
	char name[32];
	if (ReadProcessMemory(head_process_handle, (LPVOID)GMP_NAME_ADRESS, &name, 32, NULL)) {
		lua_pushstring(l, name);
		return 1;
	}
	lua_pushstring(l, "");
	return 1;
}

int l_SetLogSize(lua_State *l) {
	log_size = (int)lua_tointeger(l, 1);
	if (log_size == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, (int)log_size);
	return 1;
}

int l_GetLogSize(lua_State *l) {
	lua_pushinteger(l, (int)log_size);
	return 1;
}

int l_GetNetwork(lua_State *l) {
	int network = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)NETWORK_ADDRESS, &network, 1, NULL)) {
		lua_pushinteger(l, network);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_ReadProcessMemory(lua_State *l) {
	int value = 0;
	if (lua_tointeger(l, 1) == 0 || lua_tointeger(l, 2) == 0 || lua_isnil(l, 1) || lua_isnil(l, 2)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	if (ReadProcessMemory(head_process_handle, (LPVOID)lua_tointeger(l, 1), &value, (int)lua_tointeger(l, 2), NULL)) {
		lua_pushinteger(l, value);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_WriteProcessMemory(lua_State *l) {
	int value = (int)lua_tointeger(l, 2);
	if (lua_tointeger(l, 1) == 0 || lua_tointeger(l, 3) == 0 || lua_isnil(l, 1) || lua_isnil(l, 2) || lua_isnil(l, 3)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)lua_tointeger(l, 1), &value, (int)lua_tointeger(l, 3), NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

// Since v0.4.2 changed return integer to boolean
int l_IsReadPointer(lua_State *l) {
	if (lua_tointeger(l, 1) == 0 || lua_tointeger(l, 2) == 0 || lua_isnil(l, 1) || lua_isnil(l, 2)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (!IsBadReadPtr((LPVOID)lua_tointeger(l, 1), (int)lua_tointeger(l, 2))) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

// Since v0.4.2 changed return integer to boolean
int l_IsWritePointer(lua_State *l) {
	if (lua_tointeger(l, 1) == 0 || lua_tointeger(l, 2) == 0 || lua_isnil(l, 1) || lua_isnil(l, 2)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (!IsBadWritePtr((LPVOID)lua_tointeger(l, 1), (int)lua_tointeger(l, 2))) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetGameFrame(lua_State *l) {
	int value = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)FRAME_ADDRESS, &value, 1, NULL)) {
		lua_pushinteger(l, value);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetCarDamage(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x75;
	int car_damage = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_damage, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarModel(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x84;
	int car_model = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_model, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarColor(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x50;
	int car_sprite_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int car_color = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_ptr + 0x24), &car_color, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarEngine(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x9C;
	int car_engine_state = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_engine_state, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarDoorLock(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x98;
	int car_door_lock = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_door_lock, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarEmergySiren(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0xA4;
	int car_emergy_siren = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_emergy_siren, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetCarHorn(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0xA7;
	int car_horn = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_horn, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetCarDamage(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0x75;
	int car_damage = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_damage, 1, NULL)) {
		lua_pushinteger(l, car_damage);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetCarModel(lua_State *l) {	
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	car_struct += 0x84;
	int car_model = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_model, 1, NULL)) {
		lua_pushinteger(l, car_model);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_GetCarColor(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	car_struct += 0x50;
	int car_sprite_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int car_color = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_ptr + 0x24), &car_color, 2, NULL)) {
		lua_pushinteger(l, car_color);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_GetCarEngine(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0x9C;
	int car_engine_state = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_engine_state, 1, NULL)) {
		lua_pushinteger(l, car_engine_state);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetCarDoorLock(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0x98;
	int car_door_lock = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_door_lock, 1, NULL)) {
		lua_pushinteger(l, car_door_lock);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetCarEmergySiren(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0xA4;
	int car_emergy_siren = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_emergy_siren, 1, NULL)) {
		lua_pushinteger(l, car_emergy_siren);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetCarHorn(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0xA7;
	int car_horn = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_horn, 1, NULL)) {
		lua_pushinteger(l, car_horn);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPlayerKills(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int player_kills = 0;
	int player_kills_ptr = PLAYER_KILL;
	player_kills_ptr += ((playerid * 0x2) - 0x2);
	if (ReadProcessMemory(head_process_handle, (LPVOID)player_kills_ptr, &player_kills, 2, NULL)) {
		lua_pushinteger(l, player_kills);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPlayerKills(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int player_kills = (int)lua_tointeger(l, 2);
	int player_kills_ptr = PLAYER_KILL;
	player_kills_ptr += ((playerid * 0x2) - 0x2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_kills_ptr, &player_kills, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPlayerFrags(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int player_frags = (int)lua_tointeger(l, 2);
	int player_frags_ptr = PLAYER_FRAG;
	player_frags_ptr += ((playerid * 0x2) - 0x2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_frags_ptr, &player_frags, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPlayerFrags(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int player_frags = 0;
	int player_frags_ptr = PLAYER_FRAG;
	player_frags_ptr += ((playerid * 0x2) - 0x2);
	if (ReadProcessMemory(head_process_handle, (LPVOID)player_frags_ptr, &player_frags, 2, NULL)) {
		lua_pushinteger(l, player_frags);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPlayerName(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int player_name = 0;
	wchar_t wname[MAX_PLAYER_NAME];
	swprintf(wname, L"%S", lua_tostring(l, 2));
	player_name = PLAYER_NAME;
	player_name += ((playerid * 0x20) - 0x20);
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_name, &wname, MAX_PLAYER_NAME, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPlayerName(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid == 0 || lua_isnil(l, 1)) {
		lua_pushstring(l, "");
		return 1;
	}
	int player_name = 0;
	wchar_t wname[MAX_PLAYER_NAME];
	wname[0] = '\0';
	player_name = PLAYER_NAME;
	player_name += ((playerid * 0x20) - 0x20);
	//int wname_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_name, &wname, MAX_PLAYER_NAME, NULL)) {
		lua_pushstring(l, "");
		return 1;
	}
	//if (!ReadProcessMemory(head_process_handle, (LPVOID)wname_ptr, &wname, 32, NULL)) {
	////	lua_pushstring(l, "");
	//	return 1;
	//}
	char name[MAX_PLAYER_NAME];
	wcstombs(name, wname, MAX_PLAYER_NAME);
	//name[0] = (char)wname;
	//name = (char)wname;
	//char *n = new char[32];
	//wcstombs(n, name, 32);
	//char pname[32];
	//for (int i = 0, k = 0; i < wcslen(name); i++) {
	//	if (name[i] != '\0') {
	//		pname[k] = name[i];
	//		k++;
	//	}
	//}
	lua_pushstring(l, name);
	return 1;
}

/* Since v0.4.2 changed method to IsPedRangeOfPoint
int l_GetPedRangeOfPoint(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	int ped_pos_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
		lua_pushnumber(l, 255);
		return 1;
	}
	float x1 = (float)lua_tonumber(l, 2);
	float y1 = (float)lua_tonumber(l, 3);
	float z1 = (float)lua_tonumber(l, 4);
	float x2 = (float)posx / 16384;
	float y2 = (float)posy / 16384;
	float z2 = (float)posz / 16384;
	lua_pushnumber(l, (float)sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)));
	return 1;
}
*/

int l_IsPedRangeOfPoint(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int ped_pos_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (posx == 0 && posy == 0 && posz == 0) {
		lua_pushboolean(l, false);
		return 1;
	}
	float x1 = (float)lua_tonumber(l, 2);
	float y1 = (float)lua_tonumber(l, 3);
	float z1 = (float)lua_tonumber(l, 4);
	float x2 = (float)posx / 16384;
	float y2 = (float)posy / 16384;
	float z2 = (float)posz / 16384;
	if (lua_tonumber(l, 5) >= sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2))) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_IsCarRangeOfPoint(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (posx == 0 && posy == 0 && posz == 0) {
		lua_pushboolean(l, false);
		return 1;
	}
	float x1 = (float)lua_tonumber(l, 2);
	float y1 = (float)lua_tonumber(l, 3);
	float z1 = (float)lua_tonumber(l, 4);
	float x2 = (float)posx / 16384;
	float y2 = (float)posy / 16384;
	float z2 = (float)posz / 16384;
	if (lua_tonumber(l, 5) >= sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2))) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_IsPedAnyRangeOfPoint(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int car_struct = 0;
	//writeLog(LOG_FILENAME, "a", true, "l_IsPedAnyRangeOfPoint(): ped_struct %X\n", ped_struct);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	int posx = 0;
	int posy = 0;
	int posz = 0;
	if (car_struct > 0) {
		int car_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
	}
	else {
		int ped_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x168), &ped_sprite_struct, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		int ped_pos_ptr = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
			lua_pushboolean(l, false);
			return 1;
		}
	}
	if (posx == 0 && posy == 0 && posz == 0) {
		lua_pushboolean(l, false);
		return 1;
	}
	float x1 = (float)lua_tonumber(l, 2);
	float y1 = (float)lua_tonumber(l, 3);
	float z1 = (float)lua_tonumber(l, 4);
	float x2 = (float)posx / 16384;
	float y2 = (float)posy / 16384;
	float z2 = (float)posz / 16384;
	if (lua_tonumber(l, 5) >= sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2))) {
		//writeLog(LOG_FILENAME, "a", true, "IsPedAnyRangeOfPoint() dist %.01f >= %.01f\n", lua_tonumber(l, 5), sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)));
		lua_pushboolean(l, true);
		return 1;
	}
	//writeLog(LOG_FILENAME, "a", true, "IsPedAnyRangeOfPoint() not dist %.01f < %.01f\n", lua_tonumber(l, 5), sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)));
	lua_pushboolean(l, false);
	return 1;
}

int l_IsPedInAnyCar(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int car_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	if (car_struct != 0) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_IsPedInCar(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	int car_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	if (car_struct != 0 && car_struct == (int)lua_tonumber(l, 2)) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetPedWanted(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x20A;
	int ped_wanted = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_wanted, 2, NULL)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int level = 0;
	if (ped_wanted >= 600 and ped_wanted < 1600) level = 1;
	else if (ped_wanted >= 1600 and ped_wanted < 3000) level = 2;
	else if (ped_wanted >= 3000 and ped_wanted < 5000) level = 3;
	else if (ped_wanted >= 5000 and ped_wanted < 8000) level = 4;
	else if (ped_wanted >= 8000 and ped_wanted < 12000) level = 5;
	else if (ped_wanted >= 12000) level = 6;
	lua_pushinteger(l, level);
	return 1;
}

int l_SetPedWanted(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x20A;
	int ped_wanted = (int)lua_tointeger(l, 2);
	int level = 0;
	if (ped_wanted == 1) level = 600;
	else if (ped_wanted == 2) level = 1600;
	else if (ped_wanted == 3) level = 3000;
	else if (ped_wanted == 4) level = 5000;
	else if (ped_wanted == 5) level = 8000;
	else if (ped_wanted == 6) level = 12000;
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &level, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetCarID(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0x6C;
	int car_id = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_id, 2, NULL)) {
		lua_pushinteger(l, car_id);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPedInvisible(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x208;
	int ped_invisible = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_invisible, 2, NULL)) {
		lua_pushinteger(l, ped_invisible);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedInvisible(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x208;
	int ped_invisible = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_invisible, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedMaxSpeed(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int ped_speed_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_speed_ptr, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	//writeLog(LOG_FILENAME, "a", true, "struct %X\n", ped_pos_ptr);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x8), &ped_speed_ptr, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int ped_maxspeed = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x3C), &ped_maxspeed, 4, NULL)) {
		lua_pushnumber(l, (float)ped_maxspeed / 256);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedMaxSpeed(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_speed_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_speed_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	//writeLog(LOG_FILENAME, "a", true, "struct %X\n", ped_pos_ptr);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x8), &ped_speed_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_maxspeed = int(floor(lua_tonumber(l, 2) * 256));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x3C), &ped_maxspeed, 4, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPedSpeed(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_speed_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_speed_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	//writeLog(LOG_FILENAME, "a", true, "struct %X\n", ped_pos_ptr);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x8), &ped_speed_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_speed = int(floor(lua_tonumber(l, 2) * 256));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_speed_ptr + 0x38), &ped_speed, 4, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;

	/*
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_speed = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x39), &ped_speed, 4, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
	*/
}

int l_GetPedSpeed(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int ped_speed = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x39), &ped_speed, 4, NULL)) {
		lua_pushnumber(l, (float)ped_speed / 256);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

/*
int l_GetPedSprite(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_sruct = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_sruct, 4, NULL)) {
		lua_pushinteger(l, ped_struct);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}
*/

int l_SetCarAngle(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	// 360 - 1439
	// 180 - x
	int rotation = int(floor(lua_tonumber(l, 2) * 1439 / 360));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x58), &rotation, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetCarAngle(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int rotation = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x58), &rotation, 2, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	// 360 - 1439
	//   x - 200
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

int l_SetCarPos(lua_State *l) {
	int car_struct = (int)lua_tointeger(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetCarPos(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_GetCarDirection(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2);
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x40), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x44), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2);
	}
	lua_pushnumber(l, float(posx / 16384));
	lua_pushnumber(l, float(posy / 16384));
	return (1, 2);
}

int l_SetCarDirection(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x40), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x44), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetCarArmour(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	car_struct += 0x79;
	int car_armour = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_armour, 1, NULL)) {
		lua_pushinteger(l, car_armour);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetCarArmour(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	car_struct += 0x79;
	// 1 bullet proof
	// 2 rocket proof
	// 4 flame proof
	int car_armour = (int)lua_tonumber(l, 2);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)car_struct, &car_armour, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetObjPos(lua_State *l) {
	int obj_struct = (int)lua_tointeger(l, 1);
	if (obj_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x4), &obj_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x14), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x18), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x1C), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetObjPos(lua_State *l) {
	int obj_struct = (int)lua_tointeger(l, 1);
	if (obj_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x4), &obj_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x14), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x18), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x1C), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetObjAngle(lua_State *l) {
	int obj_struct = (int)lua_tonumber(l, 1);
	if (obj_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x4), &obj_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int rotation = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)obj_struct, &rotation, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	// 360 - 1439
	//   x - 200
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

int l_GetObjID(lua_State *l) {
	int obj_struct = (int)lua_tonumber(l, 1);
	if (obj_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int obj_id = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x14), &obj_id, 2, NULL)) {
		lua_pushinteger(l, obj_id);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetObjModel(lua_State *l) {
	int obj_struct = (int)lua_tonumber(l, 1);
	if (obj_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 65535);
		return 1;
	}
	int obj_model = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(obj_struct + 0x18), &obj_model, 2, NULL)) {
		lua_pushinteger(l, obj_model);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_GetLightPos(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x4), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x8), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0xC), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_GetLightIntensity(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int light_intensity = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(light_struct), &light_intensity, 1, NULL)) {
		lua_pushinteger(l, light_intensity);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetLightRange(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int light_range = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x1), &light_range, 1, NULL)) {
		lua_pushnumber(l, light_range * 8 / 255);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 1;
}

int l_GetLightColor(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return 1, 2, 3;
	}
	int light_red = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x12), &light_red, 1, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return 1, 2, 3;
	}
	int light_green = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x11), &light_green, 1, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return 1, 2, 3;
	}
	int light_blue = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x10), &light_blue, 1, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return 1, 2, 3;
	}
	lua_pushnumber(l, light_red);
	lua_pushnumber(l, light_green);
	lua_pushnumber(l, light_blue);
	return 1, 2, 3;
}

int l_GetLightTime(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		return 1, 2, 3;
	}
	int light_timeion = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x14), &light_timeion, 1, NULL)) {
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		return 1, 2, 3;
	}
	int light_timeioff = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x15), &light_timeioff, 1, NULL)) {
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		return 1, 2, 3;
	}
	int light_timeirandom = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x16), &light_timeirandom, 1, NULL)) {
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		lua_pushinteger(l, 0);
		return 1, 2, 3;
	}
	lua_pushinteger(l, light_timeion);
	lua_pushinteger(l, light_timeioff);
	lua_pushinteger(l, light_timeirandom);
	return 1, 2, 3;
}

int l_SetLightColor(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int light_red = (int)lua_tointeger(l, 2);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x12), &light_red, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_green = (int)lua_tointeger(l, 3);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x11), &light_green, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_blue = (int)lua_tointeger(l, 4);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x10), &light_blue, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_SetLightTime(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_timeion = (int)lua_tointeger(l, 2);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x14), &light_timeion, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_timeioff = (int)lua_tointeger(l, 3);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x15), &light_timeioff, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_timeirandom = (int)lua_tointeger(l, 4);
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x16), &light_timeirandom, 1, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_SetLightPos(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x4), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x8), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0xC), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_SetLightIntensity(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_intensity = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(light_struct), &light_intensity, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetLightRange(lua_State *l) {
	int light_struct = (int)lua_tointeger(l, 1);
	if (light_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int light_range = (int)lua_tointeger(l, 2) * 255 / 8;
	if (WriteProcessMemory(head_process_handle, (LPVOID)(light_struct + 0x1), &light_range, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetArrowPos(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x44), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x48), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x4C), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_GetArrowColor(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int arrow_colorid = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x20), &arrow_colorid, 2, NULL)) {
		lua_pushinteger(l, arrow_colorid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetArrowIcon(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int arrow_iconid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x24), &arrow_iconid, 2, NULL)) {
		lua_pushinteger(l, arrow_iconid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetArrowAngle(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int rotation = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x8), &rotation, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	// 360 - 1439
	//   x - 200
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

/*
int l_SetArrowAngle(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int rotation = int(floor(lua_tonumber(l, 2) * 1439 / 360));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x8), &rotation, 4, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}
*/

int l_GetArrowZoom(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int zoom = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0xC), &zoom, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	lua_pushnumber(l, (float)zoom / 8192);
	return 1;
}

int l_SetArrowZoom(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int zoom = int(floor(lua_tonumber(l, 2) * 8192));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0xC), &zoom, 4, NULL)) {
		lua_pushnumber(l, 1);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 0;
}

int l_GetArrowCurrentZoom(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int zoom = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x10), &zoom, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	lua_pushnumber(l, (float)zoom / 8192);
	return 1;
}

int l_SetArrowCurrentZoom(lua_State *l) {
	int arrow_struct = (int)lua_tonumber(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int zoom = int(floor(lua_tonumber(l, 2) * 8192));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x10), &zoom, 4, NULL)) {
		lua_pushnumber(l, 1);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 0;
}

int l_SetArrowPos(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x44), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x48), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x4C), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_SetArrowColor(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int arrow_colorid = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x20), &arrow_colorid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetArrowIcon(lua_State *l) {
	int arrow_struct = (int)lua_tointeger(l, 1);
	if (arrow_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int arrow_iconid = (int)lua_tointeger(l, 2);
	if (arrow_iconid < 0 || arrow_iconid > 114) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (WriteProcessMemory(head_process_handle, (LPVOID)(arrow_struct + 0x24), &arrow_iconid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetSoundPos(lua_State *l) {
	int sound_struct = (int)lua_tointeger(l, 1);
	if (sound_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x4), &sound_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x14), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x18), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x1C), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetSoundPos(lua_State *l) {
	int sound_struct = (int)lua_tointeger(l, 1);
	if (sound_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x4), &sound_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x14), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x18), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x1C), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetSoundID(lua_State *l) {
	int sound_struct = (int)lua_tointeger(l, 1);
	if (sound_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	int sound_id = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x26), &sound_id, 1, NULL)) {
		lua_pushinteger(l, sound_id);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetSoundID(lua_State *l) {
	int sound_struct = (int)lua_tointeger(l, 1);
	if (sound_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int sound_id = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(sound_struct + 0x26), &sound_id, 1, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetCarSpeed(lua_State *l) {
	int car_struct = (int)lua_tonumber(l, 1);
	if (car_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	car_struct += 0x58;
	int car_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)car_struct, &car_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x40), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x44), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	float x = (float)posx / 16384;
	float y = (float)posy / 16384;
	lua_pushnumber(l, (float)sqrt((x)*(x) + (y)*(y)));
	return 1;
}

int l_GetPedCar(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x16C;
	int ped_car_struct = 0;
	//writeLog(LOG_FILENAME, "a", true, "l_GetPedCar(): ped_struct %X\n", ped_struct);
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_car_struct, 4, NULL)) {
		//writeLog(LOG_FILENAME, "a", true, "l_GetPedCar(): struct %X\n", ped_car_struct);
		lua_pushinteger(l, ped_car_struct);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedOccupation(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x240;
	int ped_occupationid = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_occupationid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedOccupation(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x240;
	int ped_occupationid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_occupationid, 2, NULL)) {
		lua_pushinteger(l, ped_occupationid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedThreatReaction(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x28C;
	int ped_threatreaction = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_threatreaction, 2, NULL)) {
		lua_pushinteger(l, ped_threatreaction);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedThreatReaction(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x28C;
	int ped_threatreaction = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_threatreaction, 2, NULL)) {
		lua_pushinteger(l, ped_threatreaction);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedThreatSearch(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x288;
	int ped_threatsearch = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_threatsearch, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedThreatSearch(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x288;
	int ped_threatsearch = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_threatsearch, 2, NULL)) {
		lua_pushinteger(l, ped_threatsearch);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedSkin(lua_State* l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x244;
	int ped_skinid = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_skinid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedSkin(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	ped_struct += 0x244;
	int ped_skinid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_skinid, 2, NULL)) {
		lua_pushinteger(l, ped_skinid);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_SetPedModel(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_sruct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_sruct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_model_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_sruct + 0x7C), &ped_model_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_model = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_model_ptr + 0x26C), &ped_model, 2, NULL)) { //GET_CHAR_GRAPHIC_TYPE?
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedModel(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_sruct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_sruct, 4, NULL)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	int ped_model_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_sruct + 0x7C), &ped_model_ptr, 4, NULL)) {
		lua_pushinteger(l, 65535);
		return 1;
	}
	int ped_model = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)(ped_model_ptr + 0x26C), &ped_model, 2, NULL)) { //GET_CHAR_GRAPHIC_TYPE?
		lua_pushinteger(l, ped_model);
		return 1;
	}
	lua_pushinteger(l, 65535);
	return 1;
}

int l_SetPedID(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x200;
	int ped_id = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_id, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedID(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x200;
	int ped_id = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_id, 2, NULL)) {
		lua_pushinteger(l, ped_id);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPlayerArmour(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	player_struct += 0x6FA;
	int player_armour = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &player_armour, 2, NULL)) {
		lua_pushnumber(l, player_armour);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 1;
}

int l_SetPlayerArmour(lua_State *l) {	
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 0;
	}
	player_struct += 0x6FA;
	int player_armour = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &player_armour, 2, NULL)) {
		lua_pushnumber(l, 1);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 0;
}

int l_GetPlayerGameCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	player_struct += 0x128;
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetPedCamPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x1AC;
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x4), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x8), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetPedCamPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	ped_struct += 0x1AC;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x4), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x8), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetPlayerGameCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	player_struct += 0x128;
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetPedCurrentCarDoorID(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x24C;
	int exit_doorid = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &exit_doorid, 2, NULL)) {
		lua_pushinteger(l, exit_doorid);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_SetPedCurrentCarDoorID(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x24C;
	int exit_doorid = (int)lua_tointeger(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &exit_doorid, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPlayerViewCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	player_struct += 0x1E4;
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

/*
int l_GetPedStructOfID(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid < 0 || playerid > 65535) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_ptr = 0;
	int ped_array_ptr = 0;
	int ped_struct = 0;
	ped_ptr = PLAYER_OFFSET;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_ptr, 4, NULL);
	ped_ptr += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_array_ptr, 4, NULL);
	ped_array_ptr += 0xC4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_array_ptr, &ped_struct, 4, NULL);
	int ped_id = ped_struct;
	int id = 0;
	int index = 1;
	int player_adress = 0;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	for (int i = 1; i <= MAX_PED_ID; i++) {
		player_adress = ped_id;
		//player_adress += (playerid * PED_STRUCT_SIZE) - PED_STRUCT_SIZE; //ped_id;
		player_adress += 0x200;
		ReadProcessMemory(head_process_handle, (LPVOID)player_adress, &id, 1, NULL);
		//writeLog(LOG_FILENAME, "a", true, "index %d id %d", index, id);
		if (id == playerid) {
			player_adress = ped_id;
			//
			//player_adress += 0x216;
			//int hp = (int)lua_tonumber(l, 2);
			//WriteProcessMemory(head_process_handle, (LPVOID)player_adress, &hp, 2, NULL);
			//hp = 0.0001;
			//WriteProcessMemory(proceid, (LPVOID)player_adress, &hp, 2, NULL);
			lua_pushinteger(l, ped_id);
			return 1;
		}
		//if (posx != 0 && posx / 16384 < 255) {
			index++;
		//	writeLog(LOG_FILENAME, "a", true, "%d struct %d id %d pos %.1f %.1f ID %d\n", i, ped_id, id, (float)(posx / 16384), (float)(posy / 16384), index);
		//}
		ped_id += PED_STRUCT_SIZE;
	}
	lua_pushinteger(l, 0);
	return 1;
} 
*/

int l_SetPlayerViewCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	player_struct += 0x1E4;
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetPlayerAuxCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	player_struct += 0x2A0;
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetPlayerAuxCam(lua_State *l) {
	int player_struct = (int)lua_tointeger(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	player_struct += 0x2A0;
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)player_struct, &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x4), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(player_struct + 0x8), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_SetPedAngle(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	// 360 - 1439
	// 180 - x
	int rotation = int(floor(lua_tonumber(l, 2) * 1439 / 360));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x40), &rotation, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedAngle(lua_State* l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int rotation = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x40), &rotation, 2, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	//writeLog(LOG_FILENAME, "a", true, "ped struct %X angle %.01f", (int)lua_tointeger(l, 1), (float)rotation);
	// 360 - 1439
	//   X - 200
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

int GetPedAngleOffset(int ped_struct) {
	if (ped_struct == 0) {
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		return 0;
	}
	return ped_sprite_struct + 0x40;
}

int l_EnablePedSecondaryAngle(lua_State *l) {
	//we override one of the functions that get the angle from the ped struct
	//to use an unused offset still inside the ped struct
	//but the function is also used by all the other peds, so we must limit it only to the selected one
	int ped_struct = (int)lua_tonumber(l, 1);
	int angle_offset = GetPedAngleOffset(ped_struct);
	if(angle_offset == 0) {
		lua_pushboolean(l, false);
		return 1;
	}
	angle_offset -= 0x40;
	byte use_secondary_offset_texture_1[] = {0xE9, 0x10, 0x01, 0x00, 0x00, 0x90, 0x90};
	WriteProcessMemory(head_process_handle, (LPVOID)0x49C5DC, &use_secondary_offset_texture_1, int(sizeof(use_secondary_offset_texture_1) / sizeof(*use_secondary_offset_texture_1)), NULL);

	byte use_secondary_offset_texture_2[] = {0x66, 0x8B, 0x4E, 0x40, 0x8D, 0x7E, 0x40, 0xE9, 0x03, 0xFF, 0xFF, 0xFF};
	WriteProcessMemory(head_process_handle, (LPVOID)0x49C6D4, &use_secondary_offset_texture_2, int(sizeof(use_secondary_offset_texture_2) / sizeof(*use_secondary_offset_texture_2)), NULL);

	byte use_secondary_offset_texture_3[] = {0x81, 0xFE};
	WriteProcessMemory(head_process_handle, (LPVOID)0x49C6F1, &use_secondary_offset_texture_3, int(sizeof(use_secondary_offset_texture_3) / sizeof(*use_secondary_offset_texture_3)), NULL);

	WriteProcessMemory(head_process_handle, (LPVOID)0x49C6F3, &angle_offset, 4, NULL);

	byte use_secondary_offset_texture_4[] = {0x74, 0x38, 0xEB, 0xD9};
	WriteProcessMemory(head_process_handle, (LPVOID)0x49C6F7, &use_secondary_offset_texture_4, int(sizeof(use_secondary_offset_texture_4) / sizeof(*use_secondary_offset_texture_4)), NULL);

	byte use_secondary_offset_texture_5[] = {0x66, 0x8B, 0x4E, 0x42, 0x8D, 0x7E, 0x42, 0xE9, 0xA6, 0xFE, 0xFF, 0xFF};
	WriteProcessMemory(head_process_handle, (LPVOID)0x49C731, &use_secondary_offset_texture_5, int(sizeof(use_secondary_offset_texture_5) / sizeof(*use_secondary_offset_texture_5)), NULL);

	byte use_secondary_offset_shoot_direction_1[] = {0xEB, 0x3E, 0x90, 0x90};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A44, &use_secondary_offset_shoot_direction_1, int(sizeof(use_secondary_offset_shoot_direction_1) / sizeof(*use_secondary_offset_shoot_direction_1)), NULL);

	byte use_secondary_offset_shoot_direction_2[] = {0x81, 0xF9};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A84, &use_secondary_offset_shoot_direction_2, int(sizeof(use_secondary_offset_shoot_direction_2) / sizeof(*use_secondary_offset_shoot_direction_2)), NULL);

	WriteProcessMemory(head_process_handle, (LPVOID)0x433A86, &angle_offset, 4, NULL);

	byte use_secondary_offset_shoot_direction_3[] = {0x74, 0xDE, 0xEB, 0xCC};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A8A, &use_secondary_offset_shoot_direction_3, int(sizeof(use_secondary_offset_shoot_direction_3) / sizeof(*use_secondary_offset_shoot_direction_3)), NULL);

	byte use_secondary_offset_shoot_direction_4[] = {0x66, 0x8B, 0x49, 0x40, 0xEB, 0xE8};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A5A, &use_secondary_offset_shoot_direction_4, int(sizeof(use_secondary_offset_shoot_direction_4) / sizeof(*use_secondary_offset_shoot_direction_4)), NULL);

	byte use_secondary_offset_shoot_direction_5[] = {0x66, 0x8B, 0x49, 0x42, 0xEB, 0xD8};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A6A, &use_secondary_offset_shoot_direction_5, int(sizeof(use_secondary_offset_shoot_direction_5) / sizeof(*use_secondary_offset_shoot_direction_5)), NULL);

	lua_pushboolean(l, true);
	return 1;
}

int l_DisablePedSecondaryAngle(lua_State *l) {
	//we disable the override that we placed before
	int get_ped_angle_function_offset = 0x49C5DC;
	byte use_primary_offset_texture[] = {0x66, 0x8B, 0x4E, 0x40, 0x8D, 0x7E, 0x40};
	WriteProcessMemory(head_process_handle, (LPVOID)(get_ped_angle_function_offset), &(use_primary_offset_texture), int(sizeof(use_primary_offset_texture) / sizeof(*use_primary_offset_texture)), NULL);

	byte use_primary_offset_shoot_direction[] = {0x66, 0x8B, 0x49, 0x40};
	WriteProcessMemory(head_process_handle, (LPVOID)0x433A44, &use_primary_offset_shoot_direction, int(sizeof(use_primary_offset_shoot_direction) / sizeof(*use_primary_offset_shoot_direction)), NULL);
	lua_pushboolean(l, true);
	return 1;
}

int l_SetPedSecondaryAngle(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	// 360 - 1439
	// 180 - x
	int rotation = int(floor(lua_tonumber(l, 2) * 1439 / 360));
	if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x42), &rotation, 2, NULL)) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_GetPedSecondaryAngle(lua_State* l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int rotation = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x42), &rotation, 2, NULL)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	//writeLog(LOG_FILENAME, "a", true, "ped struct %X angle %.01f", (int)lua_tointeger(l, 1), (float)rotation);
	// 360 - 1439
	//   X - 200
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

int l_GetPedAnyAngle(lua_State* l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	int car_struct = 0;
	//writeLog(LOG_FILENAME, "a", true, "l_GetPedCar(): ped_struct %X\n", ped_struct);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	int rotation = 0;
	if (car_struct > 0) {
		int car_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x58), &rotation, 2, NULL)) {
			lua_pushnumber(l, 0);
			return 1;
		}
	}
	else {
		int ped_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x168), &ped_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			return 1;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x40), &rotation, 2, NULL)) {
			lua_pushnumber(l, 0);
			return 1;
		}
	}
	lua_pushnumber(l, (float)360 * rotation / 1439);
	return 1;
}

int l_SetPedAnyAngle(lua_State* l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 0;
	}
	int car_struct = 0;
	//writeLog(LOG_FILENAME, "a", true, "l_GetPedCar(): ped_struct %X\n", ped_struct);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	int rotation = int(floor(lua_tonumber(l, 2) * 1439 / 360));
	if (car_struct > 0) {
		int car_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			return 0;
		}
		if (WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x58), &rotation, 2, NULL)) {
			lua_pushnumber(l, 1);
			return 1;
		}
	}
	else {
		int ped_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x168), &ped_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			return 0;
		}
		if (WriteProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x40), &rotation, 2, NULL)) {
			lua_pushnumber(l, 1);
			return 1;
		}
	}
	lua_pushnumber(l, 0);
	return 0;
}

int l_GetPedPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int ped_pos_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posx = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posy = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int posz = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_GetPedAnyPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		lua_pushnumber(l, 0);
		return (1, 2, 3);
	}
	int car_struct = 0;
	//writeLog(LOG_FILENAME, "a", true, "l_GetPedCar(): ped_struct %X\n", ped_struct);
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	int posx = 0;
	int posy = 0;
	int posz = 0;
	if (car_struct > 0) {
		int car_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
	}
	else {
		int ped_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x168), &ped_sprite_struct, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		int ped_pos_ptr = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			lua_pushnumber(l, 0);
			return (1, 2, 3);
		}
	}
	lua_pushnumber(l, (float)posx / 16384);
	lua_pushnumber(l, (float)posy / 16384);
	lua_pushnumber(l, (float)posz / 16384);
	return (1, 2, 3);
}

int l_SetPedAnyPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int car_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x16C), &car_struct, 4, NULL)) {
		car_struct = 0;
	}
	if (car_struct > 0) {
		int car_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_struct + 0x58), &car_sprite_struct, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posx = int(floor(lua_tonumber(l, 2) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x30), &posx, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posy = int(floor(lua_tonumber(l, 3) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x34), &posy, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posz = int(floor(lua_tonumber(l, 4) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(car_sprite_struct + 0x6C), &posz, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		lua_pushinteger(l, 1);
		return 1;
	}
	else {
		int ped_sprite_struct = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_struct + 0x168), &ped_sprite_struct, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int ped_pos_ptr = 0;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posx = int(floor(lua_tonumber(l, 2) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posy = int(floor(lua_tonumber(l, 3) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		int posz = int(floor(lua_tonumber(l, 4) * 16384));
		if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
			lua_pushinteger(l, 0);
			return 0;
		}
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

int l_SetPedPos(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	ped_struct += 0x168;
	int ped_sprite_struct = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_sprite_struct, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_pos_ptr = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posx = int(floor(lua_tonumber(l, 2) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posy = int(floor(lua_tonumber(l, 3) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int posz = int(floor(lua_tonumber(l, 4) * 16384));
	if (!WriteProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	lua_pushinteger(l, 1);
	return 1;
}

int l_GetPedStruct(lua_State *l) {
	int playerid = (int)lua_tointeger(l, 1);
	if (playerid <= 0 || playerid > 65535 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int ped_ptr = 0;
	int ped_array_ptr = 0;
	int ped_struct = 0;
	ped_ptr = PLAYER_OFFSET;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_ptr, 4, NULL);
	ped_ptr += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_array_ptr, 4, NULL);
	ped_array_ptr += 0xC4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_array_ptr, &ped_struct, 4, NULL);
	int ped_id = ped_struct;
	int id = 0;
	int index = 1;
	int player_adress = 0;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	for (int i = 1; i <= MAX_PED_ID; i++) {
		player_adress = ped_id;
		//player_adress += (playerid * PED_STRUCT_SIZE) - PED_STRUCT_SIZE; //ped_id;
		player_adress += 0x200;
		ReadProcessMemory(head_process_handle, (LPVOID)player_adress, &id, 2, NULL);
		//writeLog(LOG_FILENAME, "a", true, "index %d id %d", index, id);
		if (id == playerid) {
			player_adress = ped_id;
			lua_pushinteger(l, ped_id);
			return 1;
		}
		index++;
		ped_id += PED_STRUCT_SIZE;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPedFromPoint(lua_State *l) {
	int ped_ptr = 0;
	int ped_array_ptr = 0;
	int ped_struct = 0;
	ped_ptr = PLAYER_OFFSET;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_ptr, 4, NULL);
	ped_ptr += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_array_ptr, 4, NULL);
	ped_array_ptr += 0xC4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_array_ptr, &ped_struct, 4, NULL);
	int ped_id = ped_struct;
	int id = 0;
	int index = 0; 
	float x1 = (float)lua_tonumber(l, 1);
	float y1 = (float)lua_tonumber(l, 2);
	float z1 = (float)lua_tonumber(l, 3);
	float range = (float)lua_tonumber(l, 4);
	int ped_sprite_struct = 0;
	int ped_pos_ptr = 0;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	lua_newtable(l);
	for (int i = 1; i <= MAX_PED_ID; i++) {
		ped_ptr = ped_id;
		ReadProcessMemory(head_process_handle, (LPVOID)(ped_ptr + 0x200), &id, 2, NULL);
		if (id > 0 && id <= 65535) {		
			//writeLog(LOG_FILENAME, "a", true, "index %d id %d\n", index, id);
		ped_ptr += 0x168;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_sprite_struct, 4, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		float x2 = (float)posx / 16384;
		float y2 = (float)posy / 16384;
		float z2 = (float)posz / 16384;
		if ((float)sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)) <= range) {
			//writeLog(LOG_FILENAME, "a", true, "index %d struct %X\n", index, ped_id);
			index++;
			lua_pushinteger(l, index);
			lua_pushinteger(l, ped_id);
			lua_rawset(l, -3);			
		}
	    }
		//index++;
		ped_id += PED_STRUCT_SIZE;
	}
	if (index <= 0) {
		lua_pushinteger(l, 1);
		lua_pushinteger(l, 0);
		lua_rawset(l, -3);
		//lua_settable(l, -3);
	}
	return 1;
}

/*
int l_GetCarPool(lua_State *l) {
	int car_struct = 0;
	int id = 0;
	int index = 0;
	int car_offset = 0;
	ReadProcessMemory(head_process_handle, (LPVOID)CAR_STRUCT_POINTER, &car_offset, 4, NULL);
	car_offset += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)car_offset, &car_struct, 4, NULL);
	//car_struct += 0xBC;
	int car_id = car_struct;
	lua_newtable(l);
	for (int i = 1; i <= MAX_CAR_ID; i++) {
		car_offset = car_id;
		ReadProcessMemory(head_process_handle, (LPVOID)(car_offset + 0x6C), &id, 2, NULL);
		if (id > 0 && id <= 65535) {
			//writeLog(LOG_FILENAME, "a", true, "index %d id %d\n", index, id);
			index++;
			lua_pushinteger(l, index);
			lua_pushinteger(l, car_id);
			lua_rawset(l, -3);
		}
		//index++;
		car_id += CAR_STRUCT_SIZE;
	}
	if (index <= 0) {
		lua_pushinteger(l, 1);
		lua_pushinteger(l, 0);
		lua_rawset(l, -3);
		//lua_settable(l, -3);
	}
	return 1;
}
*/

int l_GetCarFromPoint(lua_State *l) {
	int car_struct = 0;
	int id = 0;
	int index = 0;
	float x1 = (float)lua_tonumber(l, 1);
	float y1 = (float)lua_tonumber(l, 2);
	float z1 = (float)lua_tonumber(l, 3);
	float range = (float)lua_tonumber(l, 4);
	int car_pos_ptr = 0;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	int car_offset = 0;
	ReadProcessMemory(head_process_handle, (LPVOID)CAR_STRUCT_POINTER, &car_offset, 4, NULL);
	car_offset += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)car_offset, &car_struct, 4, NULL);
	//car_struct += 0xBC;
	int car_id = car_struct;
	lua_newtable(l);
	for (int i = 1; i <= MAX_CAR_ID; i++) {
		car_offset = car_id;
		ReadProcessMemory(head_process_handle, (LPVOID)(car_offset + 0x6C), &id, 2, NULL);
		if (id > 0 && id <= 65535) {
			//writeLog(LOG_FILENAME, "a", true, "index %d id %d\n", index, id);
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_offset + 0x58), &car_pos_ptr, 4, NULL)) {
				car_id += CAR_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_pos_ptr + 0x30), &posx, 4, NULL)) {
				car_id += CAR_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_pos_ptr + 0x34), &posy, 4, NULL)) {
				car_id += CAR_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(car_pos_ptr + 0x6C), &posz, 4, NULL)) {
				car_id += CAR_STRUCT_SIZE;
				continue;
			}
			float x2 = (float)posx / 16384;
			float y2 = (float)posy / 16384;
			float z2 = (float)posz / 16384;
			if ((float)sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2) + (z1 - z2)*(z1 - z2)) <= range) {
				//writeLog(LOG_FILENAME, "a", true, "index %d struct %X\n", index, ped_id);
				index++;
				lua_pushinteger(l, index);
				lua_pushinteger(l, car_id);
				lua_rawset(l, -3);
			}
		}
		//index++;
		car_id += CAR_STRUCT_SIZE;
	}
	if (index <= 0) {
		lua_pushinteger(l, 1);
		lua_pushinteger(l, 0);
		lua_rawset(l, -3);
		//lua_settable(l, -3);
	}
	return 1;
}

int l_GetPedPool(lua_State *l) {
	int ped_ptr = 0;
	int ped_array_ptr = 0;
	int ped_struct = 0;
	ped_ptr = PLAYER_OFFSET;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_ptr, 4, NULL);
	ped_ptr += 0x4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_array_ptr, 4, NULL);
	ped_array_ptr += 0xC4;
	ReadProcessMemory(head_process_handle, (LPVOID)ped_array_ptr, &ped_struct, 4, NULL);
	int ped_id = ped_struct;
	int id = 0;
	int index = 0;
	int ped_sprite_struct = 0;
	int ped_pos_ptr = 0;
	int posx = 0;
	int posy = 0;
	int posz = 0;
	int health = 0;
	lua_newtable(l);
	for (int i = 1; i <= MAX_PED_ID; i++) {
		ped_ptr = ped_id;
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_ptr + 0x200), &id, 2, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_ptr + 0x216), &health, 2, NULL)) {
			ped_id += PED_STRUCT_SIZE;
			continue;
		}
		//ReadProcessMemory(head_process_handle, (LPVOID)(ped_ptr + 0x216), &health, 2, NULL);
		if (id > 0 && health > 0) {
			//writeLog(LOG_FILENAME, "a", true, "index %d id %d\n", index, id);
			ped_ptr += 0x168;
			if (!ReadProcessMemory(head_process_handle, (LPVOID)ped_ptr, &ped_sprite_struct, 4, NULL)) {
				ped_id += PED_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_sprite_struct + 0x80), &ped_pos_ptr, 4, NULL)) {
				ped_id += PED_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x14), &posx, 4, NULL)) {
				ped_id += PED_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x18), &posy, 4, NULL)) {
				ped_id += PED_STRUCT_SIZE;
				continue;
			}
			if (!ReadProcessMemory(head_process_handle, (LPVOID)(ped_pos_ptr + 0x1C), &posz, 4, NULL)) {
				ped_id += PED_STRUCT_SIZE;
				continue;
			}
			float x = (float)posx / 16384;
			float y = (float)posy / 16384;
			float z = (float)posz / 16384;
			if (x >= 0 && x <= 255 && y >= 0 && y <= 255 && z >= 0 && z <= 255) {
				//writeLog(LOG_FILENAME, "a", true, "index %d struct %X\n", index, ped_id);
				index++;
				lua_pushinteger(l, index);
				lua_pushinteger(l, ped_id);
				lua_rawset(l, -3);
			}
		}
		//index++;
		ped_id += PED_STRUCT_SIZE;
	}
	if (index <= 0) {
		lua_pushinteger(l, 1);
		lua_pushinteger(l, 0);
		lua_rawset(l, -3);
		//lua_settable(l, -3);
	}
	return 1;
}


// Since v0.4.2 changed return integer to boolean
int l_IsGameKeyPress(lua_State *l) {
	int player_struct = (int)lua_tonumber(l, 1);
	if (player_struct == 0 || lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	player_struct += (int)lua_tonumber(l, 2);
	int player_key_state = 0;
	if (!ReadProcessMemory(head_process_handle, (LPVOID)player_struct, &player_key_state, 1, NULL)) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (player_key_state) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetPedPlayer(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	ped_struct += 0x15C;
	int player_struct = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &player_struct, 4, NULL)) {
		lua_pushinteger(l, player_struct);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 1;
}

int l_GetPedHealth(lua_State *l) {
	int ped_struct = (int)lua_tonumber(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x216;
	int ped_health = 0;
	if (ReadProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_health, 2, NULL)) {
		lua_pushnumber(l, (float)ped_health);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 1;
}

int l_SetPedHealth(lua_State *l) {
	int ped_struct = (int)lua_tointeger(l, 1);
	if (ped_struct == 0 || lua_isnil(l, 1)) {
		lua_pushnumber(l, 0);
		return 1;
	}
	ped_struct += 0x216;
	int ped_health = (int)lua_tonumber(l, 2);
	if (WriteProcessMemory(head_process_handle, (LPVOID)ped_struct, &ped_health, 2, NULL)) {
		lua_pushnumber(l, 1);
		return 1;
	}
	lua_pushnumber(l, 0);
	return 0;
}

int l_GetGameHandle(lua_State* l) {
	lua_pushinteger(l, (int)GetModuleHandle((LPCWSTR)0));
	return 1;
}

int l_GetMISIBuild(lua_State* l) {
	lua_pushstring(l, APP_BUILD);
	return 1;
}

// Since v0.4.2 changed return integer to boolean
int l_IsKeyPress(lua_State *l) {
	if (lua_tointeger(l, 1) == 0) {
		lua_pushboolean(l, false);
		return 1;
	}
	if (GetKeyState((int)lua_tointeger(l, 1)) < 0) {
		lua_pushboolean(l, true);
		return 1;
	}
	lua_pushboolean(l, false);
	return 1;
}

// Since v0.4.2 changed return integer to boolean
int l_IsKeyUp(lua_State *l) {
	if (lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	for (int i = 0; i < MAX_SCRIPTS; i++) {
		if (!lua[i].lua_path) continue;
		if (lua[i].luastate == l) {
			if (GetKeyState((int)lua_tointeger(l, 1)) >= 0) {
				if (lua[i].upkeys[(int)lua_tointeger(l, 1)] == 1) {
					lua[i].upkeys[(int)lua_tointeger(l, 1)] = 0;
					lua_pushboolean(l, true);
					return 1;
				}

			}
			else {
				if (lua[i].upkeys[(int)lua_tointeger(l, 1)] == 0) {
					lua[i].upkeys[(int)lua_tointeger(l, 1)] = 1;
					//lua_pushinteger(l, 1);
					//return 1;
				}
			}
		}
	}
	lua_pushboolean(l, false);
	return 1;
}

// Since v0.4.2 changed return integer to boolean
int l_IsKeyDown(lua_State *l) {
	if (lua_isnil(l, 1)) {
		lua_pushboolean(l, false);
		return 1;
	}
	for (int i = 0; i < MAX_SCRIPTS; i++) {
		if (!lua[i].lua_path) continue;
		if (lua[i].luastate == l) {
			if (GetKeyState((int)lua_tointeger(l, 1)) < 0) {
				if (lua[i].keys[(int)lua_tointeger(l, 1)] == 0) {
					lua[i].keys[(int)lua_tointeger(l, 1)] = 1;
					lua_pushboolean(l, true);
					return 1;
				}

			}
			else {
				if (lua[i].keys[(int)lua_tointeger(l, 1)] == 1) {
					lua[i].keys[(int)lua_tointeger(l, 1)] = 0;
					//lua_pushinteger(l, 1);
					//return 1;
				}
			}
		}
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetKeyState(lua_State *l) {
	if (lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 1;
	}
	lua_pushinteger(l, (int)GetKeyState((int)lua_tointeger(l, 1)));
	return 1;
}

int l_GetXInputConnectedDevicesNumber(lua_State* l) {
	int controllers_count = 0;
	DWORD dwResult;
	for(DWORD i = 0;i< XUSER_MAX_COUNT;i++) {
		XINPUT_STATE state;
		ZeroMemory(&state, sizeof(XINPUT_STATE));

		// Simply get the state of the controller from XInput.
		dwResult = XInputGetState(i, &state);

		if (dwResult == ERROR_SUCCESS) {
			// Controller is connected
			controllers_count++;
		}
		else {
			// Controller is not connected
		}
	}
	lua_pushinteger(l, controllers_count);
	return 1;
}

int l_IsXInputKeyPress(lua_State *l) {
	DWORD dwResult;
	WORD buttonStatus = 0;
	BYTE leftTrigger = 0;
	BYTE rightTrigger = 0;
	WORD triggerDeadzone = 64;
	//defining custom fake buttons for the gamepad triggers
	#define XINPUT_GAMEPAD_LEFT_TRIGGER 0
	#define XINPUT_GAMEPAD_RIGHT_TRIGGER 0
	WORD buttons[] = {
		XINPUT_GAMEPAD_DPAD_UP,			//0
		XINPUT_GAMEPAD_DPAD_DOWN,		//1
		XINPUT_GAMEPAD_DPAD_LEFT,		//2
		XINPUT_GAMEPAD_DPAD_RIGHT,		//3
		XINPUT_GAMEPAD_START,			//4
		XINPUT_GAMEPAD_BACK,			//5
		XINPUT_GAMEPAD_LEFT_THUMB,		//6
		XINPUT_GAMEPAD_RIGHT_THUMB,		//7
		XINPUT_GAMEPAD_LEFT_SHOULDER,	//8
		XINPUT_GAMEPAD_RIGHT_SHOULDER,	//9
		XINPUT_GAMEPAD_A,				//10
		XINPUT_GAMEPAD_B,				//11
		XINPUT_GAMEPAD_X,				//12
		XINPUT_GAMEPAD_Y,				//13
		XINPUT_GAMEPAD_LEFT_TRIGGER,	//14
		XINPUT_GAMEPAD_RIGHT_TRIGGER	//15
	};
	int buttonIndex = (int)lua_tointeger(l, 1);
	// checking that the button index is valid
	if(buttonIndex < 0 || buttonIndex > 15) {
		lua_pushboolean(l, false);
		return 1;
	}
	for(DWORD i = 0;i< XUSER_MAX_COUNT;i++) {
		XINPUT_STATE state;
		ZeroMemory(&state, sizeof(XINPUT_STATE));

		// Simply get the state of the controller from XInput.
		dwResult = XInputGetState(i, &state);

		if(dwResult == ERROR_SUCCESS) {
			// Controller is connected
			buttonStatus = buttonStatus | state.Gamepad.wButtons;
			if(state.Gamepad.bLeftTrigger > leftTrigger) {
				leftTrigger = state.Gamepad.bLeftTrigger;
			}
			if(state.Gamepad.bRightTrigger > rightTrigger) {
				rightTrigger = state.Gamepad.bRightTrigger;
			}
		}
		else {
			// Controller is not connected
		}
	}
	//checking that the button is pressed
	if(buttonIndex <= 13) {
		lua_pushboolean(l, (buttons[buttonIndex] & buttonStatus) > 0);
		return 1;
	}
	else {
		//checking the triggers
		if(buttonIndex == 14) {
			//left trigger
			lua_pushboolean(l, leftTrigger > triggerDeadzone);
			return 1;
		}
		if(buttonIndex == 15) {
			//right trigger
			lua_pushboolean(l, rightTrigger > triggerDeadzone);
			return 1;
		}
	}
	lua_pushboolean(l, false);
	return 1;
}

int l_GetXInputAxes(lua_State *l) {
	SHORT sThumbLX = 0, sThumbLY = 0, sThumbRX = 0, sThumbRY = 0;
	SHORT deadzone = 6553;
	DWORD dwResult;
	for(DWORD i = 0; i< XUSER_MAX_COUNT; i++) {
		XINPUT_STATE state;
		ZeroMemory(&state, sizeof(XINPUT_STATE));

		// Simply get the state of the controller from XInput.
		//cycling the controllers in priority order to let the first controller override the others!
		dwResult = XInputGetState(XUSER_MAX_COUNT - 1 - i, &state);

		if(dwResult == ERROR_SUCCESS) {
			// Controller is connected
			//checking for deadzones
			if(abs((int)state.Gamepad.sThumbLX) > deadzone) {
				sThumbLX = state.Gamepad.sThumbLX;
			}
			if(abs((int)state.Gamepad.sThumbLY) > deadzone) {
				sThumbLY = state.Gamepad.sThumbLY;
			}
			if(abs((int)state.Gamepad.sThumbRX) > deadzone) {
				sThumbRX = state.Gamepad.sThumbRX;
			}
			if(abs((int)state.Gamepad.sThumbRY) > deadzone) {
				sThumbRY = state.Gamepad.sThumbRY;
			}
		}
		else {
			// Controller is not connected
		}
	}
	lua_pushnumber(l, (float)sThumbLX / 32767);
	lua_pushnumber(l, (float)sThumbLY / 32767);
	lua_pushnumber(l, (float)sThumbRX / 32767);
	lua_pushnumber(l, (float)sThumbRY / 32767);
	return (1, 2, 3, 4);
}

int l_SendInput(lua_State *l) {
	WORD keyCode = (WORD)lua_tointeger(l, 1);
	INPUT input;
	input.type = INPUT_KEYBOARD;
	input.ki.wScan = MapVirtualKey(keyCode, MAPVK_VK_TO_VSC);
	input.ki.time = 0;
	input.ki.dwExtraInfo = 0;
	input.ki.wVk = keyCode;
	input.ki.dwFlags = 0; // there is no KEYEVENTF_KEYDOWN
	SendInput(1, &input, sizeof(INPUT));

	input.ki.dwFlags = KEYEVENTF_KEYUP;
	SendInput(1, &input, sizeof(INPUT));

	lua_pushboolean(l, true);
	return 1;
}

int l_SendMenuInput(lua_State *l) {
	int keyCode = (int)lua_tointeger(l, 1);
	bool active = (bool)lua_toboolean(l, 2);
	DWORD keyCodesOffsets[] = {
		0x04520B1,	//0 - left
		0x04520D5,	//1 - rigth
		0x045206F,	//2 - up
		0x045208D,	//3 - down
		0x04520F9,	//4 - enter
		0x045211A	//5 - escape
	};
	byte function_to_override[] = {0xB0, 0x80, 0x90, 0x90, 0x90, 0x90};
	function_to_override[1] = active ? 0x80 : 0x0;
	switch (keyCode) {
		case 0:
		case 1:
		case 2:
		case 3:
			WriteProcessMemory(head_process_handle, (LPVOID)keyCodesOffsets[keyCode], &function_to_override, 6, NULL);
			break;
		case 4:
		case 5:
			WriteProcessMemory(head_process_handle, (LPVOID)keyCodesOffsets[keyCode], &function_to_override, 3, NULL);
			break;
		default:
			//return false
			lua_pushboolean(l, false);
			return 1;
	}
	lua_pushboolean(l, true);
	return 1;
}

int l_wait(lua_State *l) {
	if (lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	int time = (int)lua_tointeger(l, 1);
	Sleep(time >= 0 ? time : 0);
	return 1;
}

int l_WriteInLog(lua_State *l) {
	if (lua_isnil(l, 1)) {
		lua_pushinteger(l, 0);
		return 0;
	}
	if (writeLog(LOG_FILENAME, "a", false, lua_tostring(l, 1))) {
		lua_pushinteger(l, 1);
		return 1;
	}
	lua_pushinteger(l, 0);
	return 0;
}

char *GetLuaError(lua_State* l) {
	char *msg = (char *)lua_tostring(l, 1);
	return (char *)msg;
}

char* GetGameVersion(int offset, int size) {
	if (offset == GAME_OFFSET_VER_9_6 && size == GAME_SIZE_VER_9_6) {
		return (char *)GAME_VER_9_6;
	}
	else if (offset == GAME_OFFSET_VER_10_3 && size == GAME_SIZE_VER_10_3) {
		return (char *)GAME_VER_10_3;
	}
	else if (offset == GAME_OFFSET_VER_11_44 && size == GAME_SIZE_VER_11_44) {
		return (char *)GAME_VER_11_44;
	}
	return (char *)"UNKNOWN VERSION";
}